#include "..\RandGen.h"
#include <stdio.h>

typedef unsigned char BYTE;
typedef int BOOL;
#define TRUE 1
#define FALSE 0
#define MAXARGS 20
#define MAXFUNCNAMELEN 1000
#define STRLEN_MAX 2000
#define MAXEXPRESSIONLEN 100
#define MAXIDENTLEN 50
#define MAXNESTINGLEVEL 1000
#define MAXINTDEPTH 20
#define MAXGLOBALPARMS 10000
#define MAXLOCALPARMS 20
#define MAXARRAYS 8191
#define NSYSTEMVARS 13
#define NSTDFUNC 94
#define NSTDFUNC0 70
#define RAND_TIME_START 56
#define RAND_TIME_END 59
#define NBINARYOP 14
#define NSPECARRAYS 0

class String
{
public:
	int size;
	char *data;
	String();
	String(char *);
	~String();
	int GetLength();
	String Left(int);
	void MakeLower();
	void TrimRight();
	void Insert(int, char *);
	int Find(char);
	int Find(char, int j);
	int FindOutsideTheQuotes(char, int j);
	int Find(char *);
	int Find(char *, int j);
	int FindOutsideTheQuotes(char *, int j);
	char *GetBuffer();
	void ReleaseBuffer();
	void operator =(char *); // '=' must always be followed by a pointer to char (a String variable must be typecast to char*)
	void operator +=(char *); // '+=' can be followed either by a pointer to char or by a String variable
	operator char*() const;
};

class Array
{
public:
	int nElem, elSize, size;
	void *data0;
	Array();
	~Array();
	void Add0(void *);
	void SetAtGrow0(int, void *);
	void InsertAt0(int, void *);
	void RemoveAt0(int);
	void RemoveAll0();
	int GetSize();
	void SetSize0(int);
	void Copy0(const Array& src);
};

class Array_int : public Array
{
public:
	int *data;
	Array_int();
	void Add(int);
	void SetAt(int, int);
	void SetAtGrow(int, int);
	void InsertAt(int, int);
	void RemoveAt(int);
	void RemoveAll();
	void SetSize(int);
	void Copy(const Array_int& src);
	int &operator[](int);
};

class Array_long : public Array
{
public:
	long *data;
	Array_long();
	void Add(long);
	void SetAt(int, long);
	void SetAtGrow(int, long);
	void InsertAt(int, long);
	void RemoveAt(int);
	void RemoveAll();
	void SetSize(int);
	void Copy(const Array_long& src);
	long &operator[](int);
};

class Array_BYTE : public Array
{
public:
	BYTE *data;
	Array_BYTE();
	void Add(BYTE);
	void SetAt(int, BYTE);
	void SetAtGrow(int, BYTE);
	void InsertAt(int, BYTE);
	void RemoveAt(int);
	void RemoveAll();
	void SetSize(int);
	void Copy(const Array_BYTE& src);
	BYTE &operator[](int);
};

class Array_double : public Array
{
public:
	double *data;
	Array_double();
	void Add(double);
	void SetAt(int, double);
	void SetAtGrow(int, double);
	void InsertAt(int, double);
	void RemoveAt(int);
	void RemoveAll();
	void SetSize(int);
	void Copy(const Array_double& src);
	double &operator[](int);
};

class Array_longPtr : public Array
{
public:
	long **data;
	Array_longPtr();
	void Add(long*);
	void SetAt(int, long *);
	void SetAtGrow(int, long *);
	void InsertAt(int, long *);
	void RemoveAt(int);
	void RemoveAll();
	void SetSize(int);
	void Copy(const Array_longPtr& src);
	long* &operator[](int);
};

class Array_charPtr : public Array
{
public:
	char **data;
	Array_charPtr();
	void Add(char*);
	void SetAt(int, char *);
	void SetAtGrow(int, char *);
	void InsertAt(int, char *);
	void RemoveAt(int);
	void RemoveAll();
	void SetSize(int);
	void Copy(const Array_charPtr& src);
	char* &operator[](int);
};

class Array_dblPtr2 : public Array
{
public:
	double ***data;
	Array_dblPtr2();
	void Add(double**);
	void SetAt(int, double **);
	void SetAtGrow(int, double **);
	void InsertAt(int, double **);
	void RemoveAt(int);
	void RemoveAll();
	void SetSize(int);
	void Copy(const Array_dblPtr2& src);
	double** &operator[](int);
};

class Array_String : public Array
{
public:
	String *data;
	Array_String();
	~Array_String();
	void Add(char *);
	void SetAt(int, char *);
	void SetAtGrow(int, char *);
	void InsertAt(int, char *);
	void RemoveAt(int);
	void RemoveAll();
	void SetSize(int);
	void Copy(const Array_String& src);
	String &operator[](int);
};

typedef struct {
	char name[MAXIDENTLEN+1];
	double value;
} PARMDATA;

typedef struct {
	bool gParms;
	char name[MAXFUNCNAMELEN+1];
	char message[STRLEN_MAX+1];
	int errorType; // 0 - no error or warning, 1 - syntax warning (i.e., non-fatal syntax error), 2 - fatal syntax error, 3 - fatal runtime error (including user interrupt)
	int errLineNum;
	long nParm;
	PARMDATA parm[MAXLOCALPARMS];
	unsigned long unused0[5];
	long unused1[3];
} EQDATA;

class OPDATA
{
public:
	int nOperands, nInterm;
	BOOL isSubroutine, arrIndUsed, arrayModified, formalParmModified, globalParmModified, localParmModified, globalParmAddrUsed, localParmAddrUsed;
	int valuesUpdated, errorType; // errorType has not the same meaning as in EQDATA
	double waitTime;
	BOOL dllsUsed, globalsUsed, localsUsed, subroutinesUsed, arraysUsed, userArraysUsed, globalObjectsUsed, userGlobalObjectsUsed, randTimeUsed;
	BYTE subroutineArgUsed[MAXARGS];
	short int nXbreaks;
	long i_tNear, callDepth;
	long errLineNum, runtimeErrDll, runtimeErrCode, arrayError, errorFunc, builtInFuncError, runtimeErrLineNum, warnLineNum, curOp, interruptOp;
	short int nControl;
	double *parmValues; // local parameters
	Array_BYTE subroutineUsed, arrayUsed, arrayMod, isConst, isConditional, isAddress, isArrayAssignment;
	Array_long numInterm, funcNum, argCount, lineNum;
	Array_long control, transferTo, newNextControl, newNextXbreak;
	Array_longPtr argNum;
	Array_dblPtr2 argPtr;
	double *operands;
	long curSize, chunkSize, base;
	long long checkSum;
public:
	OPDATA();
	void Reset0();
	void Reset(int subroutineArgCount, int nLocParm);
	void SetOperandAt(int i, double value);
	void operator=(class OPDATA const &src);
};

typedef struct {
	int count;
	BYTE type[MAXARGS];
	String list, list_singleLine;
} FUNCARGINFO;

typedef struct {
	int count;
	BYTE type[MAXARGS];
} STDFUNCARGINFO;

typedef struct {
	double arg;
	int it;
	double t;
	int iEq, iFunc;
	EQDATA *eq;
	OPDATA *op;
	double startTime;
	BOOL calculationInProgress;
	BOOL terminate;

	double retVal;
} PROGRAM_ARGS1;

typedef struct {
	unsigned short num;
	unsigned short type_offset0;
	unsigned long offset;
} LOC;

typedef struct {
	int iFunc;
	EQDATA *eq;
	OPDATA *op, *op0;
	double arg1[MAXARGS];
	LOC *arg2[MAXARGS];
} PROGRAM_ARGS2;

typedef struct {
	signed char objType;
	unsigned short offset0, objNum;
	unsigned long nDim, nElem1;
	unsigned long a, a2;
} PROGRAM_ARGINFO;

typedef struct {
	long c__4;
	double c_b17;
    double a, p, q, u, ae, fa, fb, fc;
    long ic;
    double re, fx, cmb, tol, acmb, acbs;
    long kount;
} ROOT_VARS;

typedef struct {
	int argCount;
	long nextControl;
	double x;
	PROGRAM_ARGINFO *arg_info;
	long base;
	int nFunc1;
	int nFunc2;
	int maxIntDepth, curIntDepth, prevIntDepth;
	int *nextCtrl[MAXINTDEPTH], *iEnd[MAXINTDEPTH];
	int nInt[MAXINTDEPTH], iInt[MAXINTDEPTH];
	int curRootDepth;
	ROOT_VARS RootVars[MAXINTDEPTH];
} PROGRAM_TEMPVARS;

typedef struct {
	double *argPtr;
	PROGRAM_ARGS2 *aPtr2;
	PROGRAM_TEMPVARS *tmpVarPtr;
	long iStart, iEnd;
} EXTERN_FUNC;

typedef struct {
	char name[MAXFUNCNAMELEN+1];
	long nDim;
	unsigned long nElem[MAXARGS], stepSize[MAXARGS];
	long type; // 0 - double, 1 - float, 2 - long, 3 - ULONG, 4 - short, 5 - USHORT, 6 - char (signed), 7 - BYTE (unsigned char)
	void *elements;
	unsigned long size, nElements;
} ARRAYDATA;

class Compiler
{
public:
	char S[STRLEN_MAX], S1[STRLEN_MAX], S2[STRLEN_MAX];
	char systemVarNames[NSYSTEMVARS][200];
	char stdFuncNames[NSTDFUNC][200];
	double (*stdFunc[NSTDFUNC0]) (double **a);
	int (*advancedStdFunc[NSTDFUNC]) (int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i);
	STDFUNCARGINFO stdFuncArgInfo[NSTDFUNC];
	BOOL reverseArgList[NSTDFUNC];
	char defaultNumberFormat0[100], defaultNumberFormat[100];
	Array_BYTE funcType;
	int nGParm, nNewGParm;
	PARMDATA globalParms[MAXGLOBALPARMS], *newGlobalParms;
	double gParmValues[MAXGLOBALPARMS], sysVarValues[NSYSTEMVARS];
	BOOL waiting;
	long nSubroutines;
	long nArrays0, nArrays;
	long nDllFunc;
	int index_base;
	ARRAYDATA *arrays;
	Array_String subroutineCode;
	EQDATA *subroutine;
	OPDATA *subroutine2;
	FUNCARGINFO *subroutineArgInfo;
	BOOL resetSubroutines;
	char **resInst;
	double indeterminateValue;
	PROGRAM_ARGS1 *aPtr1;
	long callDepth;
	ROOT_VARS initRootVars;
	Compiler();
	virtual ~Compiler();
	void DefineAdvancedStdFunc();
	Array_String formatStrings;
	FILE *f_print, *f_write, *f_read;
	int currentProgram, currentLineNum;
	Array_String *outputLines, *curStringArray;
	Array_int *outputLineNumbers, *curNumberArray;
	fpos_t **fpos, *startPos, *endPos;
	bool createOutputStrings, inputLineCreated, newLine, inputLineCopied, checkKeyboard, escapeKeyPressed, feedback, fatalError;
	int typeSize[200];
	int TypeSize(long type);
	int Compile(char *code, int iLine, EQDATA *eqData, OPDATA *opData, int iFunc = -1, BOOL ignoreUnloadableSubroutines = FALSE);
	void LoadString(char **stringTable, int, char *, int);
	void RunProgram1(PROGRAM_ARGS2 *aPtr2 = 0, PROGRAM_TEMPVARS *tmpVarPtr = 0, int i = -1, int iMax = -1);
	void ArrayError(int code, int i, PROGRAM_ARGS2 *aPtr2, long iArray); // OPDATA::errorType = 1
	void BuiltInFuncError(int code, int i, PROGRAM_ARGS2 *aPtr2, long iBuiltInFunc); // OPDATA::errorType = 2
//	void DllError(int code, int i, PROGRAM_ARGS2 *aPtr2, long iDllFunc); // OPDATA::errorType = 3
	void Error(int i, PROGRAM_ARGS2 *aPtr2, int errorType); // OPDATA::errorType > 3
	double Write0(double **a, FILE *f, bool newLine, bool execute, int addToStringArray, bool openFile);
	int Size0(LOC *location);
	int Hist0(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i, int nMax, int keepInit);
	int ReadArray0(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i, double nMax_dbl);
	void PostProcessing(FILE *f_in, int iSubroutine = -1);
	int Arrays(char *code, char *msg);
	void* AllocateArray(long type, long iDim, long nDim, unsigned long *nElem);
	long long ArraySize(long type, long iDim, long nDim, unsigned long *nElem, unsigned long *nElements, double *sizeDbl);
	int FormalParameters(int iFunc);
	unsigned int op_arg_count(const int c, int iFunc = -1);
	bool is_function(char *c, int &id, int iFunc, char **argNames);
	BYTE FuncType(int id);
	BOOL CheckIdentifier(char *id, int mode, int &num, char *prefix, int &nConf, int *&typeConf, int *&numConf, char **&nameConf, char *&nameConfBuffer, char *message = NULL);
	void UpdateErrorMsg(int iFunc, EQDATA *eqData, OPDATA *opData);
	void ArrayRuntimeErrorMsg(long errorFunc, int iFunc, long arrayError, long runtimeErrCode, char *msg);
	void BuiltInFuncRuntimeErrorMsg(long errorFunc, int iFunc, long builtInFuncError, long runtimeErrCode, char *msg);
	void quickSort2(int array_size, int type, void *numbers, int permut[]);
	void q_sort2(int type, void *numbers, int left, int right, int permut[]);
//	void DLLruntimeErrorMsg(long errorFunc, int iFunc, long runtimeErrDll, long runtimeErrCode, char *msg);
	CRandGen randGen;
	char types[6][STRLEN_MAX+1];
};

Compiler *GetCompiler();
void RunProgram0(void *unused);

int Compare_double(const void *arg1, const void *arg2);
int Compare_float(const void *arg1, const void *arg2);
int Compare_int(const void *arg1, const void *arg2);
int Compare_uint(const void *arg1, const void *arg2);
int Compare_int2(const void *arg1, const void *arg2);
int Compare_uint2(const void *arg1, const void *arg2);
int Compare_int1(const void *arg1, const void *arg2);
int Compare_uint1(const void *arg1, const void *arg2);
