/*  The FORTRAN SSP library subroutine MINV translated into C
//
// The original code available at http://pdp-10.trailing-edge.com/decuslib20-02/index.html
C
C     ..................................................................
C
C        SUBROUTINE MINV
C
C        PURPOSE
C           INVERT A MATRIX
C
C        USAGE
C           CALL MINV(A,N,D,L,M)
C
C        DESCRIPTION OF PARAMETERS
C           A - INPUT MATRIX, DESTROYED IN COMPUTATION AND REPLACED BY
C               RESULTANT INVERSE.
C           N - ORDER OF MATRIX A
C           D - RESULTANT DETERMINANT
C           L - WORK VECTOR OF LENGTH N
C           M - WORK VECTOR OF LENGTH N
C
C        REMARKS
C           MATRIX A MUST BE A GENERAL MATRIX
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           THE STANDARD GAUSS-JORDAN METHOD IS USED. THE DETERMINANT
C           IS ALSO CALCULATED. A DETERMINANT OF ZERO INDICATES THAT
C           THE MATRIX IS SINGULAR.
C
C     ..................................................................
*/
#include <math.h>

void DMINV(double *A,long N,double &D,long *L, long *M)
{
double BIGA,HOLD;
long K, NK, KK, J, IZ, I, IJ, KI, JI, JP, KJ, JQ, JR, JK, IK;
//
//        SEARCH FOR LARGEST ELEMENT
//
D=1.;
NK=-N;
for (K=1;K<=N;K++) { // 80
	NK=NK+N;
	L[K-1]=K;
	M[K-1]=K;
	KK=NK+K;
	BIGA=A[KK-1];
	for (J=K;J<=N;J++) { // 20
		IZ=N*(J-1);
		for (I=K;I<=N;I++) { // 20
			IJ=IZ+I;
			if(fabs(BIGA) - fabs(A[IJ-1]) < 0.) goto LABEL_15;
			else goto LABEL_20;
LABEL_15:
			BIGA=A[IJ-1];
			L[K-1]=I;
			M[K-1]=J;
LABEL_20:
			continue;
		}
	}
//
//        INTERCHANGE ROWS
//
	J=L[K-1];
	if(J-K<=0) goto LABEL_35;
	else goto LABEL_25;
LABEL_25:
	KI=K-N;
	for (I=1;I<=N;I++) { // 30
		KI=KI+N;
		HOLD=-A[KI-1];
		JI=KI-K+J;
		A[KI-1]=A[JI-1];
		A[JI-1]=HOLD;
	}
//
//        INTERCHANGE COLUMNS
//
LABEL_35:
	I=M[K-1];
	if(I-K<=0) goto LABEL_45;
	else goto LABEL_38;
LABEL_38:
	JP=N*(I-1);
	for(J=1;J<=N;J++) { // 40
		JK=NK+J;
		JI=JP+J;
		HOLD=-A[JK-1];
		A[JK-1]=A[JI-1];
		A[JI-1]=HOLD;
	}
//
//        DIVIDE COLUMN BY MINUS PIVOT (VALUE OF PIVOT ELEMENT IS
//        CONTAINED IN BIGA)
//
LABEL_45:
	if (BIGA) goto LABEL_48;
	else goto LABEL_46;
LABEL_46:
	D=0.;
	return;
LABEL_48:
	for (I=1;I<=N;I++) { // 55
		if(I-K) goto LABEL_50;
		else goto LABEL_55;
LABEL_50:
		IK=NK+I;
		A[IK-1]=A[IK-1]/(-BIGA);
LABEL_55:
		continue;
	}
//
//        REDUCE MATRIX
//
	for (I=1;I<=N;I++) { // 65
		IK=NK+I;
		HOLD=A[IK-1];
		IJ=I-N;
		for (J=1;J<=N;J++) { // 65
			IJ=IJ+N;
			if(I-K) goto LABEL_60;
			else goto LABEL_65;
LABEL_60:
			if(J-K) goto LABEL_62;
			else goto LABEL_65;
LABEL_62:
			KJ=IJ-I+K;
			A[IJ-1]=HOLD*A[KJ-1]+A[IJ-1];
LABEL_65:
			continue;
		}
	}
//
//        DIVIDE ROW BY PIVOT
//
	KJ=K-N;
	for (J=1;J<=N;J++) { // 75
		KJ=KJ+N;
		if (J-K) goto LABEL_70;
		else goto LABEL_75;
LABEL_70:
		A[KJ-1]=A[KJ-1]/BIGA;
LABEL_75:
		continue;
	}
//
//        PRODUCT OF PIVOTS
//
	D=D*BIGA;
//
//        REPLACE PIVOT BY RECIPROCAL
//
	A[KK-1]=1./BIGA;
}
//
//        FINAL ROW AND COLUMN INTERCHANGE
//
K=N;
LABEL_100:
K=(K-1);
if (K<=0) goto LABEL_150;
else goto LABEL_105;
LABEL_105:
I=L[K-1];
if (I-K<=0) goto LABEL_120;
else goto LABEL_108;
LABEL_108:
JQ=N*(K-1);
JR=N*(I-1);
for (J=1;J<=N;J++) { // 110
	JK=JQ+J;
	HOLD=A[JK-1];
	JI=JR+J;
	A[JK-1]=-A[JI-1];
	A[JI-1]=HOLD;
}
LABEL_120:
J=M[K-1];
if (J-K<=0) goto LABEL_100;
else goto LABEL_125;
LABEL_125:
KI=K-N;
for (I=1;I<=N;I++) { // 130
	KI=KI+N;
	HOLD=A[KI-1];
	JI=KI-K+J;
	A[KI-1]=-A[JI-1];
	A[JI-1]=HOLD;
}
goto LABEL_100;
LABEL_150:
return;
}
