// MCNScriptDlg.h : header file
//

#include "resource.h"		// main symbols
#include "GridCtrl\GridCtrl.h"
#include "GridCtrl\NewCellTypes\GridCellCheck.h"
#include "GridCtrl\NewCellTypes\GridCellCombo.h"
class CColorsGridCtrl : public CGridCtrl
{
private:
	int currentRegionNum;
public:
	afx_msg void OnShowColorDialog();
    afx_msg void OnColorSelected();
    afx_msg void OnKillFocus(CWnd *pNewWnd);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	DECLARE_MESSAGE_MAP()
};

typedef struct {
	int historyNo[2], trackNo[2], nVert[2], gen[2];
	unsigned long originTypeMask;
	double Estart[2], Efinal[2], Eloss[2], relEloss[2];
	double displ[2], peakDispl[2], path[2];
} TRACKFILTERDATA;
#include "TrackFilteringTabCtrl.h"
#include "TrackColorsDlg.h"
#include "SystemVarsDlg.h"

#define MAXMATERIALS 5000
#define MAXSURFACES 10000
#define MAXSURFACES_CELL 5000
#define MAXCELLS 5000
#define MAXTRANSFORMS 10000
#define MAXTHREADS 2048

#define NDEFAULTCOLORS 17
#define NSTDCOLORS 15 // must not exceed 255
#define NREGIONSINIT 13 // must not exceed NDEFAULTCOLORS
#define REGIONNAMELEN 50
#define PARMSETNAMELEN 200
#define SUFFIXLEN 50
#define FNLEN 259
#define NMGEDDISTUNITS 12
#define NEVENTTYPES 12

#define CHUNKSIZE_CELLS 20

#pragma once

typedef struct {
	double x, y, z;
} VECTOR;

typedef struct {
	int type; // 1 - straight line, 2 - two parallel straight lines, 3 - two rays with a common initial point, 4 - circle, 5 - two circles with a common symmetry axis,
			  // 6 - ellipse formed by a non-circular intersection of a plane with a cylinder, 7,8,9 - non-circular intersection of a plane with a one-sheet cone: 7 - ellipse, 8 - one branch of a hyperbola, 9 - parabola,
			  // 10 - non-planar intersection of a sphere and a cylinder (4 branches), 11 - non-planar intersection of two cylinders (4 branches),
			  // 12, 13, 14 - non-planar intersections that are not implemented (i.e., not parameterized): 12 - cone-sphere, 13 - cone-cylinder, 14 - two cones,
			  // 0, -1, -2, -3, -4 - no intersection: 0 - both surfaces coincide, -1 - both surfaces in the positive direction from each other, -2 - surface 2 in the positive direction from 1, and 1 in the negative direction from  2,
			  // -3 - surface 2 in the negative direction from 1, and 1 in the positive direction from 2, -4 - both surfaces in the negative direction from each other
	VECTOR init[2]; // for type 1, 2 or 3 - initial point(s) of the straight line(s), for type 4, 5 or 6 - center(s) of the circle(s) or the ellipse, for type 7, 8 or 9 - the point on the cone's symmetry axis that is closest to the intersection line,
			  // for type 10 - the center of the sphere, for type 11 - the point on the symmetry axis of cylinder 1 that is closest to the symmetry axis of cylinder 2
	VECTOR dir[2]; // for type 1, 2 or 3 - direction vector(s) of the straight line(s), for type 4 or 5 - normal vector of the plane(s) containing the circle(s), for types 6 - 10 - direction vector of the symmetry axis of the cylinder or the cone
	double coef[4]; // for type 4 and 5 - radius (radii) of the circle(s),
					// for type 6 - squared radius of the cylinder and squared tangent of the angle between the plane and the cylinder's axis (y' is equal to +/- of the square root of coef[0] - (z')^2 * coef[1]),
					// for type 7, 8 and 9 - coefficients before z' and (z')^2 under the square root (y' is equal to +/- of the square root of the result). When type = 9, coef[1] = 0.
					// for type 10 - radius of the cylinder squared, difference of the squared radii of the sphere and the cylinder, and distance from the center of the sphere to the axis of the cylinder
					// for type 11 - radius of cylinder 1 squared, radius of cylinder 2 squared, distance between the axes, and cosine of the angle between them
	double X1; // for type 7, 8 and 9 - minimum value of x' (X1 = +/-(minimum distance between the intersection line and the axis of the cone), where "-" corresponds to the case when the point of intersection of the plane with the axis of the cone is inside the cone)
	double Y1; // for type 6 or 7 - maximum value of y'
	double Z1; // for type 6 or 7 - maximum value of z' (minimum value of z' = -Z1 for type 6, and = 0 for type 7)
			   // for type 8 or 9 - distance from the point init[0] to the tip of the cone
	double tan; // tangent of the angle between the plane and the axis of the cylinder or the cone (defined only for types 6 to 9). When type = 7, the tangent of the cone angle can be calculated as (-Y1/X1 - 1) * tan.
	double tMin, tMax; // Minimum and maximum values of the curve parameter
	double Q[3][3]; // rotation matrix used for transform from the local coordinates to the global coordinates (defined only for type >= 4)
} LINE;

typedef struct {
	int ID, mb_index; // If the surface is a part of a macrobody, then mb_index is the macrobody index, and ID is the 1-based sequence number of this surface in the macrobody
				// If the surface is not a part of a macrobody, then mb_index is -1, and ID is the user-assigned ID of the surface.
	int nMb; // number of macrobodies using this surface
	int type; // 1 - plane, 2 - sphere, 3 - cylinder, 4 - cone
	VECTOR init, dir; // dir undefined for a sphere
	double D; // defined only for a plane
	double R; // defined only for a cylinder or a sphere
	double angle, sin, cos, tan; // defined only for a cone (angle is in radians)
} SURFACE;

typedef struct {
	int type;
	int ID;
	unsigned short int nS, *s;
	char *sense;
	short int mbNum0; // 0-based sequence number of the original macrobody, if the current general macrobody is a result of a transformation of another macrobody (standard or general)
	// For identifying the bounding surfaces in messages when the surfaces are specified in the definition of the macrobody as facets of other macrobodies:
	short int *mb; // 1-based macrobody sequence number (includes the sense of the region). 0 for surfaces that are not facets of macrobodies.
	short int *i; // 1-based macrobody facet number (negative for facets defined using the detailed format "<MBnum>.<facetNum>", positive if the macrobody is specified as a whole). 0 for surfaces that are not facets of macrobodies.
} MACROBODY;

typedef struct {
	int ID, mat, mat0;
	double concentration, density;
	unsigned short nS;
	unsigned short s[MAXSURFACES_CELL];
	char side[MAXSURFACES_CELL];
	unsigned short nCellEntryTallies_s[2][MAXSURFACES+1], *cellEntryTallies_s[2][MAXSURFACES+1]; // list of cell-entry tallies for each bounding surface of the cell (the value of the index is the overall 1-based sequence number of the surface, not the number inside the cell; the element no. 0 corresponds to the "inner" surface of the cell, i.e., the combined surfaces of all cells contained in it)
	char usedInCellEntryTallies[2];
	char pointFound; // positive when an interior point has been found, negative when a surface point has been found, zero when no point has been found
	VECTOR point; // an interior point of the cell
	int unionNum, hostCell, depthLevel;
	int nContainedCells, containedCells[MAXCELLS];
	double cutoffEnergy, cutoffEnergy_photon;
	int nPulseHeight, iCellPulseHeight;
	char force;
	short int cellNum0; // 0-based sequence number of the original cell, if the current cell is a result of a transformation of another cell
	// For identifying the bounding surfaces in messages when the surfaces are specified in the definition of the cell as facets of macrobodies:
	short int *mb; // 1-based macrobody sequence number (includes the sense of the region, defined as positive if it contains the macrobody and negative otherwise). 0 for surfaces that are not facets of macrobodies.
	short int *i; // 1-based macrobody facet number (negative for facets defined using the detailed format "<MbNum>.<facetNum>", positive if the macrobody is specified as a whole). 0 for surfaces that are not facets of macrobodies.
} CELL;

typedef struct {
	int ID;
	short int nParts;
	short int *parts;
} UNION;

typedef struct {
	int ID;
	VECTOR origin;
	double Q[3][3];
} TRANSFORM;

typedef struct {
	LINE *curve;
	SURFACE *surface;
	int branchNo;
	double distancePrecision;
	double transform_factor, transform_term;
	bool limited;
	bool searchForMinimum; // when false, searches for the maximum by minimizing the opposite of the absolute value
} ROOTS_PARMS;

typedef struct {
	int nHistories, lastHistory, nTracks, nTracksWithCollisions, nTracksWithTwoOrMoreVertices, nTracksEscape, nTracksTruncated;
	int iHist_min, iHist_max;
	int iTrack_min, iTrack_max;

	int nColl0_min, nColl0_max;
	double nColl0_avg, nColl0_SD;
	int nColl_min, nColl_max;
	double nColl_avg, nColl_SD;
	double startE_min, startE_max, startE_avg, startE_SD;
	double finalE_min, finalE_max, finalE_avg, finalE_SD;
	double Eloss0_min, Eloss0_max, Eloss0_avg, Eloss0_SD;
	double Eloss_min, Eloss_max, Eloss_avg, Eloss_SD;
	double relEloss0_min, relEloss0_max, relEloss0_avg, relEloss0_SD;
	double relEloss_min, relEloss_max, relEloss_avg, relEloss_SD;
	double displ0_min, displ0_max, displ0_avg, displ0_SD;
	double displ_min, displ_max, displ_avg, displ_SD;
	double peakDispl_min, peakDispl_max, peakDispl_avg, peakDispl_SD;
	double path0_min, path0_max, path0_avg, path0_SD;
	double path_min, path_max, path_avg, path_SD;
	double meanFreePath_min, meanFreePath_max, meanFreePath_avg, meanFreePath_SD;
	double meanElossPerCollision_min, meanElossPerCollision_max, meanElossPerCollision_avg, meanElossPerCollision_SD;
	double stoppingPower_min, stoppingPower_max, stoppingPower_avg, stoppingPower_SD;

	double nColl0_sum, nColl0_sum2;
	double nColl_sum, nColl_sum2;
	double startE_sum, startE_sum2;
	double finalE_sum, finalE_sum2;
	double Eloss0_sum, Eloss0_sum2;
	double Eloss_sum, Eloss_sum2;
	double relEloss0_sum, relEloss0_sum2;
	double relEloss_sum, relEloss_sum2;
	double displ0_sum, displ0_sum2;
	double displ_sum, displ_sum2;
	double peakDispl_sum, peakDispl_sum2;
	double path0_sum, path0_sum2;
	double path_sum, path_sum2;
	double meanFreePath_sum, meanFreePath_sum2;
	double meanElossPerCollision_sum, meanElossPerCollision_sum2;
} TRACKSTATS;

typedef struct {
	int iVert[2];
	double E[2], Eloss[2], relEloss[2];
	double path[2];
} TRACKSEGMENTDATA;

typedef struct {
	unsigned char index;
	unsigned char r, g, b;
	DWORD dummy;
} COLORDATA;

typedef struct {
	char name[REGIONNAMELEN+1];
	COLORDATA color;
	double transp, refl;
	BOOL show;
} REGIONDATA;

typedef struct {
	long long sourceParticleNum;
	int nVertices;
	long gen, originType;
	char ot;
	char type; // 0 - absorption, 1 - escape, -1 - truncated
	VECTOR *coord;
	double *energy;
} TRACKDATA;

typedef struct {
	char trackFN[FNLEN+1];
	__int64 fileSize;
	int format;
	FILETIME lastWriteTime;
	int nHistories;
	int *histStartTrackNum;
	int nTracks;
	TRACKDATA *trackData;
} TRACKSETDATA;

typedef struct {
	double maximumObjectSizeRatio, sourceSizeRatio;
	double viewSize;
	VECTOR viewCenter;
	double azimuth, elevation, twist, perspective;
	double systemVars[10];
	VECTOR cutawayPlanePoint, cutawayPlaneNormal;
	TRACKFILTERDATA trackFilter;
	TRACKSEGMENTDATA trackSegmentSelection;
	BOOL trackFilteringBeforeSegmentSelection;
	BOOL useCutawayPlane, raytraceInGraphicsWindow, showWireframe, raytraceInLingeringWindow, showTracks;
	int portNum;
	int nIterations;
	int unitEnergy, unitDistance;
	int nRegions;
	COLORDATA trackColor[NEVENTTYPES], beamAxisColor, backgroundColor;
	BOOL keepTrackDataInMemory, showTrackSegments, extendEscapeTracks;
	int nVerticesTotalMax, nVerticesTrackMax;
	DWORD cmdProcessID;
	UINT_PTR cmdTimerID;
	INT_PTR cmdTimerIndex;
	BOOL cmdWindowCreated;
	DWORD mgedProcessID;
	UINT_PTR mgedTimerID;
	INT_PTR mgedTimerIndex;
	BOOL mgedWindowCreated;
	BOOL useCustomTrackFN, copyToClipboard;
	int createLog, checkSyntax_MCNelectron;
	BOOL quitAfterChecking, MCNP_style_cmdLine, checkSyntax_MCNScript, dummy;
	int cmdFolderOption, nThreads;
	char MCNelectronFolder[FNLEN+1], inputFileFolder[FNLEN+1];
	char outputFileName[FNLEN+1]; // used only when suffix is empty
	char logFileName[FNLEN+1]; // used only when logSuffix is empty
	char altCSinfoFileName[FNLEN+1];
	char ENDFfolder[FNLEN+1];
	char cmdFolder[FNLEN+1]; // used only when cmdFolderOption = 3
	char MCNelectronExeName[FNLEN+1], fn[FNLEN+1];
	char trackFN[FNLEN+1]; // used only when useCustomTrackFN = TRUE
	char scriptFolder[FNLEN+1], MGEDcmdLine[3*FNLEN+1];
	char parmSetName[PARMSETNAMELEN+1];
	char MGEDsuffix[SUFFIXLEN+1], suffix[SUFFIXLEN+1], logSuffix[SUFFIXLEN+1];
	char additionalArgs[2*FNLEN+1];
} VIEWPARM;

class CGridCtrlModified : public CGridCtrl
{
public:
    afx_msg void OnKillFocus(CWnd *pNewWnd);
	DECLARE_MESSAGE_MAP()
};

class CEditDisable : public CEdit
{
private:
	BOOL Ctrl_down;
public:
	BOOL disable;
	CEditDisable();
	afx_msg LRESULT OnPaste(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnCut(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnUndo(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnDelete(WPARAM wParam, LPARAM lParam);
	BOOL PreTranslateMessage(MSG* pMsg);
	DECLARE_MESSAGE_MAP()
};

class CPicture : public CWnd
{
    DECLARE_DYNCREATE(CPicture)
public:
	CPicture::CPicture();
	BOOL RegisterWindowClass();
	BOOL Create(const RECT& rect, CWnd* parent, UINT nID,
                DWORD dwStyle = WS_CHILD | WS_BORDER | WS_TABSTOP | WS_VISIBLE);
	afx_msg void OnPaint();
	DECLARE_MESSAGE_MAP()
};

#define PICTURE_CLASSNAME    _T("MFCPicture")  // Window class name

class CColorRect : public CWnd
{
    DECLARE_DYNCREATE(CColorRect)
public:
	COLORREF color;
	int sel;
	CColorRect::CColorRect();
	BOOL RegisterWindowClass();
	BOOL Create(const RECT& rect, CWnd* parent, UINT nID,
                DWORD dwStyle = WS_CHILD | WS_BORDER | WS_TABSTOP | WS_VISIBLE);
	afx_msg void OnPaint();
	DECLARE_MESSAGE_MAP()
};

#define COLORRECT_CLASSNAME    _T("MFCColorRect")  // Window class name

class CStartButton : public CButton
{
// Construction
public:
	CStartButton(CWnd* pParent = NULL) { };	// standard constructor
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	UINT_PTR *pTimerID;
	DECLARE_MESSAGE_MAP()
};

class CStartButton_MCNelectron : public CStartButton
{
// Construction
public:
	CStartButton_MCNelectron(CWnd* pParent = NULL) { };	// standard constructor
	virtual void OnResetButton();
	DECLARE_MESSAGE_MAP()
};

class CStartButton_MGED : public CStartButton
{
// Construction
public:
	CStartButton_MGED(CWnd* pParent = NULL) { };	// standard constructor
	virtual void OnResetButton();
	DECLARE_MESSAGE_MAP()
};

class CTrackDataFileProgressDlg : public CDialog
{
// Construction
public:
	CTrackDataFileProgressDlg(CWnd* pParent = NULL); // standard constructor
	virtual BOOL OnInitDialog();
	BOOL PreTranslateMessage(MSG* pMsg);
    afx_msg void OnStop();
    void OnWindowPosChanging(WINDOWPOS FAR* lpwndpos);

// Dialog Data
	enum { IDD = IDD_TRACKDATAPROGRESS };
	int nSteps;
	CProgressCtrl m_progress;
	BOOL m_visible;

	TRACKSTATS *pTrackStats;
	char *scriptFN;
	char **trackScriptFN;
	TRACKFILTERDATA *pTrackFilter;
	TRACKSEGMENTDATA *pTrackSegmentSelection;
	BOOL trackFilteringBeforeSegmentSelection;

	DECLARE_MESSAGE_MAP()
};

// CMCNScriptDlg dialog
class CMCNScriptDlg : public CDialogEx
{
    DECLARE_DYNCREATE(CMCNScriptDlg)
// Construction
public:
	CMCNScriptDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	enum { IDD = IDD_MCNSCRIPT_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

// Implementation
protected:
	HICON m_hIcon;
	int initialWidth, initialHeight, clippedHeight;
public:
	int nViewParmSets, curParmSet;
	CColorsGridCtrl m_colorsGrid;
	CGridCtrl *pSystemVarsGrid, *pTrackColorsGrid;
	CComboBox m_beamAxisColor, m_backgroundColor;
	CColorRect m_beamAxisColorPic, m_backgroundColorPic;
	CArray<VIEWPARM,VIEWPARM> viewParmSets;
	CArray<TRACKSETDATA,TRACKSETDATA> trackSets;
	bool systemVarsShown, graphicalParametersShown, processingTrackData, trackColorsShown;
	CButton *pOriginTypeCheckbox;
	CTrackDataFileProgressDlg *progressDlg;
protected:
	REGIONDATA **regionData;
	bool overwritePrompt;
	UINT timerCount;
	CArray<int,int> iParmSetFromCmdTimerIndex, iParmSetFromMGEDtimerIndex;
	CGridCtrlModified m_viewCenterGrid, m_cutawayPlanePointGrid, m_cutawayPlaneNormalGrid, *pTrackFilterGrid;
	CEdit *pGenMinEdit, *pGenMaxEdit;
	int colorsRowHeight, colorsHeaderHeight;
public:
	COLORDATA defaultColor[NDEFAULTCOLORS], stdColor[NSTDCOLORS];
protected:
	REGIONDATA defaultRegionData;
	TRACKSETDATA defaultTrackSetData;
public:
	char stdColorName[NSTDCOLORS+1][20];
	TRACKFILTERDATA defaultTrackFilter;
	TRACKSEGMENTDATA defaultTrackSegmentSelection;
	unsigned long bitMask[NEVENTTYPES];
protected:
	int defaultColorUsed[NDEFAULTCOLORS];
	int nUsedDefaultColors;
	bool fileLoaded;
	CEdit m_inputFileName, m_suffix, m_logSuffix, m_MGEDcommandLine;
public:
	CEditDisable m_cmdLine, m_outputFileName, m_logFileName, m_trackFileName;
	CStatic m_nTracks;
	CStatic m_MGEDcmdLineCapt;
	CButton m_createLog, m_checkSyntax_MCNScript, m_keepTrackDataInMemoryCheck;
	CSystemVarsDlg* pSystemVarsDlg;
	CTrackColorsDlg* pTrackColorsDlg;
	CFilteringTabCtrl m_trackFilteringTabCtrl;
protected:
	CPicture m_coordSys;
	CEditDisable m_outputScriptName;
	CComboBox m_parmSetList;
	CStartButton_MCNelectron m_startMCNelectron;
	CStartButton_MGED m_startMGED;
	CButton m_deleteParmSet, m_useCustomTrackFN, m_copyToClipboard, m_loadTrackFile, m_makeRTscript;
	CEdit m_viewSizeEdit, m_maximumObjectSizeRatioEdit, m_sourceSizeRatioEdit;
	CEdit m_azimuthEdit, m_elevationEdit, m_twistEdit, m_perspectiveEdit;
	CButton m_cutawayPlaneCheck, m_raytraceInGraphicsWindowCheck, m_showWireframeCheck, m_raytraceInLingeringWindowCheck, m_showTracksCheck, m_showTrackSegmentsCheck, m_extendEscapeTracksCheck, m_overwritePromptCheck, m_refresh;
	CEdit m_portNumEdit, m_nIterationsEdit, m_nVerticesTotalMaxEdit, m_nVerticesTrackMaxEdit, m_MGEDsuffixEdit;
	CToolTipCtrl m_toolTip;

	short int nSurfaces, nMacrobodies, nCells, nMaterials, nUnions, nTransforms;
	SURFACE *surf;
	MACROBODY *macrobody;
	double **arb_entries;
	CELL *cell;
	UNION *unionData;
	int *materialID;
	char **materialName;
	TRANSFORM *transform, *cellTransform;

	int complexGeometry;
	int sourcePositionType; // 1 - point, 2 - surface (rotationally symmetric distribution), 3 - uniform distribution inside a cell
	int sourceDirectionType; // 1 - beam, 2 - cone, 3 - isotropic
	double beam_X, beam_Y, beam_Z;
	double beam_A, beam_B, beam_C; // defined only for sourceDirectionType = 1 and 2
	double sourceConeCos; // defined only for sourceDirectionType = 2
	VECTOR sourceCenter; // defined only for sourcePositionType = 1 (excluding sourceDirectionType = 1) and 2
	VECTOR sourceNormal; // defined only for sourcePositionType = 2
	double sourceConstantDensityRadius, sourceRMSwidth; // defined only for sourcePositionType = 2
	int sourceCell;
public:
	double maximumObjectSizeRatio, sourceSizeRatio;
	double viewSize;
	VECTOR viewCenter;
	double azimuth, elevation, twist, perspective;
	double systemVars[10];
	VECTOR cutawayPlanePoint, cutawayPlaneNormal;
	TRACKFILTERDATA trackFilter;
	TRACKSEGMENTDATA trackSegmentSelection;
	BOOL trackFilteringBeforeSegmentSelection;
	BOOL useCutawayPlane, raytraceInGraphicsWindow, showWireframe, raytraceInLingeringWindow, showTracks;
	int portNum;
	int nIterations;
	int unitEnergy, unitDistance;
	int nRegions;
	COLORDATA trackColor[NEVENTTYPES], beamAxisColor, backgroundColor;
	BOOL keepTrackDataInMemory, showTrackSegments, extendEscapeTracks;
	int nVerticesTotalMax, nVerticesTrackMax;
	DWORD cmdProcessID;
	UINT_PTR cmdTimerID;
	INT_PTR cmdTimerIndex;
	BOOL cmdWindowCreated;
	DWORD mgedProcessID;
	UINT_PTR mgedTimerID;
	INT_PTR mgedTimerIndex;
	BOOL mgedWindowCreated;
	BOOL useCustomTrackFN, copyToClipboard;
	int createLog, checkSyntax_MCNelectron;
	BOOL quitAfterChecking, MCNP_style_cmdLine, checkSyntax_MCNScript, dummy;
	int cmdFolderOption, nThreads;
	char MCNelectronFolder[FNLEN+1], inputFileFolder[FNLEN+1];
	char outputFileName[FNLEN+1]; // used only when suffix is empty
	char logFileName[FNLEN+1]; // used only when logSuffix is empty
	char altCSinfoFileName[FNLEN+1];
	char ENDFfolder[FNLEN+1];
	char cmdFolder[FNLEN+1]; // used only when cmdFolderOption = 3
	char MCNelectronExeName[FNLEN+1], fn[FNLEN+1];
	char trackFN[FNLEN+1]; // used only when useCustomTrackFN = TRUE
	char scriptFolder[FNLEN+1], MGEDcmdLine[3*FNLEN+1];
	char parmSetName[PARMSETNAMELEN+1];
	char MGEDsuffix[SUFFIXLEN+1], suffix[SUFFIXLEN+1], logSuffix[SUFFIXLEN+1];
	char additionalArgs[2*FNLEN+1];

protected:
	CArray<REGIONDATA,REGIONDATA> regions;
	char cmdLine[6*FNLEN];

	int *usedMaterialNum, nUsedMaterials;
	int *cellsUsingTheMaterial[MAXCELLS], nCellsUsingTheMaterial[MAXMATERIALS];
	int nMaterialsWithSource;
	double unitConversionFactor;
	double cutawayPlaneCoefs[4];
public:
	TRACKSTATS trackStats;
	bool stopReadingTrackData;
protected:

	HACCEL m_hAccel, m_hAccel_trackStats;
	char exeName[1000];
	bool parFileSpecified;
	FILE *f_par, *f_in, *f_out;

public:
	char exePath[1000];
	bool changed;
	int prevDefaultColor, prevSel;
	COLORREF prevColor;
	char fn1[1000], outScriptFN[1000];

	char MGEDdistUnitName[NMGEDDISTUNITS][11];
	double MGEDdistUnit[NMGEDDISTUNITS];
	char energyUnits[3][10], distanceUnits[4][10];
	double energyFactor[3], distanceFactor[4];
	COLORREF GetRegionColor(int i);
	COLORREF GetTrackColor(int i);
	COLORREF GetBeamAxisColor();
	COLORREF GetBackgroundColor();
	void SetRegionColor(int i, COLORREF color);
	void SetTrackColor(int i, COLORREF color);
	void UpdateRegionColor(int i);
	void UpdateTrackColor(int i);
	void UpdateCellColor(int i, int j);
	void UpdateCellColor_tracks(int i, int j);
	void UpdateCellRGB(int i);
	void UpdateCellRGB_tracks(int i);
	void UpdateBeamAxisColor();
	void UpdateBackgroundColor();
	void ColorName(COLORDATA *color, bool includeBlack = false);
	void UpdateTrackFilter(CGridCtrlModified *pGrid, CButton *pCheck, CEdit *pGenMin, CEdit *pGenMax, TRACKFILTERDATA *pTrackFilter);
	void UpdateShow(int i);
	void UpdateShow_tracks(BOOL check, int i);
protected:
	void UpdateColorsView();
	void UpdateViewParms();
	virtual BOOL OnInitDialog();
public:
	VECTOR ProjectionToPlane(VECTOR point, SURFACE *plane);
	double DistanceToSurface(VECTOR *a, SURFACE *s);
	double DistanceToSurface(double t, void *parms);
	int LinePositionRelativeToSurface(LINE *line, int branchNo, SURFACE *surf, double distancePrecision);
	void LineParameterLimits(LINE *line, double distancePrecision);
	VECTOR SurfaceNormal(SURFACE *s, VECTOR point, double distancePrecision);
	VECTOR LineTangentDir(LINE *line, int branch, double t);
	int NonIntersectingSurfaces(SURFACE *s1, SURFACE *s2, double distancePrecision);
	int Intersection(SURFACE *s1, SURFACE *s2, LINE &p, double distancePrecision);
	bool SurfacesCoincide(SURFACE *s1, SURFACE *s2, double distancePrecision);
	void RotationMatrix(double Q[3][3], VECTOR a);
	void RotationMatrix(double Q[3][3], VECTOR a, VECTOR b);
	VECTOR LocalToGlobal(VECTOR init, double *Q, VECTOR point);
	VECTOR LocalToGlobal(LINE *p, int n, VECTOR point);
	VECTOR LocalToGlobal(LINE *p, int n, double t, double distancePrecision);
	VECTOR Rotate(LINE *p, VECTOR point);
	VECTOR Rotate2(LINE *p, VECTOR point);
	void MatrixProduct(double Q[3][3], double Q0[3][3]);
	double FitFunc_ConeCylinder(double t, double phi);
	int LineSurfaceIntersection(LINE *curve, SURFACE *surface, int *branch, double distancePrecision, double *param, VECTOR *intersectionPoints, int &side);
	int CheckCellGeometry0(unsigned short int *s, short int j, short int k, char *side, double distancePrecision, bool &notImplemented);
	int CheckCellGeometry(short int nS, unsigned short int *s, char *side, VECTOR &point, char &pointFound, double &distancePrecision);
	int CheckCellGeometry2(CELL *c, double &distancePrecision, bool &surfacesRemoved, bool *removeSurface);
	int AdjustPlanePosition(SURFACE *p, CELL *c, double height, double &distancePrecision, int &iSurf_max, int nIterations, LINE **intersectionLines, bool invertNormDir, int invertSenseSnum);
	VECTOR CheckPairOfSurfaces(int n, int &i, int &j, int exclude1, int exclude2, VECTOR point, int nIntersectingSurfaces, int *intersectingSurfaces, unsigned short *s, char *side, VECTOR *normal, double distancePrecision, int &ier);
	VECTOR TestDirection(int nTestSurf, VECTOR *point, VECTOR *tangent, unsigned short *s, char *side, int *iTestSurf, int k_twoRays, double distancePrecision, LINE *intersectionLine);
	int LineCellIntersection(LINE *line, short int nS, SURFACE **s, char *sense, VECTOR &intersectionPoint, int &intersectionSurface, double distancePrecision, int excludeSurface1 = -1, int excludeSurface2 = -1);
	int LineCellIntersection(LINE *line, CELL *c, VECTOR &intersectionPoint, int &intersectionSurface, double distancePrecision, int excludeSurface1 = -1, int excludeSurface2 = -1);
	double Roots_func(double, void *);
	int Roots(double *t, void *parms, double step0, double tMin, double tMax, bool periodic, double t_precision, double f_precision, double f_max);
	double Iterate(double &t, void *parms, double &u, double t_precision, double t_max, double f_precision, double f_threshold, bool improvedPrecision, double f_max, int &ier);
	double Iterate2(double &t, double &phi, double t_step0, double phi_step0, double t_precision, double phi_precision, double t_max, double phi_max, double f_precision, double f_threshold);
	int CheckSurfaceID0(char *line, char *&ptr, char &sense, short int &cellNum, short int &surfaceNum, short int &facetNum, bool allowZeroID = false);
	int CheckSurfaceID(char *line, char *&ptr, bool isCell, int ID, unsigned short int &nS, unsigned short int *&s, char *&sense, short int *&mb, short int *&i_mb, int chunkSize);
	void SurfaceIDstring(SURFACE *s, char sense, short int mb, short int mb_i, char string[100]);
	bool ConvertRPPtoBOX(SURFACE *p, double *&arb_entries);

	void GetFileName(CEdit *pEdit, char *title, char *filter, char *buffer);
protected:
	afx_msg void OnLoadInputFile();
	afx_msg void OnReloadInputFile();
	afx_msg void OnLoadTrackFile();
	void LoadInputFile();
	char *NextToken(char *line, char *strToken, char *strDelimit);
public:
	void LoadTrackFile(CWnd *parent, TRACKSTATS *pTrackStats, char *scriptFN = NULL, char **trackScriptFN = NULL, TRACKFILTERDATA *pTrackFilter = NULL, TRACKSEGMENTDATA *pTrackSegmentSelection = NULL, BOOL trackFilteringBeforeSegmentSelection = TRUE);
	void LoadTrackFile2();
    void FreeAll();
    BOOL OnOpen();
    BOOL OnSave();
    BOOL OnSaveAs();
protected:
	afx_msg void OnInputFileName();
	afx_msg void OnSuffix();
	afx_msg void OnOutputFileName();
	afx_msg void OnCreateLog();
	afx_msg void OnLogSuffix();
	afx_msg void OnLogFileName();
	afx_msg void OnStartMCNelectron();
	afx_msg void OnUseCustomTrackFN();
	afx_msg void OnCopyToClipboard();
	afx_msg void OnTrackFileName();
    afx_msg void OnMakeScript();
    afx_msg void CopyToClipboard();
    afx_msg void OnOpenInputFile();
    afx_msg void OnOpenOutputFile();
    afx_msg void OnOpenLogFile();
    afx_msg void OnOpenTrackFile();
    afx_msg void OnOpenRTscript();
    void UpdateRegionNames();
	afx_msg void OnCheckSyntax_MCNScript();
    afx_msg void OnSelectFolder();
	afx_msg void OnMGEDCommandLine();
	afx_msg void OnCheckSyntax();
	afx_msg void OnSystemVars();
    afx_msg void OnParmSetSel();
	void ColorSetSel(CComboBox *list, CColorRect *pic);
    afx_msg void OnNewParmSet();
    afx_msg void OnDeleteParmSet();
public:
	void KillCmdTimer();
	void KillMGEDtimer();
protected:
    afx_msg void OnEditName();
    afx_msg void OnStartMGED();
    afx_msg void OnBrowseMGED();
    void DefaultParmSet(bool keepNames = false);
    void DefaultTrackColors(COLORDATA *color);
public:
	bool OnCommandLine(CEdit *pEdit, char *cmdLine_old, char *cmdLine_new, bool isFolder = false);
	int CheckInteger(char *inputString, char *outputString, int minValue, int maxValue);
	double CheckDouble(char *inputString, char *outputString, double minValue, double maxValue);
protected:
	void SetupVectorGrid(CGridCtrlModified *pGrid);
	void CutawayPlaneCoefs();
	void UpdateCutawayGroup();
	void NewParameterSet();
    afx_msg void OnGridClick(NMHDR *pNotifyStruct, LRESULT* pResult);
    afx_msg void OnColorsGridStartEdit(NMHDR *pNotifyStruct, LRESULT* pResult);
    afx_msg void OnColorsGridEndEdit(NMHDR *pNotifyStruct, LRESULT* pResult);
public:
	bool TrackFilterGridEndEdit(TRACKFILTERDATA *pTrackFilter, CGridCtrlModified *pGrid, int iRow, int iColumn, char *tmp, LRESULT *pResult);
protected:
	afx_msg void OnMaximumObjectSizeRatioEdit();
	afx_msg void OnSourceSizeRatioEdit();
	afx_msg void OnViewSizeEdit();
    afx_msg void OnViewCenterGridEndEdit(NMHDR *pNotifyStruct, LRESULT* pResult);
	afx_msg void OnAzimuthEdit();
	afx_msg void OnElevationEdit();
	afx_msg void OnTwistEdit();
	afx_msg void OnAzimuthSpin(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnElevationSpin(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnTwistSpin(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnPerspectiveEdit();
	afx_msg void OnUseCutaway();
    afx_msg void OnCutawayPlanePointGridEndEdit(NMHDR *pNotifyStruct, LRESULT* pResult);
    afx_msg void OnCutawayPlaneNormalGridEndEdit(NMHDR *pNotifyStruct, LRESULT* pResult);
	afx_msg void OnBeamAxisColorSetSel();
	afx_msg void OnBackgroundColorSetSel();
    afx_msg void OnShowTrackColors();
	afx_msg void OnPortNumEdit();
	afx_msg void OnNiterationsEdit();
	afx_msg void OnNverticesTotalMaxEdit();
	afx_msg void OnNverticesTrackMaxEdit();
	afx_msg void OnMGEDsuffixEdit();
	afx_msg void OnRaytraceInGraphicsWindow();
	afx_msg void OnShowWireframe();
	afx_msg void OnRaytraceInLingeringWindow();
	afx_msg void OnShowTracks();
public:
	afx_msg void OnKeepTrackDataInMemory();
	void MakeCmdLine();
protected:
	afx_msg void OnShowTrackSegments();
	afx_msg void OnExtendEscapeTracks();
	afx_msg void OnOverwritePrompt();
	afx_msg void OnTrackStats();
	afx_msg void OnRefresh();
	afx_msg void OnDefaultFiltering();
    afx_msg void OnReset();
	afx_msg void OnAutosizeGrid();
	afx_msg void OnHideShow();
	void UpdateFileNames();
	afx_msg void OnEditCmdLine();
	afx_msg void OnOK() {};
	afx_msg void OnCancel();
	afx_msg void OnPaint();
	afx_msg void OnTimer(UINT_PTR nIDEvent);
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnInitMenu(CMenu *pMenu);
	BOOL PreTranslateMessage(MSG *pMsg);
	BOOL DestroyWindow();
	afx_msg void OnHelpMCNelectronUserManual();
    afx_msg void OnReload();
	afx_msg void OnConsoleActivate();
	afx_msg void OnConsoleClose();
	DECLARE_MESSAGE_MAP()
};

class CNewParmSetDlg : public CDialogEx
{
// Construction
public:
	CNewParmSetDlg(CWnd* pParent = NULL); // standard constructor
	virtual BOOL OnInitDialog();
	afx_msg void OnOK();


// Dialog Data
	enum { IDD = IDD_NEWPARMSET };
	CMCNScriptDlg *parent;
	CEdit m_title;
	CComboBox m_insertAfter;
};

class CEditNameDlg : public CDialogEx
{
// Construction
public:
	CEditNameDlg(CWnd* pParent = NULL); // standard constructor
	virtual BOOL OnInitDialog();
	afx_msg void OnOK();

// Dialog Data
	enum { IDD = IDD_EDITNAME };
	CMCNScriptDlg *parent;
	CEdit m_title;
};

class CTrackStatsDlg : public CDialogEx
{
    DECLARE_DYNCREATE(CTrackStatsDlg)
// Construction
public:
	CTrackStatsDlg(CWnd* pParent = NULL); // standard constructor
	HACCEL m_hAccel;
	virtual BOOL OnInitDialog();
// Dialog Data
	enum { IDD = IDD_TRACKSTATS };
	CMCNScriptDlg *parent;
	bool dialogInitialized;
	CButton m_trackFilteringBeforeSegmentSelection, m_segmentSelectionBeforeTrackFiltering, m_keepTrackDataInMemoryCheck;
	CComboBox m_unitEnergy, m_unitDistance;
	CStatic m_nHistories, m_nTracks, m_nTracksEscape, m_nTracksTruncated;
	CGridCtrlModified m_trackStatsGrid, m_trackSegmentGrid, *pTrackFilterGrid;
	CButton *pOriginTypeCheckbox;
	CEdit *pGenMinEdit, *pGenMaxEdit;
	CFilteringTabCtrl m_trackFilteringTabCtrl;
	void Clear_nTracks();
protected:
	CToolTipCtrl m_toolTip;
	int nRows, nCols;
	TRACKSTATS trackStats;
	TRACKFILTERDATA trackFilter;
	TRACKSEGMENTDATA trackSegmentSelection;
	BOOL trackFilteringBeforeSegmentSelection;
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	void UpdateTrackStats();
	void UpdateTrackSegmentGrid();
	void UpdateUnits();
    afx_msg void OnTrackSegmentGridEndEdit(NMHDR *pNotifyStruct, LRESULT* pResult);
	afx_msg void OnRefresh();
	afx_msg void OnDefaultFiltering();
	afx_msg void OnKeepTrackDataInMemory();
	afx_msg void OnTrackFilteringBeforeSegmentSelection();
	afx_msg void OnSegmentSelectionBeforeTrackFiltering();
	afx_msg void OnEnergyUnitSelected();
	afx_msg void OnDistanceUnitSelected();
	afx_msg void OnAutosizeGrid();
	virtual void OnOK();
	BOOL PreTranslateMessage(MSG *pMsg);
	DECLARE_MESSAGE_MAP()
};

class CCmdLineDlg : public CDialogEx
{
// Construction
public:
	CCmdLineDlg(CWnd* pParent = NULL); // standard constructor
	virtual BOOL OnInitDialog();
// Dialog Data
	enum { IDD = IDD_CMDLINE };
	CButton m_sameAsInput, m_sameAsMCNelectron, m_sameAsMCNScript, m_checkSyntax_MCNelectron, m_quitAfterChecking, m_MCNP_style;
	CButton m_createLog, m_custom, m_selectFolder, m_browseOutput, m_browseLog, m_browseENDF;
	CEdit m_exeName, m_nThreads, m_inputFileName, m_suffix, m_logSuffix, m_altCSinfoFileName, m_additionalArgs, m_ENDFfolder;
	CEditDisable m_cmdFolder, m_outputFileName, m_logFileName;
	CToolTipCtrl m_toolTip;

	CMCNScriptDlg *parent;
	bool changed;
	char fn1[1000];
	int nThreads, cmdFolderOption;
	int createLog, checkSyntax_MCNelectron;
	BOOL quitAfterChecking, MCNP_style_cmdLine;
	char cmdFolder[FNLEN+1]; // used only when cmdFolderOption = 3
	char MCNelectronFolder[FNLEN+1], MCNelectronExeName[FNLEN+1];
	char inputFileFolder[FNLEN+1], fn[FNLEN+1];
	char suffix[SUFFIXLEN+1];
	char outputFileName[FNLEN+1]; // used only when suffix is empty
	char logSuffix[SUFFIXLEN+1];
	char logFileName[FNLEN+1]; // used only when logSuffix is empty
	char altCSinfoFileName[FNLEN+1];
	char ENDFfolder[FNLEN+1];
	char additionalArgs[2*FNLEN+1];

	afx_msg void OnBrowseExe();
	afx_msg void OnExeName();
	afx_msg void OnNthreads();
	afx_msg void OnNthreadsSpin(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnBrowseInput();
	afx_msg void OnInputFileName();
	afx_msg void OnSystemVars();
	afx_msg void OnSuffix();
	afx_msg void OnBrowseOutput();
	afx_msg void OnOutputFileName();
	afx_msg void OnLogSuffix();
	afx_msg void OnBrowseLog();
	afx_msg void OnLogFileName();
	afx_msg void OnBrowseAlt();
	afx_msg void OnAltCSinfoFileName();
    afx_msg void OnSameAsInput();
    afx_msg void OnSameAsMCNelectron();
    afx_msg void OnSameAsMCNScript();
    afx_msg void OnCustom();
    afx_msg void OnSelectFolder();
    afx_msg void OnOpenInputFile();
    afx_msg void OnOpenOutputFile();
    afx_msg void OnOpenLogFile();
    afx_msg void OnOpenAltCSinfoFile();
    afx_msg void OnBrowseENDF();
	afx_msg void OnENDFfolder();
	void UpdateCmdFolderOptionGroup();
	virtual void OnOK();
	BOOL PreTranslateMessage(MSG *pMsg);
	DECLARE_MESSAGE_MAP()
};
