#include "stdafx.h"
#include "MCNScriptDlg.h"
#include "MCNScript.h"
#include "TrackColorsDlg.h"
#include "TabFilteringByParticleType.h"

IMPLEMENT_DYNAMIC(CTrackColorsDlg, CDialog)

CTrackColorsDlg::CTrackColorsDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTrackColorsDlg::IDD, pParent)
{
	parent = pParent;
	cancelClick = FALSE;
}

CTrackColorsDlg::~CTrackColorsDlg()
{
}

void CTrackColorsDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COLORS, m_trackColorsGrid);
}

BEGIN_MESSAGE_MAP(CTrackColorsGridCtrl, CGridCtrl)
	ON_CBN_SELCHANGE(IDC_INPLACE_CONTROL, OnColorSelected)
	ON_COMMAND(ID_SHOWCOLORDIALOG, OnShowColorDialog)
	ON_WM_KILLFOCUS()
	ON_WM_LBUTTONDBLCLK()
END_MESSAGE_MAP()

BEGIN_MESSAGE_MAP(CTrackColorsDlg, CDialog)
	ON_NOTIFY(NM_CLICK, IDC_COLORS, OnTrackColorsGridClick)
	ON_NOTIFY(GVN_BEGINLABELEDIT, IDC_COLORS, OnTrackColorsGridStartEdit)
	ON_NOTIFY(GVN_ENDLABELEDIT, IDC_COLORS, OnTrackColorsGridEndEdit)
	ON_EN_KILLFOCUS(IDC_GENMIN, OnGenMinEdit)
	ON_EN_KILLFOCUS(IDC_GENMAX, OnGenMaxEdit)
END_MESSAGE_MAP()

BOOL CTrackColorsDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	VERIFY(m_genMin.SubclassDlgItem(IDC_GENMIN, this));
	VERIFY(m_genMax.SubclassDlgItem(IDC_GENMAX, this));
	UpdateData();
	CRect rect;
	m_trackColorsGrid.GetClientRect(&rect);
	colorsRowHeight = GetSystemMetrics(SM_CYVSCROLL) + 2;
	int colorsRowsVisible = rect.bottom / colorsRowHeight - 1;
	if (colorsRowsVisible > NEVENTTYPES) colorsRowsVisible = NEVENTTYPES;
	colorsHeaderHeight = rect.bottom - colorsRowsVisible * colorsRowHeight;
	int additionalHeight = (colorsHeaderHeight - colorsRowHeight) / colorsRowsVisible;
	colorsRowHeight += additionalHeight;
	colorsHeaderHeight -= colorsRowsVisible * additionalHeight;
	m_trackColorsGrid.SetColumnResize(FALSE);
	m_trackColorsGrid.nAutoCols = 1;
	m_trackColorsGrid.autoCol.SetSize(m_trackColorsGrid.nAutoCols);
	m_trackColorsGrid.autoCol[0] = 1;
	m_trackColorsGrid.SetFont(GetFont());
	m_trackColorsGrid.m_disableCut = TRUE;
	m_trackColorsGrid.SetRowCount(1);
	m_trackColorsGrid.SetColumnCount(8);
	m_trackColorsGrid.isColReadOnly.SetSize(8);
	int i;
	for (i = 0; i < 4; i++) m_trackColorsGrid.isColReadOnly[i] = TRUE;
	for (i = 4; i < 8; i++) m_trackColorsGrid.isColReadOnly[i] = FALSE;
	m_trackColorsGrid.SetFixedRowCount(1);
	m_trackColorsGrid.SetFixedColumnCount(1);
	m_trackColorsGrid.SetRowHeight(0, colorsHeaderHeight);
	m_trackColorsGrid.SetItemText(0, 0, "No.");
	m_trackColorsGrid.SetItemText(0, 1, "Particle type");
	m_trackColorsGrid.SetItemText(0, 2, "Include");
	m_trackColorsGrid.SetItemText(0, 3, "Color");
	m_trackColorsGrid.SetItemText(0, 4, "Color name");
	m_trackColorsGrid.SetItemText(0, 5, "R");
	m_trackColorsGrid.SetItemText(0, 6, "G");
	m_trackColorsGrid.SetItemText(0, 7, "B");
	char tmp[1000];
    m_trackColorsGrid.SetSelectedRange(-1,-1,-1,-1);
	m_trackColorsGrid.SetFocusCell(-1,-1);
	m_trackColorsGrid.SetRowCount(NEVENTTYPES + 1);
	char *trackTypeName[NEVENTTYPES] = {"Source particles", "Bremsstrahlung photons", "Positron annihilation photons",
		"X-ray photons emitted after electron- or positron-impact ionization", "X-ray photons emitted after photoelectric and Compton collisions",
		"Electrons from pair production", "Positrons from pair production", "Compton recoil electrons",
		"Photoelectrons", "Auger electrons emitted after photoelectric and Compton collisions",
		"Auger electrons emitted after electron- or positron-impact ionization", "Knock-on electrons"};
	for (i = 0; i < NEVENTTYPES; i++) {
		int i1 = i + 1;
		m_trackColorsGrid.SetRowHeight(i1, colorsRowHeight);
		m_trackColorsGrid.SetCellType(i1, 2, RUNTIME_CLASS(CGridCellCheck));
		m_trackColorsGrid.SetCellType(i1, 4, RUNTIME_CLASS(CGridCellCombo));
	    m_trackColorsGrid.SetItemState(i1, 1, m_trackColorsGrid.GetItemState(i1, 1) | GVIS_READONLY);
	    m_trackColorsGrid.SetItemState(i1, 3, m_trackColorsGrid.GetItemState(i1, 3) | GVIS_READONLY);
	    m_trackColorsGrid.SetItemState(i1, 4, m_trackColorsGrid.GetItemState(i1, 4) | GVIS_NOPASTE);
		sprintf(tmp, " %d ", i);
		m_trackColorsGrid.SetItemText(i1, 0, tmp);
		m_trackColorsGrid.SetItemText(i1, 1, trackTypeName[i]);
	}
	m_trackColorsGrid.cancelClick = &cancelClick;
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CTrackColorsDlg::OnTrackColorsGridClick(NMHDR *pNotifyStruct, LRESULT* /*pResult*/)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
	CMCNScriptDlg *p = (CMCNScriptDlg *)parent;
	if (pItem->iColumn != 2 || pItem->iRow < 1) return;
	if (p->processingTrackData) {
		MessageBox("The track filtering parameters cannot be modified while the track data are being processed.", NULL, MB_OK | MB_ICONINFORMATION);
		return;
	}
	CGridCellCheck *pCheck = (CGridCellCheck *)m_trackColorsGrid.GetCell(pItem->iRow, 2);
	pCheck->m_bChecked = !pCheck->m_bChecked;
	int i = pItem->iRow - 1;
	if (pCheck->m_bChecked) {
		pTrackFilter->originTypeMask |= p->bitMask[i];
		pOriginTypeCheckbox[i].SetCheck(1);
	}
	else if (pTrackFilter->originTypeMask & p->bitMask[i]) {
		pTrackFilter->originTypeMask ^= p->bitMask[i];
		pOriginTypeCheckbox[i].SetCheck(0);
	}
	p->UpdateShow_tracks(pCheck->m_bChecked, i);
	m_trackColorsGrid.InvalidateRect(pCheck->m_Rect);
	m_trackColorsGrid.Invalidate();
	m_trackColorsGrid.UpdateWindow();
	CWnd *parent = pOriginTypeCheckbox[i].GetParent()->GetParent()->GetParent();
	if (parent->IsKindOf(RUNTIME_CLASS(CMCNScriptDlg))) {
		p->changed = true;
		p->m_nTracks.SetWindowText("");
	}
	else {
		CTrackStatsDlg *p2 = (CTrackStatsDlg *)parent;
		p2->Clear_nTracks();
	}
}

void CTrackColorsDlg::OnTrackColorsGridStartEdit(NMHDR *pNotifyStruct, LRESULT* pResult)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
	*pResult = 0;
	if (pItem->iColumn < 4 || pItem->iColumn > 7) return;
	int j;
	int i = pItem->iRow - 1;
	CMCNScriptDlg *p = (CMCNScriptDlg *)parent;
	for (j = 0; j < NSTDCOLORS && (p->trackColor[i].r != p->stdColor[j].r || p->trackColor[i].g != p->stdColor[j].g || p->trackColor[i].b != p->stdColor[j].b); j++);
	prevSel = j;
	if (j == NSTDCOLORS) prevColor = RGB(p->trackColor[i].r, p->trackColor[i].g, p->trackColor[i].b);
	for (j = 0; j < NDEFAULTCOLORS && (p->trackColor[i].r != p->defaultColor[j].r || p->trackColor[i].g != p->defaultColor[j].g || p->trackColor[i].b != p->defaultColor[j].b); j++);
	prevDefaultColor = j;
}

void CTrackColorsDlg::OnTrackColorsGridEndEdit(NMHDR *pNotifyStruct, LRESULT *pResult)
{
	NM_GRIDVIEW* pItem = (NM_GRIDVIEW*) pNotifyStruct;
	CMCNScriptDlg *p = (CMCNScriptDlg *)parent;
	if (pItem->iColumn == 4) {
		CGridCellCombo* pCell = (CGridCellCombo*)m_trackColorsGrid.GetCell(pItem->iRow, pItem->iColumn);
		int j = pCell->GetCurSel();
		if (j != prevSel || j == NSTDCOLORS && RGB(p->trackColor[pItem->iRow-1].r, p->trackColor[pItem->iRow-1].g, p->trackColor[pItem->iRow-1].b) != prevColor) {
			p->changed = true;
		}
		*pResult = 0;
		return;
	}
	if (pItem->iColumn == 2) {
		*pResult = -1;
		return;
	}
    CGridCellBase* pCell = m_trackColorsGrid.GetCell(pItem->iRow, pItem->iColumn);
	CEdit *pEdit = (CEdit *)pCell->GetEditWnd();
	CString text = m_trackColorsGrid.GetItemText(pItem->iRow, pItem->iColumn);
	char tmp[1000];
	strcpy(tmp, text.GetBuffer());
	int i = pItem->iRow - 1;
	char outputString[100];
	if (pItem->iColumn >= 5 && pItem->iColumn <= 7) {
		int num = p->CheckInteger(tmp, outputString, 0, 255);
		if (outputString[0] == '\0') {
			*pResult = -1;
			return;
		}
		int prev = *(&(p->trackColor[i].r) + (pItem->iColumn - 5));
		*(&(p->trackColor[i].r) + (pItem->iColumn - 5)) = num;
		if (num != prev) p->changed = true;
		m_trackColorsGrid.SetItemText(pItem->iRow, pItem->iColumn, outputString);
		p->UpdateTrackColor(i);
		m_trackColorsGrid.Invalidate();
		m_trackColorsGrid.UpdateWindow();
	}
	*pResult = 0;
}

void CTrackColorsGridCtrl::OnColorSelected()
{
	CCellID cellID =  GetFocusCell();
	CGridCellCombo *pCell = (CGridCellCombo *)GetCell(cellID.row, cellID.col);
	int i = cellID.row - 1;
	int j = pCell->GetCurSel();
	CMCNScriptDlg *p = (CMCNScriptDlg *)(GetParent()->GetParent());
	if (j != p->prevSel) p->changed = true;
	if (j == NSTDCOLORS) {
		currentTrackType = i;
		PostMessage(WM_COMMAND, ID_SHOWCOLORDIALOG);
		return;
	}
	p->UpdateCellColor_tracks(i, j);
	p->UpdateCellRGB_tracks(i);
	Invalidate();
	UpdateWindow();
}

void CTrackColorsGridCtrl::OnShowColorDialog()
{
	CMCNScriptDlg *p = (CMCNScriptDlg *)(GetParent()->GetParent());
	int i = currentTrackType;
	COLORREF color = p->GetTrackColor(i);
	CColorDialog colorDlg(color, CC_FULLOPEN);
	if (colorDlg.DoModal() == IDOK) {
		color = colorDlg.GetColor();
		if (color != p->prevColor) {
			p->changed = true;
			p->SetTrackColor(i, color);
			p->UpdateTrackColor(i);
			p->UpdateCellRGB_tracks(i);
			Invalidate();
			UpdateWindow();
		}
	}
}

void CTrackColorsGridCtrl::OnKillFocus(CWnd *pNewWnd)
{
	CCellID cellID = GetFocusCell();
	CGridCtrl::OnKillFocus(pNewWnd);
	CGridCellCombo* pCell = (CGridCellCombo*)GetCell(cellID.row, 4);
	if (pCell && pCell->IsEditing()) return;
    SetSelectedRange(-1,-1,-1,-1);
	SetFocusCell(-1,-1);
}

void CTrackColorsGridCtrl::OnLButtonDblClk(UINT nFlags, CPoint point)
{
	CGridCtrl::OnLButtonDown(nFlags, point);
}

void CTrackColorsDlg::OnGenMinEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_genMin.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	CMCNScriptDlg *p = (CMCNScriptDlg *)parent;
	CTabFilteringByParticleType *tab = (CTabFilteringByParticleType *)pOriginTypeCheckbox->GetParent();
	CWnd *tabParent = tab->GetParent()->GetParent();
	int num = p->CheckInteger(tmp, outputString, 1, 2147483647);
	if (outputString[0] != '\0' && num != pTrackFilter->gen[0]) {
		if (!p->processingTrackData) {
			pTrackFilter->gen[0] = num;
			if (tabParent->IsKindOf(RUNTIME_CLASS(CMCNScriptDlg))) {
				p->changed = true;
				p->m_nTracks.SetWindowText("");
			}
			else {
				CTrackStatsDlg *p2 = (CTrackStatsDlg *)tabParent;
				p2->Clear_nTracks();
			}
		}
		else {
			MessageBox("The track filtering parameters cannot be modified while the track data are being processed.", NULL, MB_OK | MB_ICONINFORMATION);
			cancelClick = TRUE;
		}
	}
	sprintf(tmp, "%d", pTrackFilter->gen[0]);
	m_genMin.SetWindowText(tmp);
	tab->m_genMin.SetWindowText(tmp);
}

void CTrackColorsDlg::OnGenMaxEdit()
{
	char tmp[100], outputString[100];
	int nBytes = m_genMax.GetLine(0, tmp, 99);
	tmp[nBytes] = '\0';
	int num;
	bool empty = (strspn(tmp, " \t") == strlen(tmp));
	CMCNScriptDlg *p = (CMCNScriptDlg *)parent;
	CTabFilteringByParticleType *tab = (CTabFilteringByParticleType *)pOriginTypeCheckbox->GetParent();
	CWnd *tabParent = tab->GetParent()->GetParent();
	if (empty) num = -1;
	else num = p->CheckInteger(tmp, outputString, -1, 2147483647);
	if ((empty || outputString[0] != '\0') && num != pTrackFilter->gen[1]) {
		if (!p->processingTrackData) {
			pTrackFilter->gen[1] = num;
			if (tabParent->IsKindOf(RUNTIME_CLASS(CMCNScriptDlg))) {
				p->changed = true;
				p->m_nTracks.SetWindowText("");
			}
			else {
				CTrackStatsDlg *p2 = (CTrackStatsDlg *)tabParent;
				p2->Clear_nTracks();
			}
		}
		else {
			MessageBox("The track filtering parameters cannot be modified while the track data are being processed.", NULL, MB_OK | MB_ICONINFORMATION);
			cancelClick = TRUE;
		}
	}
	if (pTrackFilter->gen[1] >= 0) {
		sprintf(tmp, "%d", pTrackFilter->gen[1]);
		m_genMax.SetWindowText(tmp);
		tab->m_genMax.SetWindowText(tmp);
	}
	else {
		m_genMax.SetWindowText("");
		tab->m_genMax.SetWindowText("");
	}
}
