#include "stdafx.h"

// The code in this file (excluding the functions GetParentProcessID and GetChildProcessID) is based on the code found on the web page http://stackoverflow.com/questions/1888863/how-to-get-main-window-handle-from-process-id
// Function GetParentProcessID was found at http://www.codeexperts.com/showthread.php?1380-get-parent-process-id-from-child-process-id

#include "WindowHandleFromProcessID.h"

BOOL is_main_window(HWND handle)
{   
	return GetWindow(handle, GW_OWNER) == (HWND)0 && IsWindowVisible(handle);
}

BOOL CALLBACK enum_windows_callback(HWND handle, LPARAM lParam)
{
	if (!is_main_window(handle)) return TRUE;
	handle_data& data = *(handle_data*)lParam;
	unsigned long process_id = 0;
	GetWindowThreadProcessId(handle, &process_id);
	if (process_id != data.process_id) return TRUE;
	data.best_handle = handle;
	return FALSE;   
}

BOOL CALLBACK enum_windows_callback2(HWND handle, LPARAM lParam)
{
// enum_windows_callback2 calls GetParentProcessID for each window handle. This is wasteful, because GetParentProcessID cycles through all active processes.
// enum_windows_callback3 is much more "lightweight", because it cycles only through a small fixed set of processes (they are found by GetChildProcessID before calling EnumWindows).
// However, this set of processes does not include any higher-generation child processes and has no more than NCHILDPROC_MAX of first-generation child processes.
	if (!is_main_window(handle)) return TRUE;
	handle_data2& data = *(handle_data2*)lParam;
	unsigned long process_id = 0;
	GetWindowThreadProcessId(handle, &process_id);
	while (process_id != -1 && process_id != data.process_id) process_id = GetParentProcessID(process_id);
	if (process_id == data.process_id) {
		data.handle[data.nHandles] = handle;
		data.nHandles++;
		if (data.nHandles == NHANDLES_MAX) return FALSE;
	}
	return TRUE;   
}

DWORD childProcID[NCHILDPROC_MAX];
BOOL CALLBACK enum_windows_callback3(HWND handle, LPARAM lParam)
{
// enum_windows_callback2 calls GetParentProcessID for each window handle. This is wasteful, because GetParentProcessID cycles through all active processes.
// enum_windows_callback3 is much more "lightweight", because it cycles only through a small fixed set of processes (they are found by GetChildProcessID before calling EnumWindows).
// However, this set of processes does not include any higher-generation child processes and has no more than NCHILDPROC_MAX of first-generation child processes.
	if (!is_main_window(handle)) return TRUE;
	handle_data3& data = *(handle_data3*)lParam;
	unsigned long process_id = 0;
	GetWindowThreadProcessId(handle, &process_id);
	bool match = (process_id == data.process_id);
	if (!match) {
		int i;
		for (i = 0; i < data.nChildProc && data.childProcID[i] != process_id; i++);
		if (i < data.nChildProc) match = true;
	}
	if (match) {
		data.handle[data.nHandles] = handle;
		data.nHandles++;
		if (data.nHandles == NHANDLES_MAX) return FALSE;
	}
	return TRUE;   
}

HWND find_main_window(unsigned long process_id)
{
	handle_data data;
	data.process_id = process_id;
	data.best_handle = 0;
	EnumWindows(enum_windows_callback, (LPARAM)&data);
	return data.best_handle;
}

int find_main_window2(unsigned long process_id, HWND* handle)
{
	handle_data2 data;
	data.process_id = process_id;
	data.handle = handle;
	data.nHandles = 0;
	EnumWindows(enum_windows_callback2, (LPARAM)&data);
	return data.nHandles;
}

int find_main_window3(unsigned long process_id, HWND* handle)
{
	handle_data3 data;
	data.process_id = process_id;
	data.handle = handle;
	data.nHandles = 0;
	data.childProcID = childProcID;
	data.nChildProc = GetChildProcessID(process_id, childProcID);
	EnumWindows(enum_windows_callback3, (LPARAM)&data);
	if (data.nHandles || !data.nChildProc) return data.nHandles;
	else return -data.nChildProc;
}

#include <TlHelp32.h>
DWORD GetParentProcessID(DWORD dwProcessID)
{
	DWORD dwParentProcessID = -1;
	HANDLE hProcessSnapshot;
	PROCESSENTRY32 processEntry32;
	
	hProcessSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
	if(hProcessSnapshot != INVALID_HANDLE_VALUE) {
		processEntry32.dwSize = sizeof(PROCESSENTRY32);
		if(Process32First(hProcessSnapshot, &processEntry32)) {
			do {
				if (dwProcessID == processEntry32.th32ProcessID) {
					dwParentProcessID = processEntry32.th32ParentProcessID;
					break;
				}
			}
			while(Process32Next(hProcessSnapshot, &processEntry32));
			
			CloseHandle(hProcessSnapshot);
		}
	}

	return dwParentProcessID;
}

int GetChildProcessID(DWORD dwProcessID, DWORD *dwChildID)
{
	HANDLE hProcessSnapshot;
	PROCESSENTRY32 processEntry32;
	int nChildProc = 0;
	
	hProcessSnapshot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
	if(hProcessSnapshot != INVALID_HANDLE_VALUE) {
		processEntry32.dwSize = sizeof(PROCESSENTRY32);
		if(Process32First(hProcessSnapshot, &processEntry32)) {
			do {
				if (processEntry32.th32ParentProcessID == dwProcessID) {
					dwChildID[nChildProc] = processEntry32.th32ProcessID;
					nChildProc++;
					if (nChildProc == NCHILDPROC_MAX) break;
				}
			}
			while(Process32Next(hProcessSnapshot, &processEntry32));
			
			CloseHandle(hProcessSnapshot);
		}
	}

	return nChildProc;
}
