/****************************************************************************

Copyright (C) 2017 Andrius Poskus (e-mail:  andrius.poskus@ff.vu.lt  or  andrius.poskus@live.com)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

The code in this file is based on the source code of GraphiXT (another free software by me). The main changes in comparison
with GraphiXT v1.21 are the following:
1)	removal of the features related to the graphical user interface of GraphiXT;
2)	replacement of the MFC classes CString and CArray by similar non-MFC classes String and Array_<typeName> (because
	GraphiXT uses the MFC library, whereas MCNelectron does not);
3)	support of the following new syntax elements of the user code (i.e., the code embedded in the MCNelectron input file):
	(a) the "for" loop,
	(b) the option to make array indices zero-based (as in C),
	(c) built-in functions for writing formatted text to a file and reading array data from a file,
	(d) several additional mathematical built-in functions,
	(e) the possibility to separate statements by semicolons (as in C) and to use multiple statements on a single line.

****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <float.h>
#include <process.h>
//#include <Windows.h> // needed only for "Sleep(0)" (this line must be removed for MFC applications)
#include "StringTable.h"
#include "Compiler.h"

#ifdef _MSC_VER
#ifndef _CRT_SECURE_NO_DEPRECATE
#define _CRT_SECURE_NO_DEPRECATE (1)
#endif
#pragma warning(disable : 4996)
#endif

Compiler *app;

Compiler *GetCompiler()
{
	return app;
}

int FltPntException(double x)
{
	if (x == x) {
		if (x <= DBL_MAX && x >= -DBL_MAX) return 0;
		else return 1;
	}
	else return 2;
}

double Add(double **a) {
	return **a + *a[1];
}
double Subtract(double **a) {
	return **a - *a[1];
}
double Divide(double **a) {
	return **a / *a[1];
}
double Multiply(double **a) {
	return **a * *a[1];
}
double Power(double **a) {
	return pow(**a, *a[1]);
}
double Equality(double **a) {
	return *a[1];
}
double GT(double **a) {
	if (**a > *a[1]) return 1.;
	else return 0.;
}
double LT(double **a) {
	if (**a < *a[1]) return 1.;
	else return 0.;
}
double GE(double **a) {
	if (**a >= *a[1]) return 1.;
	else return 0.;
}
double LE(double **a) {
	if (**a <= *a[1]) return 1.;
	else return 0.;
}
double EQ(double **a) {
	if (**a == *a[1]) return 1.;
	else return 0.;
}
double NE(double **a) {
	if (**a != *a[1]) return 1.;
	else return 0.;
}
double AND(double **a) {
	if (**a && *a[1]) return 1.;
	else return 0.;
}
double OR(double **a) {
	if (**a || *a[1]) return 1.;
	else return 0.;
}
BOOL CheckFormat(char *format, char *numberFormat, char *prefix, char *suffix)
{
	char part[1001], *prefixFmt, *suffixFmt;
	numberFormat[0] = '%';
	size_t i, j;
	size_t len = strlen(format);
	BOOL percentFound = FALSE;
	BOOL numberFound = FALSE;
	BOOL singlePercentFound = FALSE;
	for (i = 0; i < len; i++) {
		char c = format[i];
		if (c == '%') {
			if (!percentFound) percentFound = TRUE;
			else percentFound = FALSE;
		}
		else if (percentFound) {
			if (numberFound) break;
			singlePercentFound = TRUE;
			char *endPtr;
			double a = strtod(format + i, &endPtr);
			size_t len1 = endPtr - format - i;
			if (len1) strncpy(part, format + i, len1);
			part[len1] = '\0';
			for (j = 0; j < len1 && part[j] != 'e' && part[j] != 'E'; j++);
			if (j < len1) {
				strncpy(numberFormat+1, part, j + 1);
				numberFormat[j+2] = '\0';
				numberFound = TRUE;
				format[i-1] = '\0';
				prefixFmt = format;
			}
			else {
				c = format[i+len1];
				if (c == 'g' || c == 'G' || c == 'e' || c == 'E' || c == 'f') {
					sprintf(numberFormat+1, "%s%c", part, c);
					numberFound = TRUE;
					format[i-1] = '\0';
					prefixFmt = format;
				}
				else break;
			}
			suffixFmt = format + i + j + 1;
			i += len1;
			percentFound = FALSE;
		}
	}
	if (singlePercentFound && numberFound && i == len && !percentFound) {
		sprintf(prefix, prefixFmt);
		sprintf(suffix, suffixFmt);
		return TRUE;
	}
	else return FALSE;
}
double Compiler::Write0(double **a, FILE *f, bool lineEnd, bool execute, int addToStringArray, bool openFile) {
	char tmp[1001];
	int j = (int)(*a[0] + 0.5);
	size_t nValues = (size_t)(*a[1] + 0.5);
	double **v = a + 2;
	size_t i, k = 0;
	String outputLine, fmt;
	for (i = 0; i <= nValues; i++) {
		fmt = (char *)(formatStrings[j]);
		k += fmt.size;
		if (k > 1000) {
			fmt = (char *)(fmt.Left((int)(fmt.size - (k - 1000))));
			k = 1000;
		}
		outputLine += fmt; j++;
		if (i == nValues) break;
		if (k > 999) break;
		sprintf(tmp, defaultNumberFormat, *v[i]);
//		sprintf(tmp, formatStrings[j], *v[i]);
		k += strlen(tmp);
		if (k > 1000) break;
		outputLine += tmp;// j++;
	}
	if (lineEnd) outputLine += "\n";
	if (f) fputs(outputLine.data, f);
	double rc;
	if (execute) {
		if (f_write) fflush(f_write);
		checkKeyboard = false;
		rc = system(outputLine);
		checkKeyboard = true;
	}
	else if (openFile) {
		if (strspn(outputLine, " \t") < strlen(outputLine)) {
			FILE *f = fopen(outputLine, "wt");
			if (f) {
				if (f_write) fclose(f_write);
				f_write = f;
				rc = 1.;
			}
			else if (f_write) rc = -1.;
			else rc = 0.;
		}
		else if (f_write) rc = -1.;
		else rc = 0.;
	}
	else if (!f && !addToStringArray) {
		if (strspn(outputLine, " \t") < strlen(outputLine)) {
			FILE *f = fopen(outputLine, "r+t");
			if (f) {
				if (f_read) fclose(f_read);
				f_read = f;
				rc = 1.;
			}
			else if (f_read) rc = -1.;
			else rc = 0.;
		}
		else if (f_read) rc = -1.;
		else rc = 0.;
	}
	else rc = *a[1];
	if (addToStringArray > 0) {
		int nStrings = curStringArray->nElem;
		if (newLine || !nStrings) {
			curStringArray->Add(outputLine);
			curNumberArray->Add(aPtr1->op->lineNum[aPtr1->op->curOp]); // line number of the top-level program
		}
		else curStringArray->data[nStrings-1] += outputLine;
		newLine = lineEnd;
		if (lineEnd) {
			inputLineCreated = true; // Notify the main thread that the string is available
			while(!inputLineCopied) {// Wait until the main thread copies the string
				Sleep(0); // needed to avoid deadlock between the main thread and the compiler thread
			}
			inputLineCopied = false;
		}
	}
	else if (addToStringArray < 0) {
		char formatStr_bak[1001], numberFormat[100], prefix[1001], suffix[1001];
		strncpy(formatStr_bak, outputLine, 1000);
		formatStr_bak[1000] = '\0';
		if (strspn(outputLine, " \t") == strlen(outputLine)) strcpy(defaultNumberFormat, defaultNumberFormat0);
		else if (strlen(outputLine) < 100 && CheckFormat(outputLine, numberFormat, prefix, suffix)) strcpy(defaultNumberFormat, formatStr_bak);
	}
	return rc;
}
double Dir(double **a) {
	return app->Write0(a, NULL, true, false, app->createOutputStrings, false);
}
double Dir2(double **a) {
	return app->Write0(a, NULL, false, false, app->createOutputStrings, false);
}
double Write(double **a) {
	return app->Write0(a, app->f_write, true, false, app->createOutputStrings && !app->f_write, false);
}
double Write2(double **a) {
	return app->Write0(a, app->f_write, false, false, app->createOutputStrings && !app->f_write, false);
}
double Print(double **a) {
	return app->Write0(a, app->f_print, true, false, 0, false);
}
double Print2(double **a) {
	return app->Write0(a, app->f_print, false, false, 0, false);
}
double System(double **a) {
	return app->Write0(a, NULL, false, true, 0, false);
}
double FopenW(double **a) {
	return app->Write0(a, NULL, false, false, 0, true);
}
double FopenR(double **a) {
	return app->Write0(a, NULL, false, false, 0, false);
}
double SetNumberFormat(double **a) {
	return app->Write0(a, NULL, false, false, -1, false);
}
double FcloseW(double **) {
	if (!app->f_write) return 0.;
	int rc = fclose(app->f_write);
	if (!rc) app->f_write = NULL;
	return (double)rc;
}
double FcloseR(double **) {
	if (!app->f_read) return 0.;
	int rc = fclose(app->f_read);
	if (!rc) app->f_read = NULL;
	return (double)rc;
}
double SkipLines(double **a) {
	if (!app->f_read) return 0.;
	long long n = (long long)(*a[0] + 0.5);
	char line[1001];
	line[1000] = '\0';
	long long i;
	for (i = 0; i < n && !feof(app->f_read); i++) {
		line[0] = '\0';
		fgets(line, 1000, app->f_read);
		if (line[0] == '\0') break;
	}
	return (double)i;
}
double Not(double **a) {return (!**a);}
double Exp(double **a) {return exp(**a);}
double Ln(double **a) {return log(**a);}
double Lg(double **a) {return log10(**a);}
double Sqrt(double **a) {return sqrt(**a);}
double Sin(double **a) {return sin(**a);}
double Cos(double **a) {return cos(**a);}
double Tg(double **a) {return tan(**a);}
double Arcsin(double **a) {return asin(**a);}
double Arccos(double **a) {return acos(**a);}
double Arctg(double **a) {return atan(**a);}
double Arctg2(double **a) {return atan2(**a, *a[1]);}
double Sinh(double **a) {return sinh(**a);}
double Cosh(double **a) {return cosh(**a);}
double Tanh(double **a) {return tanh(**a);}
double J0(double **a) {return _j0(**a);}
double J1(double **a) {return _j1(**a);}
double Jn(double **a) {
	if (**a >= 0.) return _jn((int)(**a + 0.5), *a[1]);
	else return _jn((int)(**a - 0.5), *a[1]);
}
double Y0(double **a) {return _y0(**a);}
double Y1(double **a) {return _y1(**a);}
double Yn(double **a) {
	if (**a >= 0.) return _yn((int)(**a + 0.5), *a[1]);
	else return _yn((int)(**a - 0.5), *a[1]);
}
double Abs(double **a) {return fabs(**a);}
double Ldexp(double **a) {
	if (*a[1] >= 0.) return ldexp(**a, (int)(*a[1] + 0.5));
	else return ldexp(**a, (int)(*a[1] - 0.5));
}
double Fmod(double **a) {return fmod(**a, *a[1]);}
double Ceil(double **a) {return ceil(**a);}
double Floor(double **a) {return floor(**a);}
double Near(double **a) {
	if (**a >= 0.) return floor(**a + 0.5);
	else return ceil(**a - 0.5);
}
double Srand(double **a) {
	srand((unsigned int)**a);
	return 0.;
}
double Srand2(double **a) {
	int s = (int)(*a[0] + 0.5);
	int ij = s / 30082;
	int kl = s - 30082 * ij;
	app->randGen.RandomInitialise(ij, kl);
	return 0.;
}
double Rand(double **) {return rand();}
double Rand2(double **a) {
	int type = (int)(*a[0] + 0.5);
	if (type <= 0) return app->randGen.random2();
	else return app->randGen.RandomGaussian(0., 1.);
}
double Time(double **) {
	time_t timer;
	return (double)time(&timer);
}
double Clock(double **) {
	return (double)clock() / CLOCKS_PER_SEC * 1000.;
}
double Indirect(double **a) {
	unsigned long long addr = (unsigned long long)(*a[0] + 0.5);
	return *((double *)addr);
}
double Memcpy(double **a) {
	if (*a[2] <= -0.5 || *a[2] >= 2147483647.5) return -1.;
	size_t nBytes = (size_t)(*a[2] + 0.5);
	if (!nBytes) return 0.;
	try {
		void *destAddr = (void *)((unsigned long long)(*a[0] + 0.5));
		void *sourceAddr = (void *)((unsigned long long)(*a[1] + 0.5));
		memcpy(destAddr, sourceAddr, nBytes);
	}
	catch(...) {
		return -1.;
	}
	return 0.;
}

double derf_(double y);
double derfc_(double x);
double dgamma_(double x);
double dlgama_(double x);
double gammp(double a, double x);

double Erf(double **a) {
	if (!FltPntException(**a)) return derf_(**a);
	else return app->indeterminateValue;
}
double Erfc(double **a) {
	if (!FltPntException(**a)) return derfc_(**a);
	else return app->indeterminateValue;
}
double Gamma(double **a) {return dgamma_(**a);}
double Lngamma(double **a) {return dlgama_(**a);}
double GammP(double **a) {return gammp(**a, *a[1]);}
double Max(double **a) {return (**a >= *a[1] ? **a : *a[1]);}
double Min(double **a) {return (**a < *a[1] ? **a : *a[1]);}
double Select(double **a) {
	if (**a) return *a[1];
	else return *a[2];
}

int (*Compare[8])(const void *arg1, const void *arg2);
int Compare_double(const void *arg1, const void *arg2)
{
	double value1 = *((double *)arg1);
	double value2 = *((double *)arg2);
	if (value1 < value2) return -1;
	else if (value1 == value2) return 0;
	else return 1;
}
int Compare_float(const void *arg1, const void *arg2)
{
	float value1 = *((float *)arg1);
	float value2 = *((float *)arg2);
	if (value1 < value2) return -1;
	else if (value1 == value2) return 0;
	else return 1;
}
int Compare_int(const void *arg1, const void *arg2)
{
	int value1 = *((int *)arg1);
	int value2 = *((int *)arg2);
	if (value1 < value2) return -1;
	else if (value1 == value2) return 0;
	else return 1;
}
int Compare_uint(const void *arg1, const void *arg2)
{
	unsigned int value1 = *((unsigned int *)arg1);
	unsigned int value2 = *((unsigned int *)arg2);
	if (value1 < value2) return -1;
	else if (value1 == value2) return 0;
	else return 1;
}
int Compare_int2(const void *arg1, const void *arg2)
{
	short int value1 = *((short int *)arg1);
	short int value2 = *((short int *)arg2);
	if (value1 < value2) return -1;
	else if (value1 == value2) return 0;
	else return 1;
}
int Compare_uint2(const void *arg1, const void *arg2)
{
	unsigned short int value1 = *((unsigned short int *)arg1);
	unsigned short int value2 = *((unsigned short int *)arg2);
	if (value1 < value2) return -1;
	else if (value1 == value2) return 0;
	else return 1;
}
int Compare_int1(const void *arg1, const void *arg2)
{
	char value1 = *((char *)arg1);
	char value2 = *((char *)arg2);
	if (value1 < value2) return -1;
	else if (value1 == value2) return 0;
	else return 1;
}
int Compare_uint1(const void *arg1, const void *arg2)
{
	unsigned char value1 = *((unsigned char *)arg1);
	unsigned char value2 = *((unsigned char *)arg2);
	if (value1 < value2) return -1;
	else if (value1 == value2) return 0;
	else return 1;
}

Compiler::Compiler()
{
	app = this;
	f_print = stdout;
	f_write = NULL;
	f_read = NULL;
	createOutputStrings = true;
	inputLineCreated = false;
	newLine = true;
	inputLineCopied = false;
	checkKeyboard = true;
	escapeKeyPressed = false;
	feedback = false;
	fatalError = false;
	
	index_base = 1;

	resInst = stringTable;
	LoadString(resInst, 2674, types[1], STRLEN_MAX);
	LoadString(resInst, 2719, types[2], STRLEN_MAX);
	LoadString(resInst, 2675, types[3], STRLEN_MAX);
	LoadString(resInst, 2676, types[4], STRLEN_MAX);
	LoadString(resInst, 2677, types[5], STRLEN_MAX);

	stdFunc[0] = Add;
	stdFunc[1] = Subtract;
	stdFunc[2] = Divide;
	stdFunc[3] = Multiply;
	stdFunc[4] = Power;
	stdFunc[5] = Equality;
	stdFunc[6] = GT;
	stdFunc[7] = LT;
	stdFunc[8] = GE;
	stdFunc[9] = LE;
	stdFunc[10] = EQ;
	stdFunc[11] = NE;
	stdFunc[12] = AND;
	stdFunc[13] = OR;
	stdFunc[14] = Dir;
	stdFunc[15] = Dir2;
	stdFunc[16] = Write;
	stdFunc[17] = Write2;
	stdFunc[18] = Print;
	stdFunc[19] = Print2;
	stdFunc[20] = System;
	stdFunc[21] = FopenW;
	stdFunc[22] = FopenR;
	stdFunc[23] = SetNumberFormat;
	stdFunc[24] = FcloseW;
	stdFunc[25] = FcloseR;
	stdFunc[26] = SkipLines;
	stdFunc[27] = Not;
	stdFunc[28] = Exp;
	stdFunc[29] = Ln;
	stdFunc[30] = Lg;
	stdFunc[31] = Sqrt;
	stdFunc[32] = Sin;
	stdFunc[33] = Cos;
	stdFunc[34] = Tg;
	stdFunc[35] = Arcsin;
	stdFunc[36] = Arccos;
	stdFunc[37] = Arctg;
	stdFunc[38] = Arctg2;
	stdFunc[39] = Sinh;
	stdFunc[40] = Cosh;
	stdFunc[41] = Tanh;
	stdFunc[42] = J0;
	stdFunc[43] = J1;
	stdFunc[44] = Jn;
	stdFunc[45] = Y0;
	stdFunc[46] = Y1;
	stdFunc[47] = Yn;
	stdFunc[48] = Abs;
	stdFunc[49] = Ldexp;
	stdFunc[50] = Fmod;
	stdFunc[51] = Ceil;
	stdFunc[52] = Floor;
	stdFunc[53] = Near;
	stdFunc[54] = Srand;
	stdFunc[55] = Srand2;
	stdFunc[RAND_TIME_START] = Rand;
	stdFunc[RAND_TIME_START+1] = Rand2;
	stdFunc[RAND_TIME_START+2] = Time;
	stdFunc[RAND_TIME_END] = Clock;
	stdFunc[60] = Indirect;
	stdFunc[61] = Memcpy;
	stdFunc[62] = Erf;
	stdFunc[63] = Erfc;
	stdFunc[64] = Gamma;
	stdFunc[65] = Lngamma;
	stdFunc[66] = GammP;
	stdFunc[67] = Max;
	stdFunc[68] = Min;
	stdFunc[69] = Select;

	DefineAdvancedStdFunc();

	strcpy(stdFuncNames[0], "Add");
	strcpy(stdFuncNames[1], "Subtract");
	strcpy(stdFuncNames[2], "Divide");
	strcpy(stdFuncNames[3], "Multiply");
	strcpy(stdFuncNames[4], "Power");
	strcpy(stdFuncNames[5], "Equality");
	strcpy(stdFuncNames[6], "GT");
	strcpy(stdFuncNames[7], "LT");
	strcpy(stdFuncNames[8], "GE");
	strcpy(stdFuncNames[9], "LE");
	strcpy(stdFuncNames[10], "EQ");
	strcpy(stdFuncNames[11], "NE");
	strcpy(stdFuncNames[12], "AND");
	strcpy(stdFuncNames[13], "OR");
	strcpy(stdFuncNames[14], "Dir");
	strcpy(stdFuncNames[15], "Dir2");
	strcpy(stdFuncNames[16], "Write");
	strcpy(stdFuncNames[17], "Write2");
	strcpy(stdFuncNames[18], "Print");
	strcpy(stdFuncNames[19], "Print2");
	strcpy(stdFuncNames[20], "System");
	strcpy(stdFuncNames[21], "FopenW");
	strcpy(stdFuncNames[22], "FopenR");
	strcpy(stdFuncNames[23], "SetNumberFormat");
	strcpy(stdFuncNames[24], "FcloseW");
	strcpy(stdFuncNames[25], "FcloseR");
	strcpy(stdFuncNames[26], "SkipLines");
	strcpy(stdFuncNames[27], "Not");
	strcpy(stdFuncNames[28], "exp");
	strcpy(stdFuncNames[29], "ln");
	strcpy(stdFuncNames[30], "lg");
	strcpy(stdFuncNames[31], "sqrt");
	strcpy(stdFuncNames[32], "sin");
	strcpy(stdFuncNames[33], "cos");
	strcpy(stdFuncNames[34], "tg");
	strcpy(stdFuncNames[35], "arcsin");
	strcpy(stdFuncNames[36], "arccos");
	strcpy(stdFuncNames[37], "arctg");
	strcpy(stdFuncNames[38], "arctg2");
	strcpy(stdFuncNames[39], "sinh");
	strcpy(stdFuncNames[40], "cosh");
	strcpy(stdFuncNames[41], "tanh");
	strcpy(stdFuncNames[42], "j0");
	strcpy(stdFuncNames[43], "j1");
	strcpy(stdFuncNames[44], "jn");
	strcpy(stdFuncNames[45], "y0");
	strcpy(stdFuncNames[46], "y1");
	strcpy(stdFuncNames[47], "yn");
	strcpy(stdFuncNames[48], "abs");
	strcpy(stdFuncNames[49], "ldexp");
	strcpy(stdFuncNames[50], "fmod");
	strcpy(stdFuncNames[51], "ceil");
	strcpy(stdFuncNames[52], "floor");
	strcpy(stdFuncNames[53], "near");
	strcpy(stdFuncNames[54], "srand");
	strcpy(stdFuncNames[55], "srand2");
	strcpy(stdFuncNames[RAND_TIME_START], "rand");
	strcpy(stdFuncNames[RAND_TIME_START+1], "rand2");
	strcpy(stdFuncNames[RAND_TIME_START+2], "time");
	strcpy(stdFuncNames[RAND_TIME_END], "clock");
	strcpy(stdFuncNames[60], "indirect");
	strcpy(stdFuncNames[61], "memcpy");
	strcpy(stdFuncNames[62], "erf");
	strcpy(stdFuncNames[63], "erfc");
	strcpy(stdFuncNames[64], "gamma");
	strcpy(stdFuncNames[65], "lngamma");
	strcpy(stdFuncNames[66], "gammp");
	strcpy(stdFuncNames[67], "max");
	strcpy(stdFuncNames[68], "min");
	strcpy(stdFuncNames[69], "select");

	strcpy(stdFuncNames[NSTDFUNC0], "loc");
	reverseArgList[NSTDFUNC0] = FALSE;
	stdFuncArgInfo[NSTDFUNC0].count = 1;
	stdFuncArgInfo[NSTDFUNC0].type[0] = 0;
	strcpy(stdFuncNames[NSTDFUNC0+1], "Invert");
	reverseArgList[NSTDFUNC0+1] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+1].count = 1;
	stdFuncArgInfo[NSTDFUNC0+1].type[0] = 3;
	strcpy(stdFuncNames[NSTDFUNC0+2], "Size");
	reverseArgList[NSTDFUNC0+2] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+2].count = 1;
	stdFuncArgInfo[NSTDFUNC0+2].type[0] = 2;
	strcpy(stdFuncNames[NSTDFUNC0+3], "Size2");
	reverseArgList[NSTDFUNC0+3] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+3].count = 2;
	stdFuncArgInfo[NSTDFUNC0+3].type[0] = 0;
	stdFuncArgInfo[NSTDFUNC0+3].type[1] = 0;

	// Integration, summation, iteration and root-finding functions. Currently, all those functions have at least 4 arguments.
	// If a function with less than 4 arguments is added to this category in the future, there must be a mechanism that
	// prevents it from being passed to another function as an argument.
	strcpy(stdFuncNames[NSTDFUNC0+4], "Int");
	reverseArgList[NSTDFUNC0+4] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+4].count = 4;
	strcpy(stdFuncNames[NSTDFUNC0+5], "Int2");
	reverseArgList[NSTDFUNC0+5] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+5].count = 7;
	strcpy(stdFuncNames[NSTDFUNC0+6], "Inti");
	reverseArgList[NSTDFUNC0+6] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+6].count = 4;
	strcpy(stdFuncNames[NSTDFUNC0+7], "Inti2");
	reverseArgList[NSTDFUNC0+7] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+7].count = 7;
	strcpy(stdFuncNames[NSTDFUNC0+8], "Intw");
	reverseArgList[NSTDFUNC0+8] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+8].count = 6;
	strcpy(stdFuncNames[NSTDFUNC0+9], "Intw2");
	reverseArgList[NSTDFUNC0+9] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+9].count = 10;
	strcpy(stdFuncNames[NSTDFUNC0+10], "Sum");
	reverseArgList[NSTDFUNC0+10] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+10].count = 4;
	strcpy(stdFuncNames[NSTDFUNC0+11], "Sum2");
	reverseArgList[NSTDFUNC0+11] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+11].count = 5;
	strcpy(stdFuncNames[NSTDFUNC0+12], "Iter");
	reverseArgList[NSTDFUNC0+12] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+12].count = 4;
	strcpy(stdFuncNames[NSTDFUNC0+13], "Root");
	reverseArgList[NSTDFUNC0+13] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+13].count = 4;
	strcpy(stdFuncNames[NSTDFUNC0+14], "Root2");
	reverseArgList[NSTDFUNC0+14] = TRUE;
	stdFuncArgInfo[NSTDFUNC0+14].count = 7;
	int i, j;
	for (i = NSTDFUNC0 + 4; i <= NSTDFUNC0 + 14; i++) {
		for (j = 0; j < stdFuncArgInfo[i].count; j++) stdFuncArgInfo[i].type[j] = 0;
	}

	stdFuncArgInfo[NSTDFUNC0+14].type[6] = 1;
	strcpy(stdFuncNames[NSTDFUNC0+15], "Return");
	reverseArgList[NSTDFUNC0+15] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+15].count = 1;
	stdFuncArgInfo[NSTDFUNC0+15].type[0] = 0;
	strcpy(stdFuncNames[NSTDFUNC0+16], "Find");
	reverseArgList[NSTDFUNC0+16] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+16].count = 3;
	stdFuncArgInfo[NSTDFUNC0+16].type[0] = 0;
	stdFuncArgInfo[NSTDFUNC0+16].type[1] = 2;
	stdFuncArgInfo[NSTDFUNC0+16].type[2] = 0;
	strcpy(stdFuncNames[NSTDFUNC0+17], "Find2");
	reverseArgList[NSTDFUNC0+17] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+17].count = 5;
	stdFuncArgInfo[NSTDFUNC0+17].type[0] = 0;
	stdFuncArgInfo[NSTDFUNC0+17].type[1] = 2;
	stdFuncArgInfo[NSTDFUNC0+17].type[2] = 0;
	stdFuncArgInfo[NSTDFUNC0+17].type[3] = 0;
	stdFuncArgInfo[NSTDFUNC0+17].type[4] = 0;
	strcpy(stdFuncNames[NSTDFUNC0+18], "Hist");
	reverseArgList[NSTDFUNC0+18] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+18].count = 3;
	stdFuncArgInfo[NSTDFUNC0+18].type[0] = 2;
	stdFuncArgInfo[NSTDFUNC0+18].type[1] = 2;
	stdFuncArgInfo[NSTDFUNC0+18].type[2] = 2;
	strcpy(stdFuncNames[NSTDFUNC0+19], "Hist2");
	reverseArgList[NSTDFUNC0+19] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+19].count = 5;
	stdFuncArgInfo[NSTDFUNC0+19].type[0] = 2;
	stdFuncArgInfo[NSTDFUNC0+19].type[1] = 2;
	stdFuncArgInfo[NSTDFUNC0+19].type[2] = 2;
	stdFuncArgInfo[NSTDFUNC0+19].type[3] = 0;
	stdFuncArgInfo[NSTDFUNC0+19].type[4] = 0;
	strcpy(stdFuncNames[NSTDFUNC0+20], "ReadArray");
	reverseArgList[NSTDFUNC0+20] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+20].count = 1;
	stdFuncArgInfo[NSTDFUNC0+20].type[0] = 2;
	strcpy(stdFuncNames[NSTDFUNC0+21], "ReadArray2");
	reverseArgList[NSTDFUNC0+21] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+21].count = 2;
	stdFuncArgInfo[NSTDFUNC0+21].type[0] = 2;
	stdFuncArgInfo[NSTDFUNC0+21].type[1] = 0;
	strcpy(stdFuncNames[NSTDFUNC0+22], "Sort");
	reverseArgList[NSTDFUNC0+22] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+22].count = 1;
	stdFuncArgInfo[NSTDFUNC0+22].type[0] = 2;
	strcpy(stdFuncNames[NSTDFUNC0+23], "Sort2");
	reverseArgList[NSTDFUNC0+23] = FALSE;
	stdFuncArgInfo[NSTDFUNC0+23].count = 2;
	stdFuncArgInfo[NSTDFUNC0+23].type[0] = 2;
	stdFuncArgInfo[NSTDFUNC0+23].type[1] = 2;

	funcType.SetSize(NSTDFUNC);
	for (i = 0; i < NSTDFUNC; i++) funcType[i] = FuncType(i);

	for (i = 0; i < 200; i++) typeSize[i] = TypeSize(i);

	strcpy(systemVarNames[0], "@0");
	strcpy(systemVarNames[1], "@1");
	strcpy(systemVarNames[2], "@2");
	strcpy(systemVarNames[3], "@3");
	strcpy(systemVarNames[4], "@4");
	strcpy(systemVarNames[5], "@5");
	strcpy(systemVarNames[6], "@6");
	strcpy(systemVarNames[7], "@7");
	strcpy(systemVarNames[8], "@8");
	strcpy(systemVarNames[9], "@9");
	strcpy(systemVarNames[10], "IER");
	strcpy(systemVarNames[11], "AbsErr");
	strcpy(systemVarNames[12], "nEval");
	memset(sysVarValues, 0, NSYSTEMVARS * sizeof(double));

	nGParm = 0;
	nSubroutines = 0;
	nArrays0 = 0;
	nArrays = NSPECARRAYS;
	nDllFunc = 0;

	aPtr1 = new PROGRAM_ARGS1;

	indeterminateValue = 2. * DBL_MAX - 2. * DBL_MAX;

	strcpy(defaultNumberFormat0, "%.14g");
	strcpy(defaultNumberFormat, defaultNumberFormat0);

	initRootVars.c__4 = 4;
	initRootVars.c_b17 = 1.;
	initRootVars.a = 0.;
	initRootVars.p = 0.;
	initRootVars.q = 0.;
	initRootVars.u = 0.;
	initRootVars.ae = 0.;
	initRootVars.fa = 0.;
	initRootVars.fb = 0.;
	initRootVars.fc = 0.;
	initRootVars.ic = 0;
	initRootVars.re = 0.;
	initRootVars.fx = 0.;
	initRootVars.cmb = 0.;
	initRootVars.tol = 0.;
	initRootVars.acmb = 0.;
	initRootVars.acbs = 0.;
	initRootVars.kount = 0;

	randGen.RandomInitialise(1802, 9373);

	Compare[0] = Compare_double;
	Compare[1] = Compare_float;
	Compare[2] = Compare_int;
	Compare[3] = Compare_uint;
	Compare[4] = Compare_int2;
	Compare[5] = Compare_uint2;
	Compare[6] = Compare_int1;
	Compare[7] = Compare_uint1;

	subroutine = (EQDATA *)malloc(0);
	subroutine2 = (OPDATA *)malloc(0);
	subroutineArgInfo = (FUNCARGINFO *)malloc(0);
	outputLines = (Array_String *)malloc(0);
	outputLineNumbers = (Array_int *)malloc(0);
	arrays = (ARRAYDATA *)malloc(0);
	fpos = (fpos_t **)malloc(0);
	startPos = (fpos_t *)malloc(0);
	endPos = (fpos_t *)malloc(0);
}

BOOL Compiler::CheckIdentifier(char *id, int mode, int &num, char *prefix, int &nConf, int *&typeConf, int *&numConf, char **&nameConf, char *&nameConfBuffer, char *message)
{
	char tmp[1001];
	Array_int typeC, numC;
	Array_String namesC;
	typeC.SetSize(0);
	numC.SetSize(0);
	namesC.SetSize(0);
	size_t nBlanks = strspn(id, " \t");
	if (!prefix) {
		if (nBlanks == strlen(id)) {
			LoadString(resInst, 2288, S, STRLEN_MAX); goto ERROR0;
		}
		else if (strpbrk(id, "+-/*^=,().!?<>;:[]{}|\\'%\"")) {
			LoadString(resInst, 2289, S, STRLEN_MAX); goto ERROR0;
		}
		else if (strpbrk(id, "0123456789") == id + nBlanks) {
			LoadString(resInst, 2290, S, STRLEN_MAX); goto ERROR0;
		}
	}
	else {
		if (nBlanks == strlen(id)) {
			LoadString(resInst, 2238, S, STRLEN_MAX); goto ERROR0;
		}
		else if (strpbrk(id, "+-/*^=,().!?<>;:[]{}|\\'%\"")) {
			LoadString(resInst, 2232, S, STRLEN_MAX); goto ERROR0;
		}
		else if (strpbrk(id, "0123456789") == id + nBlanks) {
			LoadString(resInst, 2233, S, STRLEN_MAX); goto ERROR0;
		}
	}
	strcpy(tmp, id + nBlanks);
	strrev(tmp);
	strcpy(id, tmp + strspn(tmp, " \t"));
	strrev(id);
	int i, n;
	BOOL getParent;
	if (mode < 0) {
		mode = -mode;
		getParent = TRUE;
	}
	else getParent = FALSE;
	if (mode != 1) {
		n = nGParm;
		for (i = 0; i < n && strcmpi(id, globalParms[i].name); i++);
		if (i < n) {
			typeC.Add(1);
			numC.Add(i);
			namesC.Add(globalParms[i].name);
		}
	}
	for (i = 0; i < NSYSTEMVARS && strcmpi(id, systemVarNames[i]); i++);
	if (i < NSYSTEMVARS) {
		LoadString(resInst, 2292, S, STRLEN_MAX); num = -1; goto ERROR1;
	}
	for (i = 0; i < NSPECARRAYS && strcmpi(id, arrays[i+nArrays0].name); i++);
	if (i < NSPECARRAYS) {
		LoadString(resInst, 2372, S, STRLEN_MAX); num = -2; goto ERROR1;
	}
	for (i = NBINARYOP; i < NSTDFUNC && strcmpi(id, stdFuncNames[i]); i++);
	if (i < NSTDFUNC) {
		LoadString(resInst, 2293, S, STRLEN_MAX); num = -3; goto ERROR1;
	}
	int num1;
	if (mode != 3) num1 = -1;
	else num1 = num;
	for (i = 0; i < nSubroutines && (i == num1 || strcmpi(id, subroutine[i].name)); i++);
	if (i < nSubroutines) {
		if (mode != 3) {
			typeC.Add(3);
			numC.Add(i);
			namesC.Add(subroutine[i].name);
		}
		else {
			LoadString(resInst, 2235, S, STRLEN_MAX);
			num = i;
			goto ERROR1;
		}
	}
/*	n = nDllFunc;
	for (i = 0; i < n && strcmpi(id, dllFuncName2[i]); i++);
	if (i < n) {
		typeC.Add(4);
		numC.Add(i);
		namesC.Add(dllFuncName2[i]);
	}*/
	if (mode != 5 && mode != 6) num1 = -1;
	else num1 = num;
	for (i = 0; i < nArrays0 && (i == num1 || strcmpi(id, arrays[i].name)); i++);
	if (i < nArrays0) {
		if (mode != 5) {
			typeC.Add(5);
			numC.Add(i);
			namesC.Add(arrays[i].name);
		}
		else {
			LoadString(resInst, 2236, S, STRLEN_MAX);
			num = i;
			goto ERROR1;
		}
	}
	int nConf1 = typeC.GetSize();
	if (nConf1) {
		int nConf_new = nConf + nConf1;
		typeConf = (int *)realloc(typeConf, nConf_new * sizeof(int));
		numConf = (int *)realloc(numConf, nConf_new * sizeof(int));
		nameConf = (char **)realloc(nameConf, nConf_new * sizeof(char *));
		size_t nBytes = 0;
		for (i = 0; i < nConf; i++) {
			size_t len = strlen(nameConfBuffer + nBytes);
			nBytes += (len + 1);
		}
		int j;
		size_t nBytes1 = 0;
		for (j = 0; j < nConf1; j++) {
			nBytes1 += (strlen(namesC[j]) + 1);
		}
		size_t nBytes_new = nBytes + nBytes1;
		char *nameConfBuffer_prev = nameConfBuffer;
		nameConfBuffer = (char *)realloc(nameConfBuffer, nBytes_new);
		__int64 diff = nameConfBuffer - nameConfBuffer_prev;
		for (i = 0; i < nConf; i++) nameConf[i] += diff;
		size_t pos = nBytes;
		for (i = nConf, j = 0; j < nConf1; j++, i++) {
			typeConf[i] = typeC[j];
			numConf[i] = numC[j];
			nameConf[i] = nameConfBuffer + pos;
			strcpy(nameConf[i], namesC[j]);
			pos += (strlen(nameConf[i]) + 1);
		}
		nConf = nConf_new;
	}
	return TRUE;
ERROR0:
	if (!prefix) strcpy(message, S);
	else sprintf(message, S, prefix);
	return FALSE;
ERROR1:
	sprintf(message, S, id);
	return FALSE;
}

int Find(double value, double *a, int n, int start = -1, int nSwitch = 40);
int Find(double value, double *a, int n, int start, int nSwitch)
{
	int i;
	if (n < nSwitch) {
		if (start < 0 || start >= n) start = n / 2;
		if (a[start] < value) {
			for (i = start; i < n && a[i] < value; i++);
			if (i == n || a[i] > value) i--;
		}
		else {
			for (i = start; i >= 0 && a[i] > value; i--);
		}
	}
	else {
		if (value < a[0]) return -1;
		else if (value >= a[n-1]) return n - 1;
		if (start < 0 || start >= n) start = n / 2;
		i = start;
		int left = 0, right = n - 1;
		do {
			if (a[i] < value) left = i;
			else if (a[i] == value) break;
			else right = i;
			i = (left + right) / 2;
		} while(i > left);
	}
	return i;
}

int Find1(double value, float *a, int n, int start = -1, int nSwitch = 40);
int Find1(double value, float *a, int n, int start, int nSwitch)
{
	int i;
	if (n < nSwitch) {
		if (start < 0 || start >= n) start = n / 2;
		if (a[start] < value) {
			for (i = start; i < n && a[i] < value; i++);
			if (i == n || a[i] > value) i--;
		}
		else {
			for (i = start; i >= 0 && a[i] > value; i--);
		}
	}
	else {
		if (value < a[0]) return -1;
		else if (value >= a[n-1]) return n - 1;
		if (start < 0 || start >= n) start = n / 2;
		i = start;
		int left = 0, right = n - 1;
		do {
			if (a[i] < value) left = i;
			else if (a[i] == value) break;
			else right = i;
			i = (left + right) / 2;
		} while(i > left);
	}
	return i;
}

void ArrayAssignment(void *ptr, long type, double value);
void* AddressAssignment(unsigned long long *ptr, long type, long offset);

void DMINV(double *A,long N,double &D,long *L, long *M);
void DSINV(double *A,long N,float EPS,long &IER);
void DMFSD(double *A,long N,float EPS,long &IER);
int dqags_(EXTERN_FUNC *f, double *a, double *b, double *epsabs, double *epsrel, double *result, double *abserr, 
	long *neval, long *ier, long *limit, long *lenw, long *last, long *iwork, double *work);
int dqagi_(EXTERN_FUNC *f, double *bound, long *inf, double *epsabs, double *epsrel, double *result, double *abserr,
	long *neval, long *ier, long *limit, long *lenw, long *last, long *iwork, double *work);
int dqawo_(EXTERN_FUNC *f, double *a, double *b, double *omega, long *integr, double *epsabs, double *epsrel,
	double *result, double *abserr, long *neval, long *ier, long *leniw, long *maxp1, long *lenw, long *last,
	long *iwork, double *work);
int dqawf_(EXTERN_FUNC *f, double *a, double *omega, long *integr, double *epsabs, double *result, double *abserr, 
	long *neval, long *ier, long *limlst, long *lst, long *leniw, long *maxp1, long *lenw, long *iwork, double *work);
int root_(double *t, double *ft, double *b, double *c__, double *relerr, double *abserr, long *iflag, ROOT_VARS *RootVars);

int Int_Inti(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i);

void RunProgram0(void *unused)
{
	app->RunProgram1();
	app->aPtr1->calculationInProgress = FALSE;
}

OPDATA *op;
int j, l;

int Loc(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	if (*op->argPtr[i][0] == 0) { // loc(<formal parameter>)
		int argNum = (int)(*op->argPtr[i][1]);
		if (tmpVarPtr->arg_info[argNum].objType == 1) {
			op->operands[op->numInterm[i]] = aPtr2->arg1[argNum];
		}
		else {
			LOC *location2 = (LOC *)(op->operands + op->numInterm[i] + op->nOperands);
			location2->type_offset0 = abs(tmpVarPtr->arg_info[argNum].objType) * 8192;
			location2->num = tmpVarPtr->arg_info[argNum].objNum;
			if (abs(tmpVarPtr->arg_info[argNum].objType) == 2) {
				location2->type_offset0 += tmpVarPtr->arg_info[argNum].offset0;
				location2->offset = tmpVarPtr->arg_info[argNum].a2;
				op->operands[op->numInterm[i]] = aPtr2->arg1[argNum];
			}
			else {
				int num2 = location2->num + 1;
				if (tmpVarPtr->arg_info[argNum].objType > 3) num2 += NSTDFUNC;
				if (tmpVarPtr->arg_info[argNum].objType > 4) num2 += app->nSubroutines;
				op->operands[op->numInterm[i]] = num2;
			}
		}
	}
	else if (*op->argPtr[i][0] == 1) { // address of a variable
		op->operands[op->numInterm[i]] = (double)(unsigned long long)(op->argPtr[i][1]);
	}
	else if (*op->argPtr[i][0] == -1) { // location of an array element returned by the reference to the array
		op->operands[op->numInterm[i]] = *op->argPtr[i][1];
		op->operands[op->numInterm[i]+op->nOperands] = op->argPtr[i][1][op->nOperands];
	}
	else { // location of a function or array (returned when only the identifier is specified as an argument of loc())
		LOC *location2 = (LOC *)(op->operands + op->numInterm[i] + op->nOperands);
		location2->type_offset0 = (unsigned short)(*op->argPtr[i][0]) * 8192;
		location2->num = (unsigned short)(*op->argPtr[i][1]);
		if (location2->type_offset0 == 2 * 8192) { // array
			op->operands[op->numInterm[i]] = (double)(unsigned long long)(app->arrays[location2->num].elements);
			location2->offset = 0;
	}
	else {
		int num2 = location2->num + 1;
		if (location2->type_offset0 > 3 * 8192) num2 += NSTDFUNC;
			if (location2->type_offset0 > 4 * 8192) num2 += app->nSubroutines;
			op->operands[op->numInterm[i]] = num2;
		}
	}
	return 0;
}

int Invert(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	// j and l are global
	LOC *location = (LOC *)(op->argPtr[i][0] + op->nOperands);
	if (location->num >= app->nArrays0) {
		app->ArrayError(3, i, aPtr2, location->num); return 1;
	}
	if (app->arrays[location->num].type) {
		app->BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	int nD0 = app->arrays[location->num].nDim;
	unsigned long nEl1 = app->arrays[location->num].nElem[nD0-2] - location->offset;
	if (nEl1 != app->arrays[location->num].nElem[nD0-1]) {
		app->BuiltInFuncError(2, i, aPtr2, funcNum); return 1;
	}
	unsigned long long arrayAddress = (unsigned long long)((*op->argPtr[i][0]) + 0.5);
	if (!arrayAddress) {
		app->ArrayError(2, i, aPtr2, location->num); return 1;
	}
	double *arr = (double *)arrayAddress;
	long *wrk1 = NULL, *wrk2 = NULL;
	try
	{
	wrk1 = new long[nEl1];
	wrk2 = new long[nEl1];
	}
	catch(...)
	{
		if (wrk1) delete wrk1;
		if (wrk2) delete wrk2;
		app->BuiltInFuncError(3, i, aPtr2, funcNum); return 1;
	}
	double D;
	DMINV(arr, nEl1, D, wrk1, wrk2);
	op->operands[op->numInterm[i]] = D;
	delete wrk1;
	delete wrk2;
	return 0;
}

int Compiler::Size0(LOC *location)
{
	int iDim = app->arrays[location->num].nDim - 1;
	return arrays[location->num].nElem[iDim] - location->offset;
}

int Size(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	LOC *location = (LOC *)(op->argPtr[i][0] + op->nOperands);
	op->operands[op->numInterm[i]] = app->Size0(location);
	return 0;
}

int Size2(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	LOC *location = (LOC *)(op->argPtr[i][0] + op->nOperands);
	if (location->type_offset0 / 8192 == 2 && location->num >= 0 && location->num < app->nArrays) {
		int iDim = app->index_base ? (int)((*op->argPtr[i][1]) - 0.5) : (int)((*op->argPtr[i][1]) + 0.5);
		unsigned short o0 = location->type_offset0 - 2 * 8192;
		if ((app->index_base && (*op->argPtr[i][1]) >= 0.5 || !app->index_base && (*op->argPtr[i][1]) >= -0.5) && o0 + iDim < app->arrays[location->num].nDim) {
			if (!iDim) op->operands[op->numInterm[i]] = app->arrays[location->num].nElem[o0] - location->offset;
			else op->operands[op->numInterm[i]] = app->arrays[location->num].nElem[o0+iDim];
		}
		else op->operands[op->numInterm[i]] = 0.;
	}
	else {
		app->BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	return 0;
}

int Int_Inti(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	long ier, last, neval, limit; // j and l are global
	double epsabs, epsrel, abserr, result;
	int offset;
	if (funcNum == NSTDFUNC0 + 4 || funcNum == NSTDFUNC0 + 6) { // Int, Inti
		epsabs = 0.;
		epsrel = 1e-5;
		limit = 200;
		offset = 0;
	}
	else { // Int2, Inti2
		epsabs = *(op->argPtr[i][2]);
		epsrel = *(op->argPtr[i][1]);
		limit = (long)(*(op->argPtr[i][0]) + 0.5);
		if (limit > 1000000) {
			app->BuiltInFuncError(7, i, aPtr2, funcNum); return 1;
		}
		offset = 3;
	}
	long lenw = 4 * limit;
	EXTERN_FUNC *f = NULL;
	long *iwork = NULL;
	double *work = NULL;
	try
	{
	f = new EXTERN_FUNC;
	if (limit > 0) iwork = new long[limit];
	else iwork = new long[1];
	if (lenw > 0) work = new double[lenw];
	else work = new double[1];
	}
	catch(...)
	{
		if (f) delete f;
		if (iwork) delete iwork;
		if (work) delete work;
		app->BuiltInFuncError(9, i, aPtr2, funcNum); return 1;
	}
	f->argPtr = op->argPtr[i][2+offset];
	f->aPtr2 = aPtr2;
	f->tmpVarPtr = tmpVarPtr;
	f->iStart = op->control[tmpVarPtr->nextControl-1];
	f->iEnd = i - 1;
	if (funcNum < NSTDFUNC0 + 6) { // Int, Int2
		double a = *(op->argPtr[i][1+offset]);
		double b = *(op->argPtr[i][offset]);
		dqags_(f, &a, &b, &epsabs, &epsrel, &result, &abserr, &neval, &ier, &limit, &lenw, &last, iwork, work);
	}
	else { // Inti, Inti2
		double bound = *(op->argPtr[i][1+offset]);
		long inf;
		if (*(op->argPtr[i][offset]) >= 0.) {
			inf = (long)(*(op->argPtr[i][offset]) + 0.5);
			if (!inf) inf = 1;
			else if (inf > 2) inf = 2;
		}
		else {
			inf = (long)(*(op->argPtr[i][offset]) - 0.5);
			if (!inf) inf = -1;
			else if (inf < -2) inf = 2;
		}
		dqagi_(f, &bound, &inf, &epsabs, &epsrel, &result, &abserr, &neval, &ier, &limit, &lenw, &last, iwork, work);
	}
	app->sysVarValues[10] = ier;
	app->sysVarValues[11] = abserr;
	app->sysVarValues[12] = neval;
	delete f;
	delete iwork;
	delete work;
	if (ier) {
		long curOp_bak = aPtr2->op->curOp;
		app->BuiltInFuncError(ier, i, aPtr2, funcNum);
		aPtr2->op->callDepth++; // because "BuiltInFuncError" decrements callDepth
		aPtr2->op->curOp = curOp_bak;
		if (ier == 10) result = app->indeterminateValue;
	}
	op->operands[op->numInterm[i]] = result;
	return 0;
}

int Intw(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	double epsabs, epsrel;
	long leniw, maxp1;
	int offset; // j and l are global
	if (funcNum == NSTDFUNC0 + 8) { // Intw
		epsabs = 0.;
		epsrel = 1e-5;
		leniw = 400;
		maxp1 = 100;
		offset = 0;
	}
	else { // Intw2
		epsabs = *(op->argPtr[i][3]);
		epsrel = *(op->argPtr[i][2]);
		leniw = (long)(*(op->argPtr[i][1]) + 0.5);
		if (leniw > 2000000) {
			app->BuiltInFuncError(7, i, aPtr2, funcNum); return 1;
		}
		maxp1 = (long)(*(op->argPtr[i][0]) + 0.5);
		if (maxp1 > 200000) {
			app->BuiltInFuncError(8, i, aPtr2, funcNum); return 1;
		}
		offset = 4;
	}
	double omega = *(op->argPtr[i][1+offset]);
	long integr = (long)(*(op->argPtr[i][offset]) + 0.5);
	if (integr < 1) integr = 1;
	else if (integr > 2) integr = 2;
	double result;
	if (omega || integr == 1) {
		EXTERN_FUNC *f = NULL;
		long ier, neval, *iwork = NULL;
		double abserr, *work = NULL;
		long last;
		long lenw = 2 * leniw + 25 * maxp1;
		try
		{
		f = new EXTERN_FUNC;
		if (leniw > 0) iwork = new long[leniw];
		else iwork = new long[1];
		if (lenw > 0) work = new double[lenw];
		else work = new double[1];
		}
		catch(...)
		{
			if (f) delete f;
			if (iwork) delete iwork;
			if (work) delete work;
			app->BuiltInFuncError(9, i, aPtr2, funcNum); return 1;
		}
		f->aPtr2 = aPtr2;
		f->tmpVarPtr = tmpVarPtr;
		f->iStart = op->control[tmpVarPtr->nextControl-1];
		f->iEnd = i - 1;
		f->argPtr = op->argPtr[i][4+offset];
		double a = *(op->argPtr[i][3+offset]);
		double b = *(op->argPtr[i][2+offset]);
		dqawo_(f, &a, &b, &omega, &integr, &epsabs, &epsrel, &result, &abserr, &neval, &ier, &leniw, &maxp1, &lenw, &last, iwork, work);
		app->sysVarValues[10] = ier;
		app->sysVarValues[11] = abserr;
		app->sysVarValues[12] = neval;
		delete f;
		delete iwork;
		delete work;
		if (ier) {
			long curOp_bak = op->curOp;
			app->BuiltInFuncError(ier, i, aPtr2, funcNum);
			op->callDepth++; // because "BuiltInFuncError" decrements callDepth
			op->curOp = curOp_bak;
			if (ier == 10) result = app->indeterminateValue;
		}
	}
	else {
		result = 0.;
		app->sysVarValues[10] = 0;
		app->sysVarValues[11] = 0;
		app->sysVarValues[12] = 1;
	}
	op->operands[op->numInterm[i]] = result;
	return 0;
}

int Sum(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	long n, n1, n2; // j and l are global
	double a = *(op->argPtr[i][1]);
	if (a >= 0.) n1 = (long)(a + 0.5);
	else n1 = (long)(a - 0.5);
	a = *(op->argPtr[i][0]);
	if (a >= 0.) n2 = (long)(a + 0.5);
	else n2 = (long)(a - 0.5);
	int iStart = op->control[tmpVarPtr->nextControl-1];
	double sum = 0.;
	if (n1 <= n2) {
		for (n = n1; n <= n2; n++) {
			*op->argPtr[i][2] = n;
			app->RunProgram1(aPtr2, tmpVarPtr, iStart, i - 1);
			sum += app->aPtr1->retVal;
		}
	}
	else {
		for (n = n1; n >= n2; n--) {
			*op->argPtr[i][2] = n;
			app->RunProgram1(aPtr2, tmpVarPtr, iStart, i - 1);
			sum += app->aPtr1->retVal;
		}
	}
	op->operands[op->numInterm[i]] = sum;
	return 0;
}

int Sum2(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	double incr = fabs(*(op->argPtr[i][0]));
	if (!incr) {
		app->BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	double a2 = *(op->argPtr[i][1]);
	double a1 = *(op->argPtr[i][2]);
	int iStart = op->control[tmpVarPtr->nextControl-1];
	double a, sum = 0.; // j and l are global
	if (a1 <= a2) {
		for (a = a1; a <= a2; a += incr) {
			*op->argPtr[i][3] = a;
			app->RunProgram1(aPtr2, tmpVarPtr, iStart, i - 1);
			sum += app->aPtr1->retVal;
		}
	}
	else {
		for (a = a1; a >= a2; a -= incr) {
			*op->argPtr[i][3] = a;
			app->RunProgram1(aPtr2, tmpVarPtr, iStart, i - 1);
			sum += app->aPtr1->retVal;
		}
	}
	op->operands[op->numInterm[i]] = sum;
	return 0;
}

int Iter(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	long n, n1, n2; // j and l are global
	double a = *(op->argPtr[i][1]);
	if (a >= 0.) n1 = (long)(a + 0.5);
	else n1 = (long)(a - 0.5);
	a = *(op->argPtr[i][0]);
	if (a >= 0.) n2 = (long)(a + 0.5);
	else n2 = (long)(a - 0.5);
	int iStart = op->control[tmpVarPtr->nextControl-1];
	if (n1 <= n2) {
		for (n = n1; n <= n2; n++) {
			*op->argPtr[i][2] = n;
			app->RunProgram1(aPtr2, tmpVarPtr, iStart, i - 1);
		}
	}
	else {
		for (n = n1; n >= n2; n--) {
			*op->argPtr[i][2] = n;
			app->RunProgram1(aPtr2, tmpVarPtr, iStart, i - 1);
		}
	}
	op->operands[op->numInterm[i]] = app->aPtr1->retVal;
	return 0;
}

int Root(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	// j and l are global
	tmpVarPtr->curRootDepth++;
	ROOT_VARS *pRootVars = tmpVarPtr->RootVars + tmpVarPtr->curRootDepth;
	*pRootVars = app->initRootVars;
	double b = *(op->argPtr[i][1]);
	double c = *(op->argPtr[i][0]);
	double relerr = 1e-9;
	double abserr = 1e-100;
	long iflag;
	double t = 0.5 * (b + c);
	int iStart = op->control[tmpVarPtr->nextControl-1];
	do {
		*op->argPtr[i][2] = t;
		app->RunProgram1(aPtr2, tmpVarPtr, iStart, i - 1);
		root_(&t, &(app->aPtr1->retVal), &b, &c, &relerr, &abserr, &iflag, pRootVars);
	} while(iflag < 0);
	iflag--;
	*op->argPtr[i][0] = c;
	*op->argPtr[i][1] = b;
	*op->argPtr[i][2] = t;
	if (iflag > 1) {
	//if (iflag) {
		long curOp_bak = op->curOp;
		app->BuiltInFuncError(iflag, i, aPtr2, funcNum);
		op->callDepth++; // because "BuiltInFuncError" decrements callDepth
		op->curOp = curOp_bak;
	}
	op->operands[op->numInterm[i]] = b;
	tmpVarPtr->curRootDepth--;
	return 0;
}

int Root2(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	// j and l are global
	tmpVarPtr->curRootDepth++;
	ROOT_VARS *pRootVars = tmpVarPtr->RootVars + tmpVarPtr->curRootDepth;
	*pRootVars = app->initRootVars;
	long iflag;
	//double *iflagDbl = op->argPtr[i][0];
						double *iflagDbl = (double *)((unsigned long long)(*op->argPtr[i][0] + 0.5));
	double abserr = *(op->argPtr[i][1]);
	double relerr = *(op->argPtr[i][2]);
	double c = *(op->argPtr[i][3]);
	double b = *(op->argPtr[i][4]);
	double t = 0.5 * (b + c);
	int iStart = op->control[tmpVarPtr->nextControl-1];
	do {
		*op->argPtr[i][5] = t;
		app->RunProgram1(aPtr2, tmpVarPtr, iStart, i - 1);
		root_(&t, &(app->aPtr1->retVal), &b, &c, &relerr, &abserr, &iflag, pRootVars);
	} while(iflag < 0);
	iflag--;
	*iflagDbl = iflag;
	if (iflag > 1) {
	//if (iflag) {
		long curOp_bak = op->curOp;
		app->BuiltInFuncError(iflag, i, aPtr2, funcNum);
		op->callDepth++; // because "BuiltInFuncError" decrements callDepth
		op->curOp = curOp_bak;
	}
	op->operands[op->numInterm[i]] = b;
	tmpVarPtr->curRootDepth--;
	return 0;
}

int Find(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	LOC *location = (LOC *)(op->argPtr[i][1] + op->nOperands);
	if (app->arrays[location->num].type > 1) {
		app->BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	BOOL isDbl = (app->arrays[location->num].type == 0);
	// j and l are global
	int iDim = app->arrays[location->num].nDim - 1;
	int n = app->arrays[location->num].nElem[iDim] - location->offset;
	double value = *op->argPtr[i][0];
	if (value == value) {
		int startPos = app->index_base ? (int)(*op->argPtr[i][2] - 0.5) : (int)(*op->argPtr[i][2] + 0.5);
		if (isDbl) {
			double *a = (double *)((unsigned long long)(*op->argPtr[i][1] + 0.5));
			op->operands[op->numInterm[i]] = Find(value, a, n, startPos);
		}
		else {
			float *a = (float *)((unsigned long long)(*op->argPtr[i][1] + 0.5));
			op->operands[op->numInterm[i]] = Find1(value, a, n, startPos);
		}
		if (app->index_base) op->operands[op->numInterm[i]]++;
	}
	else {
		op->operands[op->numInterm[i]] = app->indeterminateValue;
	}
	return 0;
}

int Find2(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	double value = *op->argPtr[i][0];
	if (value != value) {
		op->operands[op->numInterm[i]] = app->indeterminateValue;
		return 0;
	}
	LOC *location = (LOC *)(op->argPtr[i][1] + op->nOperands);
	if (app->arrays[location->num].type > 1) {
		app->BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	BOOL isDbl = (app->arrays[location->num].type == 0);
	// j and l are global
	int iDim = app->arrays[location->num].nDim - 1;
	int n = app->arrays[location->num].nElem[iDim] - location->offset;
	if (*op->argPtr[i][3] > 0.5) {
		int n1 = (int)(*op->argPtr[i][3] + 0.5);
		if (n1 < n) n = n1;
	}
	else if (*op->argPtr[i][3] == 0.5) n = 1;
	int nSwitch;
	if (*op->argPtr[i][4] >= 1.5) nSwitch = 0;
	else nSwitch = n + 1;
	int startPos = app->index_base ? (int)(*op->argPtr[i][2] - 0.5) : (int)(*op->argPtr[i][2] + 0.5);
	if (isDbl) {
		double *a = (double *)((unsigned long long)(*op->argPtr[i][1] + 0.5));
		op->operands[op->numInterm[i]] = Find(value, a, n, startPos, nSwitch);
	}
	else {
		float *a = (float *)((unsigned long long)(*op->argPtr[i][1] + 0.5));
		op->operands[op->numInterm[i]] = Find1(value, a, n, startPos, nSwitch);
	}
	if (app->index_base) op->operands[op->numInterm[i]]++;
	return 0;
}

int Compiler::Hist0(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i, int nMax, int keepInit)
{
	// j and l are global
	LOC *location = (LOC *)(op->argPtr[i][0] + op->nOperands);
	if (arrays[location->num].type > 1) {
		BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	unsigned long long arrayAddress = (unsigned long long)((*op->argPtr[i][0]) + 0.5);
	if (!arrayAddress) {
		ArrayError(2, i, aPtr2, location->num); return 1;
	}
	double *a1 = NULL;
	float *a2;
	if (!arrays[location->num].type) a1 = (double *)arrayAddress;
	else a2 = (float *)arrayAddress;
	int nD0 = arrays[location->num].nDim;
	int na = Size0(location);
	if (nMax < 1 || nMax > na) nMax = na;

	location = (LOC *)(op->argPtr[i][1] + op->nOperands);
	if (arrays[location->num].type > 1) {
		BuiltInFuncError(2, i, aPtr2, funcNum); return 1;
	}
	arrayAddress = (unsigned long long)((*op->argPtr[i][1]) + 0.5);
	if (!arrayAddress) {
		ArrayError(2, i, aPtr2, location->num); return 1;
	}
	double *b1 = NULL;
	float *b2;
	if (!arrays[location->num].type) b1 = (double *)arrayAddress;
	else b2 = (float *)arrayAddress;
	nD0 = arrays[location->num].nDim;
	int nb = Size0(location);

	location = (LOC *)(op->argPtr[i][2] + op->nOperands);
	if (location->num >= nArrays0) {
		ArrayError(3, i, aPtr2, location->num); return 1;
	}
	if (arrays[location->num].type && arrays[location->num].type != 2) {
		BuiltInFuncError(3, i, aPtr2, funcNum); return 1;
	}
	arrayAddress = (unsigned long long)((*op->argPtr[i][2]) + 0.5);
	if (!arrayAddress) {
		ArrayError(2, i, aPtr2, location->num); return 1;
	}
	double *c1 = NULL;
	int *c2;
	if (!arrays[location->num].type) c1 = (double *)arrayAddress;
	else c2 = (int *)arrayAddress;
	nD0 = arrays[location->num].nDim;
	int nc = Size0(location);
	if (nc < nb) nb = nc;

	int k, m;
	if (b1) {
		for (k = 1; k < nb && b1[k] > b1[k-1]; k++);
	}
	else {
		for (k = 1; k < nb && b2[k] > b2[k-1]; k++);
	}
	if (k < nb) {
		BuiltInFuncError(4, i, aPtr2, funcNum); return 1;
	}

	if (!keepInit) {
		if (c1) memset(c1, 0, nb * sizeof(double));
		else memset(c2, 0, nb * sizeof(int));
	}

	long n = 0;
	if (a1) {
		if (b1) {
			if (c1) {
				for (m = 0; m < nMax; m++) {
					for (k = 0; k < nb && b1[k] < a1[m]; k++);
					if (k < nb) {
						c1[k]++;
						n++;
					}
				}
			}
			else {
				for (m = 0; m < nMax; m++) {
					for (k = 0; k < nb && b1[k] < a1[m]; k++);
					if (k < nb) {
						c2[k]++;
						n++;
					}
				}
			}
		}
		else {
			if (c1) {
				for (m = 0; m < nMax; m++) {
					for (k = 0; k < nb && b2[k] < a1[m]; k++);
					if (k < nb) {
						c1[k]++;
						n++;
					}
				}
			}
			else {
				for (m = 0; m < nMax; m++) {
					for (k = 0; k < nb && b2[k] < a1[m]; k++);
					if (k < nb) {
						c2[k]++;
						n++;
					}
				}
			}
		}
	}
	else {
		if (b1) {
			if (c1) {
				for (m = 0; m < nMax; m++) {
					for (k = 0; k < nb && b1[k] < a2[m]; k++);
					if (k < nb) {
						c1[k]++;
						n++;
					}
				}
			}
			else {
				for (m = 0; m < nMax; m++) {
					for (k = 0; k < nb && b1[k] < a2[m]; k++);
					if (k < nb) {
						c2[k]++;
						n++;
					}
				}
			}
		}
		else {
			if (c1) {
				for (m = 0; m < nMax; m++) {
					for (k = 0; k < nb && b2[k] < a2[m]; k++);
					if (k < nb) {
						c1[k]++;
						n++;
					}
				}
			}
			else {
				for (m = 0; m < nMax; m++) {
					for (k = 0; k < nb && b2[k] < a2[m]; k++);
					if (k < nb) {
						c2[k]++;
						n++;
					}
				}
			}
		}
	}
	op->operands[op->numInterm[i]] = n;

	return 0;
}

int Hist(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	return app->Hist0(funcNum, aPtr2, tmpVarPtr, i, -1, 0);
}

int Hist2(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	int nMax = (int)((*op->argPtr[i][3]) + 0.5);
	int keepInit = (int)((*op->argPtr[i][4]) + 0.5);
	return app->Hist0(funcNum, aPtr2, tmpVarPtr, i, nMax, keepInit);
}

int Compiler::ReadArray0(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i, double nMax_dbl)
{
	if (!f_read) return -1;
	// j and l are global
	LOC *location = (LOC *)(op->argPtr[i][0] + op->nOperands);
	unsigned long long arrayAddress = (unsigned long long)((*op->argPtr[i][0]) + 0.5);
	if (!arrayAddress) {
		ArrayError(2, i, aPtr2, location->num); return 1;
	}
	void *a = (void *)arrayAddress;

	ARRAYDATA *arrayData = arrays + location->num;
	int nD0 = arrayData->nDim;
	int na = Size0(location);
	if (nMax_dbl < 1 || nMax_dbl > na) nMax_dbl = na;
	int nMax = (int)(nMax_dbl + 0.5);
	int type = arrayData->type;
	int tpsz = typeSize[type];
	char *curPos = (char *)(arrayData->elements);

	char line[1001], *ptr, *endPtr;
	line[1000] = '\0';
	int k = 0;
	line[0] = '\0';
	fgets(line, 1000, f_read);
	while(!feof(f_read) || line[0] != '\0') {
		ptr = strtok(line, " \t\r\n");
		while (ptr && k < nMax) {
			double value = strtod(ptr, &endPtr);
			if (*endPtr == ',') {
				*endPtr = '.';
				value = atof(ptr);
			}
			ArrayAssignment(curPos, type, value);
			curPos += tpsz;
			k++;
			ptr = strtok(NULL, " \t\r\n");
		}
		if (k == nMax) break;
		line[0] = '\0';
		fgets(line, 1000, f_read);
	}
	op->operands[op->numInterm[i]] = k;

	return 0;
}

int ReadArray(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	int rc = app->ReadArray0(funcNum, aPtr2, tmpVarPtr, i, -1.);
	if (rc == -1) {
		app->BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	return rc;
}

int ReadArray2(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	int rc = app->ReadArray0(funcNum, aPtr2, tmpVarPtr, i, *op->argPtr[i][1]);
	if (rc == -1) {
		app->BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	return rc;
}

int Sort(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	// j and l are global
	LOC *location = (LOC *)(op->argPtr[i][0] + op->nOperands);
	void *arrayAddress = (void *)((unsigned long long)((*op->argPtr[i][0]) + 0.5));
	if (!arrayAddress) {
		app->ArrayError(2, i, aPtr2, location->num); return 1;
	}
	int type = app->arrays[location->num].type;
	size_t nElem = app->Size0(location);
	qsort(arrayAddress, nElem, app->typeSize[type], Compare[type]);
	return 0;
}

void Compiler::quickSort2(int array_size, int type, void *numbers, int permut[])
{
	if (index_base) {
		for (int i = 0; i < array_size; i++) permut[i] = i + 1;
	}
	else {
		for (int i = 0; i < array_size; i++) permut[i] = i;
	}
	q_sort2(type, numbers, 0, array_size - 1, permut);
}
void Compiler::q_sort2(int type, void *numbers, int left, int right, int permut[])
{
	int pivot, l_hold, r_hold, pivot_p;
	double pivot_num;
	void *pivot_addr = &pivot_num;
	char *arr = (char *)numbers;
	int typeSize = app->typeSize[type];
	l_hold = left;
	r_hold = right;
	memcpy(pivot_addr, arr + left * typeSize, typeSize);
	pivot_p = permut[left];
	while (left < right) {
		while ((Compare[type](arr + right * typeSize, pivot_addr) >= 0) && (left < right)) right--;
		if (left != right) {
			memcpy(arr + left * typeSize, arr + right * typeSize, typeSize);
			permut[left] = permut[right];
			left++;
		}
		while ((Compare[type](arr + left * typeSize, pivot_addr) <= 0) && (left < right)) left++;
		if (left != right) {
			memcpy(arr + right * typeSize, arr + left * typeSize, typeSize);
			permut[right] = permut[left];
			right--;
		}
	}
	memcpy(arr + left * typeSize, pivot_addr, typeSize);
	permut[left] = pivot_p;
	pivot = left;
	left = l_hold;
	right = r_hold;
	if (left < pivot) q_sort2(type, numbers, left, pivot-1, permut);
	if (right > pivot) q_sort2(type, numbers, pivot+1, right, permut);
}

int Sort2(int funcNum, PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i)
{
	// j and l are global
	LOC *location = (LOC *)(op->argPtr[i][0] + op->nOperands);
	void *arrayAddress1 = (void *)((unsigned long long)((*op->argPtr[i][0]) + 0.5));
	if (!arrayAddress1) {
		app->ArrayError(2, i, aPtr2, location->num); return 1;
	}
	int array_size = app->Size0(location);
	int type = app->arrays[location->num].type;

	location = (LOC *)(op->argPtr[i][1] + op->nOperands);
	if (app->arrays[location->num].type != 2) {
		app->BuiltInFuncError(1, i, aPtr2, funcNum); return 1;
	}
	int *arrayAddress2 = (int *)((unsigned long long)((*op->argPtr[i][1]) + 0.5));
	if (!arrayAddress2) {
		app->ArrayError(2, i, aPtr2, location->num); return 1;
	}
	if (app->Size0(location) < array_size) {
		app->BuiltInFuncError(2, i, aPtr2, funcNum); return 1;
	}

	app->quickSort2(array_size, type, arrayAddress1, arrayAddress2);
	return 0;
}

int iReturnFunc;

void Compiler::DefineAdvancedStdFunc()
{
	advancedStdFunc[NSTDFUNC0] = Loc;
	advancedStdFunc[NSTDFUNC0+1] = Invert;
	advancedStdFunc[NSTDFUNC0+2] = Size;
	advancedStdFunc[NSTDFUNC0+3] = Size2;
	advancedStdFunc[NSTDFUNC0+4] = Int_Inti;
	advancedStdFunc[NSTDFUNC0+5] = Int_Inti;
	advancedStdFunc[NSTDFUNC0+6] = Int_Inti;
	advancedStdFunc[NSTDFUNC0+7] = Int_Inti;
	advancedStdFunc[NSTDFUNC0+8] = Intw;
	advancedStdFunc[NSTDFUNC0+9] = Intw;
	advancedStdFunc[NSTDFUNC0+10] = Sum;
	advancedStdFunc[NSTDFUNC0+11] = Sum2;
	advancedStdFunc[NSTDFUNC0+12] = Iter;
	advancedStdFunc[NSTDFUNC0+13] = Root;
	advancedStdFunc[NSTDFUNC0+14] = Root2;
	iReturnFunc = NSTDFUNC0 + 15;
	advancedStdFunc[iReturnFunc] = NULL; // Return() (defined in RunProgram1())
	advancedStdFunc[NSTDFUNC0+16] = Find;
	advancedStdFunc[NSTDFUNC0+17] = Find2;
	advancedStdFunc[NSTDFUNC0+18] = Hist;
	advancedStdFunc[NSTDFUNC0+19] = Hist2;
	advancedStdFunc[NSTDFUNC0+20] = ReadArray;
	advancedStdFunc[NSTDFUNC0+21] = ReadArray2;
	advancedStdFunc[NSTDFUNC0+22] = Sort;
	advancedStdFunc[NSTDFUNC0+23] = Sort2;
}

void Compiler::RunProgram1(PROGRAM_ARGS2 *aPtr2, PROGRAM_TEMPVARS *tmpVarPtr, int i, int iMax)
{

	int k; // j and l are global
	double currentValue;
	unsigned short off0;
	void *address;
	int iArray_mod;
	long arrayType;
	int funcType, funcNum, nD;
	unsigned long nEl1;
	unsigned long long *addr;

	BOOL delete_a2;
	if (!aPtr2) {
		aPtr2 = new PROGRAM_ARGS2;
		aPtr2->iFunc = aPtr1->iFunc;
		aPtr2->eq = aPtr1->eq;
		aPtr2->op = aPtr1->op;
		aPtr2->op0 = aPtr1->op;
		delete_a2 = TRUE;
	}
	else delete_a2 = FALSE;

	op = aPtr2->op;

	op->callDepth++;
	if (op->callDepth == 1 && op->curOp >= 0 && op->errorFunc < -1) {
		op->errorFunc = -1;
		op->interruptOp = op->curOp;
	}

	BOOL delete_tmpVar;
	if (!tmpVarPtr) {
		tmpVarPtr = new PROGRAM_TEMPVARS;
		tmpVarPtr->maxIntDepth = 0;
		tmpVarPtr->curIntDepth = -1;
		tmpVarPtr->prevIntDepth = -1;
		tmpVarPtr->curRootDepth = -1;
		tmpVarPtr->argCount = 0;
		tmpVarPtr->base = op->base;
		if (aPtr2->iFunc >= 0) {
			tmpVarPtr->argCount = subroutineArgInfo[aPtr2->iFunc].count;
			if (tmpVarPtr->argCount) {
				tmpVarPtr->arg_info = new PROGRAM_ARGINFO[tmpVarPtr->argCount];
			}
			for (j = 0; j < tmpVarPtr->argCount; j++) {
				BYTE argType = subroutineArgInfo[aPtr2->iFunc].type[j];
				if (!argType) {
					tmpVarPtr->arg_info[j].objType = 0;
					op->operands[NSYSTEMVARS+j] = aPtr2->arg1[j];
				}
				else if (argType == 1) {
					tmpVarPtr->arg_info[j].objType = 1;
					tmpVarPtr->arg_info[j].a = (unsigned long)(aPtr2->arg1[j] + 0.5);
					op->operands[NSYSTEMVARS+j] = *((double *)tmpVarPtr->arg_info[j].a);
				}
				else {
					op->operands[NSYSTEMVARS+j] = aPtr2->arg1[j];
					LOC *location = aPtr2->arg2[j];
					tmpVarPtr->arg_info[j].objType = location->type_offset0 / 8192;
					tmpVarPtr->arg_info[j].objNum = location->num;
					if (tmpVarPtr->arg_info[j].objType == 2) {
						tmpVarPtr->arg_info[j].offset0 = location->type_offset0 - 8192 * 2;
						tmpVarPtr->arg_info[j].a = (unsigned long)(aPtr2->arg1[j] + 0.5);
						ARRAYDATA *arr = arrays + location->num;
						BYTE *addr0 = (BYTE *)(arr->elements);
						int nDim0 = arr->nDim;
						tmpVarPtr->arg_info[j].a2 = location->offset;
						off0 = tmpVarPtr->arg_info[j].offset0;
						tmpVarPtr->arg_info[j].nElem1 = arr->nElem[off0] - tmpVarPtr->arg_info[j].a2;
						tmpVarPtr->arg_info[j].nDim = nDim0 - off0;
						*((LOC *)(op->operands + NSYSTEMVARS + j + op->nOperands)) = *location;
					}
				}
			}
		}
		tmpVarPtr->nFunc1 = NSTDFUNC + tmpVarPtr->argCount + nSubroutines;
		tmpVarPtr->nFunc2 = tmpVarPtr->nFunc1 + nDllFunc;
		delete_tmpVar = TRUE;
	}
	else delete_tmpVar = FALSE;

	if (i >= 0) goto ENTRY;
	tmpVarPtr->nextControl = 0;
	currentValue = op->operands[0];
	for (i = 0; i < op->nInterm && tmpVarPtr->nextControl < op->nControl || tmpVarPtr->nextControl < op->nControl - 1; i++) {
		if (aPtr1->terminate) {
			aPtr1->calculationInProgress = FALSE;
			_endthread();
		}
		if (i < op->control[tmpVarPtr->nextControl]) {
ENTRY:
			BOOL isFormalParm = FALSE;
			if (aPtr1->terminate) {
				aPtr1->calculationInProgress = FALSE;
				_endthread();
			}
			if (i < op->nInterm) op->curOp = i;
			int argNum = -1;
			if (op->funcNum[i] < NSTDFUNC) {
				funcType = 3;
				funcNum = op->funcNum[i];
			}
			else if (op->funcNum[i] < NSTDFUNC + tmpVarPtr->argCount) {
				argNum = op->funcNum[i] - NSTDFUNC;
				funcType = tmpVarPtr->arg_info[argNum].objType;
				funcNum = tmpVarPtr->arg_info[argNum].objNum;
				addr = (unsigned long long *)(tmpVarPtr->arg_info[argNum].a);
				nD = tmpVarPtr->arg_info[argNum].nDim;
				off0 = tmpVarPtr->arg_info[argNum].offset0;
				nEl1 = tmpVarPtr->arg_info[argNum].nElem1;
				isFormalParm = TRUE;
			}
			else if (op->funcNum[i] < tmpVarPtr->nFunc1) {
				funcType = 4;
				funcNum = op->funcNum[i] - NSTDFUNC - tmpVarPtr->argCount;
			}
			else if (op->funcNum[i] < tmpVarPtr->nFunc2) {
				funcType = 5;
				funcNum = op->funcNum[i] - tmpVarPtr->nFunc1;
			}
			else {
				funcType = 2;
				funcNum = op->funcNum[i] - tmpVarPtr->nFunc2;
				addr = (unsigned long long *)arrays[funcNum].elements;
				nD = arrays[funcNum].nDim;
				off0 = 0;
				nEl1 = arrays[funcNum].nElem[0];
			}
			if (funcType == 3) {
				if (funcNum < NSTDFUNC0) {
					op->operands[op->numInterm[i]] = stdFunc[funcNum](op->argPtr[i]);
				}
				else if (funcNum >= NSTDFUNC0 + 4 && funcNum <= NSTDFUNC0 + 14) { // Integration, summation, iteration and root finding
					if (tmpVarPtr->curIntDepth >= MAXINTDEPTH - 1) {
						Error(i, aPtr2, 6); goto RETURN2;
					}
					tmpVarPtr->curIntDepth++;
					int iNew;
					if (tmpVarPtr->curIntDepth > tmpVarPtr->prevIntDepth) {
						if (tmpVarPtr->curIntDepth == tmpVarPtr->maxIntDepth) {
							tmpVarPtr->nextCtrl[tmpVarPtr->curIntDepth] = (int *)malloc(sizeof(int));
							tmpVarPtr->iEnd[tmpVarPtr->curIntDepth] = (int *)malloc(sizeof(int));
							tmpVarPtr->maxIntDepth++;
						}
						tmpVarPtr->nInt[tmpVarPtr->curIntDepth] = 1;
						tmpVarPtr->iInt[tmpVarPtr->curIntDepth] = 0;
						tmpVarPtr->nextCtrl[tmpVarPtr->curIntDepth][0] = tmpVarPtr->nextControl;
						tmpVarPtr->iEnd[tmpVarPtr->curIntDepth][0] = i - 1;
						iNew = i;
					}
					else if (tmpVarPtr->curIntDepth == tmpVarPtr->prevIntDepth) {
						// possible only when tmpVarPtr->curIntDepth is equal to the maximum value
						while (tmpVarPtr->nextControl > 0 && op->control[tmpVarPtr->nextControl-1] > i - 1) tmpVarPtr->nextControl--;
						tmpVarPtr->nextCtrl[tmpVarPtr->curIntDepth][0] = tmpVarPtr->nextControl;
						tmpVarPtr->iEnd[tmpVarPtr->curIntDepth][0] = i - 1;
						iNew = i;
					}
					else {
						int nInt = tmpVarPtr->nInt[tmpVarPtr->curIntDepth];
						int iInt = tmpVarPtr->iInt[tmpVarPtr->curIntDepth];
						if (iInt < nInt - 1) {
							tmpVarPtr->iInt[tmpVarPtr->curIntDepth]++;
						}
						else if (tmpVarPtr->iEnd[tmpVarPtr->curIntDepth][iInt] < i - 1) {
							tmpVarPtr->nInt[tmpVarPtr->curIntDepth]++;
							tmpVarPtr->iInt[tmpVarPtr->curIntDepth]++;
							tmpVarPtr->nextCtrl[tmpVarPtr->curIntDepth] = (int *)realloc(tmpVarPtr->nextCtrl[tmpVarPtr->curIntDepth], tmpVarPtr->nInt[tmpVarPtr->curIntDepth] * sizeof(int));
							tmpVarPtr->iEnd[tmpVarPtr->curIntDepth] = (int *)realloc(tmpVarPtr->iEnd[tmpVarPtr->curIntDepth], tmpVarPtr->nInt[tmpVarPtr->curIntDepth] * sizeof(int));
							tmpVarPtr->nextCtrl[tmpVarPtr->curIntDepth][tmpVarPtr->iInt[tmpVarPtr->curIntDepth]] = tmpVarPtr->nextControl;
							tmpVarPtr->iEnd[tmpVarPtr->curIntDepth][tmpVarPtr->iInt[tmpVarPtr->curIntDepth]] = i - 1;
						}
						else {
							tmpVarPtr->iInt[tmpVarPtr->curIntDepth] = 0;
						}
						tmpVarPtr->nextControl = tmpVarPtr->nextCtrl[tmpVarPtr->curIntDepth][tmpVarPtr->iInt[tmpVarPtr->curIntDepth]];
						iNew = tmpVarPtr->iEnd[tmpVarPtr->curIntDepth][tmpVarPtr->iInt[tmpVarPtr->curIntDepth]] + 1;
					}
					tmpVarPtr->prevIntDepth = tmpVarPtr->curIntDepth;
					int rc = advancedStdFunc[funcNum](funcNum, aPtr2, tmpVarPtr, iNew);
					tmpVarPtr->curIntDepth--;
					if (rc) goto RETURN2;
				}
				else if (funcNum != iReturnFunc) {
					if (advancedStdFunc[funcNum](funcNum, aPtr2, tmpVarPtr, i)) goto RETURN2;
				}
				else { // Return
					currentValue = *(op->argPtr[i][0]);
					goto RETURN;
				}
				currentValue = op->operands[op->numInterm[i]];
				if (op->isArrayAssignment[i]) {
					if (iArray_mod < nArrays0) ArrayAssignment(address, arrayType, currentValue);
					else {
						ArrayError(3, i, aPtr2, iArray_mod); goto RETURN2;
					}
				}
				else if (funcNum == 5) {
					(*op->argPtr[i][0]) = currentValue;
					if (tmpVarPtr->argCount) {
						__int64 argAddr_offset = op->argPtr[i][0] - op->operands;
						if (argAddr_offset >= NSYSTEMVARS && argAddr_offset < NSYSTEMVARS + tmpVarPtr->argCount) *((double *)tmpVarPtr->arg_info[argAddr_offset-NSYSTEMVARS].a) = currentValue;
					}
				}
			}
			else if (funcType == 4) {
				PROGRAM_ARGS2 *_aPtr2 = new PROGRAM_ARGS2;
				_aPtr2->iFunc = funcNum;
				_aPtr2->eq = subroutine + funcNum;
				_aPtr2->op = subroutine2 + funcNum;
				_aPtr2->op0 = op;
				int argCount = subroutineArgInfo[funcNum].count;
				for (j = 0; j < argCount; j++) {
					_aPtr2->arg1[j] = *(op->argPtr[i][j]);
					BYTE type = subroutineArgInfo[funcNum].type[j];
					if (type > 1) _aPtr2->arg2[j] = (LOC *)(op->argPtr[i][j] + op->nOperands);
				}
				int errorFunc = op->errorFunc;
				if (callDepth >= MAXNESTINGLEVEL) {
					Error(0, _aPtr2, 4);
					delete _aPtr2;
					if (delete_a2) delete aPtr2;
					return;
				}
				callDepth++;
				RunProgram1(_aPtr2); // op may be changed in RunProgram1
				op = aPtr2->op;
				op->curOp = i;
				delete _aPtr2;
				callDepth--;
				if (op->errorFunc >= -1) {
					if (errorFunc < -1) op->runtimeErrLineNum = op->lineNum[i];
					if (aPtr2->iFunc >= 0 && aPtr2->op0->errorFunc < -1) {
						aPtr2->op0->errorType = op->errorType;
						if (op->errorType == 1) aPtr2->op0->arrayError = op->arrayError;
						else if (op->errorType == 2) aPtr2->op0->builtInFuncError = op->builtInFuncError;
						else if (op->errorType == 3) aPtr2->op0->runtimeErrDll = op->runtimeErrDll;
						aPtr2->op0->runtimeErrCode = op->runtimeErrCode;
						aPtr2->op0->errorFunc = op->errorFunc;
					}
				}
				op->operands[op->numInterm[i]] = aPtr1->retVal;
				currentValue = op->operands[op->numInterm[i]];
				if (op->isArrayAssignment[i]) {
					if (iArray_mod < nArrays0) ArrayAssignment(address, arrayType, currentValue);
					else {
						ArrayError(3, i, aPtr2, iArray_mod); goto RETURN2;
					}
				}
			}
/*			else if (funcType == 5) {
				long _ier = dllFuncArgCount[funcNum];
				BOOL unspecifiedError = FALSE;
				try {
					op->operands[op->numInterm[i]] = CallUserDLL(userDllFuncAddr[funcNum], op, i, &_ier, isFormalParm);
				}
				catch(...) {
					unspecifiedError = TRUE;
				}
				currentValue = op->operands[op->numInterm[i]];
				if (_ier != dllFuncArgCount[funcNum] || unspecifiedError) {
					DllError(_ier, i, aPtr2, funcNum); goto RETURN2;
				}
				if (op->isArrayAssignment[i]) {
					if (iArray_mod < nArrays0) ArrayAssignment(address, arrayType, currentValue);
					else {
						ArrayError(3, i, aPtr2, iArray_mod); goto RETURN2;
					}
				}
			}*/
			else {
				if (!addr) {
					ArrayError(2, i, aPtr2, funcNum); goto RETURN2;
				}
				double offset = 0.;
				long offset1 = 0;
				int maxDim;
				BOOL isAddress = FALSE;
				int nIndices;
				if (*op->argPtr[i][nD] >= 0.) {
					nIndices = (int)(*op->argPtr[i][nD] + 0.5);
					maxDim = nIndices - 1;
					isAddress = TRUE;
				}
				else maxDim = nD - 1;
				long type = arrays[funcNum].type;
				for (j = 0; j <= maxDim; j++) {
					if (index_base) offset = *op->argPtr[i][j] - 0.5;
					else offset = *op->argPtr[i][j] + 0.5;
					if (offset < 0. || !j && offset >= nEl1 || j && offset >= arrays[funcNum].nElem[off0+j]) {
						ArrayError(1, i, aPtr2, funcNum); goto RETURN2;
					}
					if (arrays[funcNum].stepSize[off0+j]) offset1 += (long)offset * arrays[funcNum].stepSize[off0+j];
					else {
						offset1 = (long)offset;
						if (j < maxDim) {
							addr += offset1;
							addr = (unsigned long long *)(*addr);
							if (!addr) {
								ArrayError(1, i, aPtr2, funcNum); goto RETURN2;
							}
						}
					}
				}
				if (isAddress) {
					arrayType = type;
					if (arrays[funcNum].stepSize[off0] || nIndices == nD) addr = (unsigned long long *)AddressAssignment(addr, arrayType, offset1);
					else addr += offset1;
					op->operands[op->numInterm[i]] = (double)(unsigned long long)addr;
					currentValue = op->operands[op->numInterm[i]];
					LOC *location2 = (LOC *)(op->operands + op->numInterm[i] + op->nOperands);
					location2->num = (unsigned short)funcNum;
					if (nIndices > 1) off0 += (nIndices - 1);
					location2->type_offset0 = 2 * 8192 + off0;
					location2->offset = (unsigned long)offset;
					if (nIndices < 2) location2->offset += (arrays[funcNum].nElem[off0] - nEl1);
				}
				else {
					if (type == 0) op->operands[op->numInterm[i]] = *((double *)addr + offset1);
					else if (type == 1) op->operands[op->numInterm[i]] = *((float *)addr + offset1);
					else if (type == 2) op->operands[op->numInterm[i]] = *((long *)addr + offset1);
					else if (type == 3) op->operands[op->numInterm[i]] = *((unsigned long *)addr + offset1);
					else if (type == 4) op->operands[op->numInterm[i]] = *((short *)addr + offset1);
					else if (type == 5) op->operands[op->numInterm[i]] = *((unsigned short *)addr + offset1);
					else if (type == 6) op->operands[op->numInterm[i]] = *((char *)addr + offset1);
					else if (type == 7) op->operands[op->numInterm[i]] = *((unsigned char *)addr + offset1);
					currentValue = op->operands[op->numInterm[i]];
					if (op->isAddress[i]) {
						arrayType = type;
						address = AddressAssignment(addr, arrayType, offset1);
						iArray_mod = funcNum;
					}
					else if (op->isArrayAssignment[i]) {
						if (iArray_mod < nArrays0) ArrayAssignment(address, arrayType, currentValue);
						else {
							ArrayError(3, i, aPtr2, iArray_mod); goto RETURN2;
						}
					}
				}
			}
			if (i < iMax) {
				i++;
				if (i == op->control[tmpVarPtr->nextControl]) {
					i = op->transferTo[tmpVarPtr->nextControl];
					tmpVarPtr->nextControl = op->newNextControl[tmpVarPtr->nextControl];
				}
				goto ENTRY;
			}
			else if (iMax >= 0) {
				op->callDepth--;
				op->curOp = i + 1;
				aPtr1->retVal = currentValue;
				return;
			}
			continue;
		}
		else if (op->isConditional[tmpVarPtr->nextControl] && op->operands[op->numInterm[i-1]]) {
			tmpVarPtr->nextControl++;
			i--;
			if (i < op->nInterm) op->curOp = i;
		}
		else {
			i = op->transferTo[tmpVarPtr->nextControl] - 1;
			if (op->newNextXbreak[tmpVarPtr->nextControl] < 0) {
				i++;
				tmpVarPtr->nextControl = op->newNextControl[tmpVarPtr->nextControl];
				if (i < op->nInterm) op->curOp = i;
				goto ENTRY;
			}
			tmpVarPtr->nextControl = op->newNextControl[tmpVarPtr->nextControl];
			if (i < op->nInterm) op->curOp = i;
		}
	}
RETURN:
	op->callDepth--;
	op->curOp = -1;
	aPtr1->retVal = currentValue;
RETURN2:
	if (delete_a2) delete aPtr2;
	if (delete_tmpVar) {
		if (tmpVarPtr->argCount) delete tmpVarPtr->arg_info;
		for (k = tmpVarPtr->maxIntDepth - 1; k >= 0; k--) {
			free(tmpVarPtr->nextCtrl[k]);
			free(tmpVarPtr->iEnd[k]);
		}
		delete tmpVarPtr;
	}
}

void Compiler::ArrayError(int code, int i, PROGRAM_ARGS2 *aPtr2, long iArray)
{
	if (aPtr2->op->errorFunc < -1) {
		aPtr2->op->runtimeErrCode = code;
		aPtr2->op->errorType = 1;
		aPtr2->op->arrayError = iArray + 1;
		aPtr2->op->errorFunc = aPtr2->iFunc;
		aPtr2->op->runtimeErrLineNum = aPtr2->op->lineNum[i];
		aPtr2->op->curOp = -1;
	}
	if (aPtr2->iFunc >= 0 && aPtr2->op0->errorFunc < -1) {
		aPtr2->op0->runtimeErrCode = code;
		aPtr2->op0->errorType = 1;
		aPtr2->op0->arrayError = iArray + 1;
		aPtr2->op0->errorFunc = aPtr2->iFunc;
	}
	aPtr2->op->callDepth--;
	aPtr1->retVal = indeterminateValue;
}

void Compiler::BuiltInFuncError(int code, int i, PROGRAM_ARGS2 *aPtr2, long iBuiltInFunc)
{
	if (aPtr2->op->errorFunc < -1) {
		aPtr2->op->runtimeErrCode = code;
		aPtr2->op->errorType = 2;
		aPtr2->op->builtInFuncError = iBuiltInFunc + 1;
		aPtr2->op->errorFunc = aPtr2->iFunc;
		aPtr2->op->runtimeErrLineNum = aPtr2->op->lineNum[i];
		aPtr2->op->curOp = -1;
	}
	if (aPtr2->iFunc >= 0 && aPtr2->op0->errorFunc < -1) {
		aPtr2->op0->runtimeErrCode = code;
		aPtr2->op0->errorType = 2;
		aPtr2->op0->builtInFuncError = iBuiltInFunc + 1;
		aPtr2->op0->errorFunc = aPtr2->iFunc;
	}
	aPtr2->op->callDepth--;
	aPtr1->retVal = indeterminateValue;
}
/*
void Compiler::DllError(int code, int i, PROGRAM_ARGS2 *aPtr2, long iDllFunc)
{
	if (aPtr2->op->errorFunc < -1) {
		aPtr2->op->runtimeErrCode = code;
		aPtr2->op->errorType = 3;
		aPtr2->op->runtimeErrDll = iDllFunc + 1;
		aPtr2->op->errorFunc = aPtr2->iFunc;
		aPtr2->op->runtimeErrLineNum = aPtr2->op->lineNum[i];
		aPtr2->op->curOp = -1;
	}
	if (aPtr2->iFunc >= 0 && aPtr2->op0->errorFunc < -1) {
		aPtr2->op0->runtimeErrCode = code;
		aPtr2->op0->errorType = 3;
		aPtr2->op0->runtimeErrDll = iDllFunc + 1;
		aPtr2->op0->errorFunc = aPtr2->iFunc;
	}
	aPtr2->op->callDepth--;
	aPtr1->retVal = indeterminateValue;
}
*/
void Compiler::Error(int i, PROGRAM_ARGS2 *aPtr2, int errorType) // OPDATA::errorType > 3
{
	if (aPtr2->op->errorFunc < -1) {
		aPtr2->op->errorType = errorType;
		aPtr2->op->errorFunc = aPtr2->iFunc;
		aPtr2->op->runtimeErrLineNum = aPtr2->op->lineNum[i];
		aPtr2->op->curOp = -1;
	}
	if (aPtr2->iFunc >= 0 && aPtr2->op0->errorFunc < -1) {
		aPtr2->op0->errorType = errorType;
		aPtr2->op0->errorFunc = aPtr2->iFunc;
	}
	aPtr2->op->callDepth--;
	aPtr1->retVal = indeterminateValue;
}

void ArrayAssignment(void *ptr, long type, double value)
{
	if (type == 0) *((double *)ptr) = value;
	else if (type == 1) *((float *)ptr) = (float)value;
	else if (type == 2) *((long *)ptr) = (long)value;
	else if (type == 3) *((unsigned long *)ptr) = (unsigned long)value;
	else if (type == 4) *((short *)ptr) = (short)value;
	else if (type == 5) *((unsigned short *)ptr) = (unsigned short)value;
	else if (type == 6) *((char *)ptr) = (char)value;
	else if (type == 7) *((unsigned char *)ptr) = (unsigned char)value;
}

void* AddressAssignment(unsigned long long *ptr, long type, long offset)
{
	if (type == 0) return (double *)ptr + offset;
	else if (type == 1) return (float *)ptr + offset;
	else if (type == 2) return (long *)ptr + offset;
	else if (type == 3) return (unsigned long *)ptr + offset;
	else if (type == 4) return (short *)ptr + offset;
	else if (type == 5) return (unsigned short *)ptr + offset;
	else if (type == 6) return (char *)ptr + offset;
	else if (type == 7) return (unsigned char *)ptr + offset;
	else return NULL;
}

Array_int symbolType; // 0 - operator or separator, 1 - function or array, 2 - constant, 3 -identifier, 4 - intermediate result (number),
					  // 5 to 90 + MAXARGS - address returned by "loc(id)": 5 - address of a variable,
					  // 6 to 5 + MAXARGS - address of an array with 1 to MAXARGS dimensions, 6 + MAXARGS to 90 + MAXARGS - address of one of 85 types of functions (up to 3 arguments, each of them can be one of 4 types: variable, address of a variable, one-dimensional array or two-dimensional array)
Array_int subType; // operator or separator: 0 - 18 - operators, 19 - comma, 20 - left parenthesis or bracket, 21 - right parenthesis or bracket; functions - see above
Array_double constants;

Array_int stack_symbolType; // 0 - operator or separator, 1 - function or array, 2 - constant, 3 - identifier, 4 - intermediate result (number),
							// 5 to 90 + MAXARGS - address returned by "loc(id)": 5 - address of a variable,
							// 6 to 5 + MAXARGS - address of an array with 1 to MAXARGS dimensions, 6 + MAXARGS to 90 + MAXARGS - address of one of 85 types of functions (up to 3 arguments, each of them can be one of 4 types: variable, address of a variable, one-dimensional array or two-dimensional array)
Array_int stack_subType; // operator or separator: 0 - 18 - operators, 19 - comma, 20 - left parenthesis or bracket, 21 - right parenthesis or bracket; functions - see above
Array_double stack_constants;

int addType;

BOOL OnComma(int &sl, int &nOut);
int FindSign(char *string, int j);

char *is_number(char *c, double &num, BOOL insertNull = TRUE);
int op_preced(const int c);
bool op_left_assoc(const int c);
bool is_function(char *c, int &id, int iFunc = -1, char **argNames = NULL);
bool is_operator(char c, int &opId, char *&pos);
bool is_ident(char *c, char *ident);

int Compiler::FormalParameters(int iFunc)
{
	String code = (char *)(subroutineCode[iFunc]);
	EQDATA *eq = subroutine + iFunc;
	OPDATA *op = subroutine2 + iFunc;
	char tmp[2001], testing[STRLEN_MAX+1], parmName[MAXIDENTLEN+1], id[STRLEN_MAX+1];
	String argListStr, argListStr_singleLine;
	__int64 i, j;
	int k, iLine, nConf = 0;
	int *typeConf = (int *)malloc(0);
	int *numConf = (int *)malloc(0);
	char **nameConf = (char **)malloc(0);
	char *nameConfBuffer = (char *)malloc(0);
	int argCount = 0;
	BYTE argType[MAXARGS];
	BOOL error = FALSE;
	BOOL commentLineAtTop = FALSE;
	int codeLength = code.GetLength();
	int bufferLength = codeLength + 10 + MAXIDENTLEN;
	char *buffer = new char[bufferLength];
	strcpy(buffer, code.GetBuffer()); code.ReleaseBuffer();
	LoadString(resInst, 2532, testing, STRLEN_MAX);
	argListStr = "";
	argListStr_singleLine = "";
	Array_String argNames;
	argNames.SetSize(0);
	int emptyCount = 0, commaInsertPos = 0;
	char *string = buffer;
	char *stringEnd = strchr(string, 0x0d);
	__int64 len = strlen(buffer);
	iLine = -1;
	char *subroutineBody = NULL;
	while(stringEnd || string - buffer < len) {
		iLine++;
		if (stringEnd) *stringEnd = '\0';
		char comment[2001];
		size_t nBlanks = strspn(string, " \t");

		comment[0] = '\0';
		char *ptr = strchr(string + nBlanks, '/');
		while(ptr) {
			if (*(ptr + 1) == '/') {
				*ptr = '\0';
				if (*(ptr + 2) != '\0') strcpy(comment, ptr + 2);
				break;
			}
			ptr = strchr(ptr + 1, '/');
		}

		__int64 l, depth = 1;
		__int64 len0 = strlen(string);
		int nTrailingBlanks = 0;
		for (l = nBlanks; l < len0; l++) {
			if (string[l] == '(') {
				depth++;
				nTrailingBlanks = 0;
			}
			else if (string[l] == ')') {
				depth--;
				nTrailingBlanks = 0;
				if (!depth) break;
			}
			else if (string[l] == ' ' || string[l] == '\t') nTrailingBlanks++;
			else nTrailingBlanks = 0;
		}
		if (!depth) {
			string[l] = '\0';
			len0 = l;
			subroutineBody = string + l + 1;
		}
		else if (nTrailingBlanks) {
			len0 -= nTrailingBlanks;
			string[len0] = '\0';
//			memset(string + len0, 0, nTrailingBlanks);
		}
		if (len0 > (__int64)nBlanks + 1 && (string[nBlanks] != '/' || string[nBlanks+1] != '/') || len0 == nBlanks + 1) {

			int argCount_line = 0;
			ptr = strtok(string, ",");
			while(ptr) {

				int addrType = 0; // 0 - no address, 1 - address of a variable, 2 - array, 3 - subroutine
				char *testStr = ptr + strspn(ptr, " \t");
				char *testStr2;
				if (!strnicmp(testStr, "loc", 3)) {
					testStr += 3;
					if (*testStr != '\0') {
						testStr2 = testStr + strspn(testStr, " \t");
						if (*testStr2 == '(') {
							addrType = 1;
						}
					}
					if (addrType) {
						testStr2++;
						__int64 len = strlen(testStr2);
						for (j = 0; j < len && testStr2[j] != ')' && testStr2[j] != '[' && testStr2[j] != '('; j++);
						if (testStr2[j] == ')') {
							addrType = 1;
						}
						else if (testStr2[j] == '[') {
							addrType = 2;
						}
						else if (testStr2[j] == '(') {
							addrType = 3;
						}
						else {
							error = TRUE;
							LoadString(resInst, 2704, eq->message, STRLEN_MAX);
							goto END;
						}
					}
				}

				if (!addrType) {
					char *nextParm = ptr + strspn(ptr, " \t");
					strncpy(parmName, nextParm, MAXIDENTLEN);
					parmName[MAXIDENTLEN] = '\0';
				}
				else {
					nBlanks = strspn(testStr2, " \t");
					i = j - nBlanks;
					char *nextParm = testStr2 + nBlanks;
					if (i < MAXIDENTLEN) {
						strncpy(parmName, nextParm, (size_t)i);
						parmName[i] = '\0';
					}
					else {
						strncpy(parmName, nextParm, MAXIDENTLEN);
						parmName[MAXIDENTLEN] = '\0';
					}
				}

				int nSameName = argCount;
				error = !CheckIdentifier(parmName, 0, nSameName, testing, nConf, typeConf, numConf, nameConf, nameConfBuffer, eq->message);
				if (error) goto END;
				if (!strcmpi(parmName, eq->name)) {
					error = TRUE;
					LoadString(resInst, 2678, eq->message, STRLEN_MAX);
					goto END;
				}
				for (k = 0; k < argCount && strcmpi(argNames[k], parmName); k++);
				if (k < argCount) {
					error = TRUE;
					LoadString(resInst, 2669, S, STRLEN_MAX);
					sprintf(eq->message, S, parmName);
					goto END;
				}

				if (!argCount_line) {
					if (commaInsertPos) {
						argListStr.Insert(commaInsertPos, ",");
						commaInsertPos = 0;
						argListStr += "\r\n";
					}
					if (emptyCount) argListStr += "\r\n";
					if (!argCount && commentLineAtTop) argListStr += "\r\n";
				}
				else argListStr += ", ";
				if (addrType) argListStr += "loc(";
				argListStr += parmName;

				if (!argCount) argListStr_singleLine = "";
				else argListStr_singleLine += ", ";
				if (addrType) argListStr_singleLine += "loc(";
				argListStr_singleLine += parmName;

				argNames.Add(parmName);

				char *c;

				if (!addrType) {
					argType[argCount] = 0;
					__int64 len = strlen(testStr);
					c = testStr + len;
					if (c - string < len0) *c = ',';
				}
				else if (addrType == 1) {
					argType[argCount] = 1;
					argListStr += ")";
					argListStr_singleLine += ")";
					__int64 len = strlen(testStr);
					c = testStr + len;
					if (c - string < len0) *c = ',';
				}
				else if (addrType == 2) {
					argListStr += "[";
					argListStr_singleLine += "[";
					testStr2 += j;
					__int64 len = strlen(testStr2);
					if (testStr2 + len - string < len0) testStr2[len] = ',';
					BYTE type = 2;
					for (j = 1; testStr2[j] != ']'; j++) {
						if (testStr2[j] == ',') {
							type++;
							if (type > 1 + MAXARGS) {
								error = TRUE;
								LoadString(resInst, 2722, eq->message, STRLEN_MAX);
								goto END;
							}
							argListStr += " ,";
							argListStr_singleLine += " ,";
						}
						else if (testStr2[j] == '\0') {
							error = TRUE;
							LoadString(resInst, 2724, eq->message, STRLEN_MAX);
							goto END;
						}
						else if (testStr2[j] != ' ' && testStr2[j] != '\t') {
							error = TRUE;
							LoadString(resInst, 2705, eq->message, STRLEN_MAX);
							goto END;
						}
					}
					argType[argCount] = type;
					ptr = testStr2 + j + 1;
					ptr += strspn(ptr, " \t");
					if (*ptr != ')') {
						error = TRUE;
						LoadString(resInst, 2723, eq->message, STRLEN_MAX);
						goto END;
					}
					argListStr += " ])";
					argListStr_singleLine += " ])";
					c = ptr;
				}
				else if (addrType == 3) {
					argListStr += "(";
					argListStr_singleLine += "(";
					testStr2 += j;
					__int64 len = strlen(testStr2);
					if (testStr2 + len - string < len0) testStr2[len] = ',';
					BYTE typeTmp = 0, base = 1;
					int nargs = 0;
					int depth = 1;
					__int64 nChar;
					testStr2++;
					char prevChar = '\0';
					while(depth && nargs < 4) {
						for (c = testStr2; *c != '(' && *c != ')' && *c != ',' && *c != '\0'; c++);
						if (*c == '\0') {
							error = TRUE;
							LoadString(resInst, 2725, eq->message, STRLEN_MAX);
							goto END;
						}
						nChar = c - testStr2;
						strncpy(id, testStr2, (size_t)nChar);
						id[nChar] = '\0';
						nBlanks = strspn(id, " \t");
						BOOL isEmpty = (nBlanks == strlen(id));
						if (!isEmpty) {
							strcpy(tmp, id + nBlanks);
							strrev(tmp);
							strcpy(id, tmp + strspn(tmp, " \t"));
							strrev(id);
							if (nargs && prevChar == ')') {
								argListStr += ",";
								argListStr_singleLine += ",";
							}
						}
						if (*c == ',' || *c == ')') {
							if (isEmpty) {
								if ((*c == ',' || nargs) && prevChar != ')') {
									base *= 5;
									nargs++;
								}
							}
							else {
								tmp[0] = '\0';
								if (strpbrk(id, "+-/*^=,().!?<>;:[]{}|\\'%\"")) LoadString(resInst, 2289, tmp, STRLEN_MAX);
								else if (strpbrk(id, "0123456789") == id + nBlanks) LoadString(resInst, 2290, tmp, STRLEN_MAX);
								else {
									for (i = NBINARYOP; i < NSTDFUNC && strcmpi(id, stdFuncNames[i]); i++);
									if (i < NSTDFUNC) {
										LoadString(resInst, 2293, tmp, STRLEN_MAX);
										sprintf(S, tmp, stdFuncNames[i]);
										strcpy(tmp, S);
									}
								}
								if (tmp[0] != '\0') {
									error = TRUE;
									LoadString(resInst, 2725, eq->message, STRLEN_MAX);
									strcat(eq->message, " ");
									strcat(eq->message, tmp);
									goto END;
								}
								base *= 5;
								nargs++;
								if (nargs > 1) {
									argListStr += " ";
									argListStr_singleLine += " ";
								}
								argListStr += id;
								argListStr_singleLine += id;
							}
							if (isEmpty && prevChar != ')') {
								argListStr += " ";
								argListStr_singleLine += " ";
							}
							if (*c == ',') {
								argListStr += ",";
								argListStr_singleLine += ",";
							}
							else {
								argListStr += ")";
								argListStr_singleLine += ")";
								depth--;
							}
						}
						else {
							if (isEmpty || strcmpi(id, "loc")) {
								error = TRUE;
								LoadString(resInst, 2726, eq->message, STRLEN_MAX);
								goto END;
							}
							if (nargs) {
								argListStr += " ";
								argListStr_singleLine += " ";
							}
							argListStr += "loc(";
							argListStr_singleLine += "loc(";
							testStr2 = c + 1;
							for (c = testStr2; *c != '[' && *c != ')' && *c != '\0'; c++);
							if (*c == '\0') {
								error = TRUE;
								LoadString(resInst, 2725, eq->message, STRLEN_MAX);
								goto END;
							}
							nChar = c - testStr2;
							strncpy(id, testStr2, (size_t)nChar);
							id[nChar] = '\0';
							nBlanks = strspn(id, " \t");
							if (nBlanks < strlen(id)) {
								tmp[0] = '\0';
								if (strpbrk(id, "+-/*^=,().!?<>;:[]{}|\\'%\"")) LoadString(resInst, 2289, tmp, STRLEN_MAX);
								else if (strpbrk(id, "0123456789") == id + nBlanks) LoadString(resInst, 2290, tmp, STRLEN_MAX);
								else {
									for (i = NBINARYOP; i < NSTDFUNC && strcmpi(id, stdFuncNames[i]); i++);
									if (i < NSTDFUNC) {
										LoadString(resInst, 2293, tmp, STRLEN_MAX);
										sprintf(S, tmp, stdFuncNames[i]);
										strcpy(tmp, S);
									}
								}
								if (tmp[0] != '\0') {
									error = TRUE;
									LoadString(resInst, 2725, eq->message, STRLEN_MAX);
									strcat(eq->message, " ");
									strcat(eq->message, tmp);
									goto END;
								}
								strcpy(tmp, id + nBlanks);
								strrev(tmp);
								strcpy(id, tmp + strspn(tmp, " \t"));
								strrev(id);
								argListStr += id;
								argListStr_singleLine += id;
							}
							if (*c == ')') {
								typeTmp += base;
							}
							else {
								argListStr += "[";
								argListStr_singleLine += "[";
								testStr2 = c + 1;
								for (c = testStr2; *c != ',' && *c != ']' && *c != '\0'; c++);
								if (*c == '\0') {
									error = TRUE;
									LoadString(resInst, 2725, eq->message, STRLEN_MAX);
									goto END;
								}
								nChar = c - testStr2;
								strncpy(id, testStr2, (size_t)nChar);
								id[nChar] = '\0';
								nBlanks = strspn(id, " \t");
								if (nBlanks != nChar) {
									error = TRUE;
									LoadString(resInst, 2705, eq->message, STRLEN_MAX);
									goto END;
								}
								if (*c == ']') {
									typeTmp += 2 * base;
								}
								else {
									argListStr += ",";
									argListStr_singleLine += ",";
									testStr2 = c + 1;
									for (c = testStr2; *c != ',' && *c != ']' && *c != '\0'; c++);
									if (*c == '\0') {
										error = TRUE;
										LoadString(resInst, 2725, eq->message, STRLEN_MAX);
										goto END;
									}
									nChar = c - testStr2;
									strncpy(id, testStr2, (size_t)nChar);
									id[nChar] = '\0';
									nBlanks = strspn(id, " \t");
									if (nBlanks != nChar) {
										error = TRUE;
										LoadString(resInst, 2705, eq->message, STRLEN_MAX);
										goto END;
									}
									if (*c == ',') {
										error = TRUE;
										LoadString(resInst, 2727, eq->message, STRLEN_MAX);
										goto END;
									}
									typeTmp += 3 * base;
								}
								argListStr += "]";
								argListStr_singleLine += "]";
								testStr2 = c + 1;
								for (c = testStr2; *c != ')' && *c != '\0'; c++);
								if (*c == '\0') {
									error = TRUE;
									LoadString(resInst, 2725, eq->message, STRLEN_MAX);
									goto END;
								}
								nChar = c - testStr2;
								strncpy(id, testStr2, (size_t)nChar);
								id[nChar] = '\0';
								nBlanks = strspn(id, " \t");
								if (nBlanks != nChar) {
									error = TRUE;
									LoadString(resInst, 2728, eq->message, STRLEN_MAX);
									goto END;
								}
							}
							base *= 5;
							nargs++;
							argListStr += ")";
							argListStr_singleLine += ")";
						}
						prevChar = *c;
						testStr2 = c + 1;
					}
					if (nargs > 3) {
						error = TRUE;
						LoadString(resInst, 2729, eq->message, STRLEN_MAX);
						goto END;
					}
					for (c = testStr2; *c != ')' && *c != '\0'; c++);
					if (*c == '\0') {
						error = TRUE;
						LoadString(resInst, 2381, eq->message, STRLEN_MAX);
						goto END;
					}
					nChar = c - testStr2;
					strncpy(id, testStr2, (size_t)nChar);
					id[nChar] = '\0';
					nBlanks = strspn(id, " \t");
					if (nBlanks != nChar) {
						error = TRUE;
						LoadString(resInst, 2725, eq->message, STRLEN_MAX);
						goto END;
					}
					BYTE type = MAXARGS + 2;
					for (base = 1, i = 0; i < nargs; i++, base *= 5) type += base;
					argType[argCount] = type + typeTmp;
					argListStr += ")";
					argListStr_singleLine += ")";
				}

				argCount++;
				argCount_line++;

				while (*c != ',' && *c != ')' && *c != '\0') c++;
				if (*c == ',' && *(c + 1) == ',') {
					error = TRUE;
					LoadString(resInst, 2238, S, STRLEN_MAX); sprintf(eq->message, S, testing);
					goto END;
				}
				if (*c != '\0') ptr = strtok(c + 1, ",");
				else ptr = NULL;
			}

			if (subroutineBody) break;

			commaInsertPos = argListStr.GetLength();
			if (comment[0] != '\0') {
				argListStr += " //";
				argListStr += comment;
			}

			if (argCount_line || comment[0] != '\0') emptyCount = 0;
			else emptyCount++;
		}
		else {
			if (comment[0] != '\0') {
				if (commaInsertPos || !argCount && commentLineAtTop) argListStr += "\r\n";
				if (emptyCount) argListStr += "\r\n";
				argListStr += " //";
				argListStr += comment;
				emptyCount = 0;
				if (!argCount) commentLineAtTop = TRUE;
			}
			else emptyCount++;
		}

		if (!stringEnd || stringEnd - buffer >= len || subroutineBody) break;
		if (stringEnd - buffer < len) stringEnd++;
		if (stringEnd - buffer < len && *stringEnd == 0x0a) stringEnd++;
		if (stringEnd - buffer >= len) break;
		string = stringEnd;
		stringEnd = strchr(string, 0x0d);
	}
	if (!subroutineBody) {
		error = TRUE;
		strcpy(eq->message, "The right parenthesis indicating the end of the list of formal parameters is absent.");
	}

END:
	if (!error && nConf) {
		error = TRUE;
		LoadString(resInst, 2679, eq->message, STRLEN_MAX);
		int i;
		char tmp[STRLEN_MAX+1];
		size_t len = strlen(eq->message);
		for (i = 0; i < nConf; i++) {
			sprintf(tmp, "%c%c%s \"%s\"", 0x0d, 0x0a, types[typeConf[i]], nameConf[i]);
			if (i < nConf - 1) strcat(tmp, ", ");
			else strcat(tmp, ".\n");
			len += strlen(tmp);
			if (len > STRLEN_MAX) break;
			strcat(eq->message, tmp);
		}
	}
	free(typeConf);
	free(numConf);
	free(nameConf);
	free(nameConfBuffer);
	if (error) {
		eq->errorType = 2;
		op->errLineNum = iLine;
		op->warnLineNum = -1;
		return iLine;
	}

	__int64 offset = subroutineBody - buffer;
	strcpy(buffer, code.GetBuffer() + offset); code.ReleaseBuffer();
	subroutineCode[iFunc] = buffer;
	delete buffer;
	subroutineArgInfo[iFunc].list = (char *)argListStr;
	subroutineArgInfo[iFunc].list_singleLine = (char *)argListStr_singleLine;
	subroutineArgInfo[iFunc].count = argCount;
	for (j = 0; j < argCount; j++) subroutineArgInfo[iFunc].type[j] = argType[j];
	funcType.SetAtGrow(NSTDFUNC + iFunc, FuncType(NSTDFUNC + iFunc));
	return iLine;
}

int Compiler::Arrays(char *code, char *msg)
{
	int nArrays0_bak = nArrays0;
	int len = (int)strlen(code);
	int tmpLen = len + 10 + MAXIDENTLEN;
	char *tmp = new char[tmpLen];
	strcpy(tmp, code);
	char *ptr0 = tmp;
	char *ptr = strchr(ptr0, 0x0d);
	int iLine = 0, type;
	unsigned long curPos;
	ARRAYDATA arrayData;
	char *endPtr, *elemPtr;
	bool insideInitializerList = false;
	size_t commentLength;
	while(ptr || ptr0 - tmp < len) {
		commentLength = 0;
		if (ptr) *ptr = '\0';
		char *ptr2 = strchr(ptr0, '/');
		while(ptr2) {
			if (*(ptr2 + 1) == '/') {
				commentLength = strlen(ptr2);
				*ptr2 = '\0';
				break;
			}
			ptr2 = strchr(ptr2 + 1, '/');
		}
		ptr = ptr0 + strspn(ptr0, " \t");
		if (*ptr == '\0') goto ARRAY_NEXTLINE;
		if (!insideInitializerList) {
			if (!strnicmp(ptr, "index_base", 10) && (ptr[10] == ' ' || ptr[10] == '\t' || ptr[10] == '\0')) {
				if (nArrays0 > nArrays0_bak) {
					sprintf(msg, "******** The keyword \"INDEX_BASE\" must be the first keyword after the \"ARRAYS\" directive. ********");
					goto ARRAY_ERROR;
				}
				ptr += 10;
				ptr += strspn(ptr, " \t");
				if (*ptr == '\0') {
					sprintf(msg, "******** The keyword \"INDEX_BASE\" must be followed by 0 or 1. ********");
					goto ARRAY_ERROR;
				}
				index_base = strtol(ptr, &endPtr, 10);
				if (index_base && index_base != 1 || *endPtr != '\0' && *endPtr != ' ' && *endPtr != '\t') {
					sprintf(msg, "******** The keyword \"INDEX_BASE\" must be followed by 0 or 1. ********");
					goto ARRAY_ERROR;
				}
				ptr = endPtr + strspn(endPtr, " \t");
				if (*ptr != '\0') {
					sprintf(msg, "******** Error: the remainder of the line after the directive \"INDEX_BASE %d\" is not empty. ********", index_base);
					goto ARRAY_ERROR;
				}
				goto ARRAY_NEXTLINE;
			}
			if (!strnicmp(ptr, "double", 6) && (ptr[6] == ' ' || ptr[6] == '\t')) { type = 0; ptr += 6; }
			else if (!strnicmp(ptr, "float", 5) && (ptr[5] == ' ' || ptr[5] == '\t')) { type = 1; ptr += 5; }
			else if (!strnicmp(ptr, "int", 3) && (ptr[3] == ' ' || ptr[3] == '\t')) { type = 2; ptr += 3; }
			else if (!strnicmp(ptr, "uint", 4) && (ptr[4] == ' ' || ptr[4] == '\t')) { type = 3; ptr += 4; }
			else if (!strnicmp(ptr, "int2", 4) && (ptr[4] == ' ' || ptr[4] == '\t')) { type = 4; ptr += 4; }
			else if (!strnicmp(ptr, "uint2", 5) && (ptr[5] == ' ' || ptr[5] == '\t')) { type = 5; ptr += 5; }
			else if (!strnicmp(ptr, "int1", 4) && (ptr[4] == ' ' || ptr[4] == '\t')) { type = 6; ptr += 4; }
			else if (!strnicmp(ptr, "uint1", 5) && (ptr[5] == ' ' || ptr[5] == '\t')) { type = 7; ptr += 5; }
			else {
				sprintf(msg, "******** Incorrect format of the definition of an array: a new line must start with a keyword, unless it is inside the list of array initializers. ********");
				goto ARRAY_ERROR;
			}
		}
		ptr += strspn(ptr, " \t");
		while (*ptr != '\0') {
			if (insideInitializerList) {
				double test_value = strtod(ptr, &endPtr);
				char *ptr1 = ptr;
				if (*endPtr == '}') insideInitializerList = false;
				else if (*endPtr != ',' && *endPtr != ' ' && *endPtr != '\t' && *endPtr != '\0') {
					sprintf(msg, "******** Invalid value of an array \"%s\" element. ********", arrayData.name);
					goto ARRAY_ERROR;
				}
				if (endPtr != ptr1) {
					if (curPos == arrayData.size) {
						sprintf(msg, "******** Too many initializers of array \"%s\". ********", arrayData.name);
						goto ARRAY_ERROR;
					}
					ArrayAssignment(elemPtr + curPos, type, test_value);
					curPos += typeSize[type];
				}
				ptr = insideInitializerList ? endPtr : endPtr + 1;
				ptr += strspn(ptr, " \t");
				if (*ptr == ',') ptr += (1 + strspn(ptr + 1, " \t"));
				continue;
			}
			ptr2 = strchr(ptr, '[');
			if (!ptr2 || ptr2 == ptr || *(ptr2 - 1) == ' ' || *(ptr2 - 1) == '\t') {
				sprintf(msg, "******** Incorrect format of the definition of an array. ********");
				goto ARRAY_ERROR;
			}
			*ptr2 = '\0';
			int num = -1, nConf = 0;
			int *typeConf = (int *)malloc(0);
			int *numConf = (int *)malloc(0);
			char **nameConf = (char **)malloc(0);
			char *nameConfBuffer = (char *)malloc(0);
			BOOL rc = CheckIdentifier(ptr, 5, num, "Array name", nConf, typeConf, numConf, nameConf, nameConfBuffer, S);
			free(typeConf);
			free(numConf);
			free(nameConf);
			free(nameConfBuffer);
			if (!rc) {
				sprintf(msg, "******** %s ********", S);
				goto ARRAY_ERROR;
			}
			strcpy(arrayData.name, ptr);
			arrayData.type = type;
			ptr = ptr2 + 1;
			int nDim;
			for(nDim = 0; nDim < 20; nDim++) {
				ptr += strspn(ptr, " \t");
				if (*ptr == '\0') {
					sprintf(msg, "******** Missing right bracket in the definition of array \"%s\". ********", arrayData.name);
					goto ARRAY_ERROR;
				}
				int n = strtol(ptr, &endPtr, 10);
				if (n < 1 || n > 2000000000 || *endPtr != ',' && *endPtr != ']' && *endPtr != ' ' && *endPtr != '\t') {
					sprintf(msg, "******** Invalid value of a dimension of array \"%s\". ********", arrayData.name);
					goto ARRAY_ERROR;
				}
				arrayData.nElem[nDim] = n;
				ptr = endPtr + strspn(endPtr, " \t");
				if (*ptr == ',') ptr++;
				else if (*ptr == ']') {
					nDim++;
					break;
				}
			}
			if (nDim == 20 && *ptr != ']') {
				sprintf(msg, "******** Array \"%s\" has too many dimensions. ********", arrayData.name);
				goto ARRAY_ERROR;
			}
			if (nArrays0 == MAXARRAYS) {
				sprintf(msg, "******** Error: the number of arrays is greater than %d. ********", MAXARRAYS);
				goto ARRAY_ERROR;
			}

			arrayData.nDim = nDim;
			double sizeDbl;
			long long size = ArraySize(-type - 1, 0, nDim, arrayData.nElem, &(arrayData.nElements), &sizeDbl);
			if (sizeDbl > 2147483647.) {
				double nGB = sizeDbl / 1073741824.;
				sprintf(msg, "******** Array \"%s\" would require %-.5g GB of memory. Array size cannot exceed 2 GB. ********", arrayData.name, nGB);
				goto ARRAY_ERROR;
			}
			arrayData.size = (unsigned long)size;
			if (nDim > 0) arrayData.stepSize[nDim-1] = 1;
			for (int i = nDim - 2; i >= 0; i--) arrayData.stepSize[i] = arrayData.nElem[i+1] * arrayData.stepSize[i+1];
			arrayData.elements = AllocateArray(arrayData.type, 0, 1, &(arrayData.nElements));
			if (!arrayData.elements) {
				sprintf(msg, "******** Runtime error when attempting to allocate array \"%s\". ********", arrayData.name);
				goto ARRAY_ERROR;
			}
			arrays = (ARRAYDATA *)realloc(arrays, (nArrays + 1) * sizeof(ARRAYDATA));
			if (nArrays > nArrays0) {
				int nBuiltinArrays = nArrays - nArrays0;
				ARRAYDATA *tmp = new ARRAYDATA[nBuiltinArrays];
				memcpy(tmp, arrays + nArrays0, nBuiltinArrays * sizeof(ARRAYDATA));
				memcpy(arrays + nArrays0 + 1, tmp, nBuiltinArrays * sizeof(ARRAYDATA));
				delete tmp;
			}
			arrays[nArrays0] = arrayData;
			nArrays0++;
			nArrays++;

			ptr += (1 + strspn(ptr + 1, " \t"));
			if (*ptr == ',') ptr += (1 + strspn(ptr + 1, " \t"));
			else if (*ptr == '=') {
				ptr += (1 + strspn(ptr + 1, " \t"));
				if (*ptr != '{') {
					sprintf(msg, "******** Error: The equality sign in the \"ARRAYS\" block must be followed by the left brace. ********");
					goto ARRAY_ERROR;
				}
				insideInitializerList = true;
				curPos = 0;
				elemPtr = (char *)(arrayData.elements);
				ptr += (1 + strspn(ptr + 1, " \t"));
			}
		}
ARRAY_NEXTLINE:
		if (!ptr || ptr - tmp >= len) break;
		ptr += commentLength;
		if (ptr - tmp < len) ptr++;
		if (ptr - tmp < len && *ptr == 0x0a) ptr++;
		if (ptr - tmp >= len) break;
		ptr0 = ptr;
		ptr = strchr(ptr0, 0x0d);
		iLine++;
	}
	delete tmp;
	if (nArrays0 == nArrays0_bak) {
		sprintf(msg, "******** Error: there are no arrays defined. ********");
		return iLine;
	}
	return -1;
ARRAY_ERROR:
	delete tmp;
	return iLine;
}

void* Compiler::AllocateArray(long type, long iDim, long nDim, unsigned long *nElem)
{
	try
	{
	if (iDim == nDim - 1) {
		void *ptr;
		if (type == 0) ptr = new double[nElem[iDim]];
		else if (type == 1) ptr = new float[nElem[iDim]];
		else if (type == 2) ptr = new long[nElem[iDim]];
		else if (type == 3) ptr = new unsigned long[nElem[iDim]];
		else if (type == 4) ptr = new short[nElem[iDim]];
		else if (type == 5) ptr = new unsigned short[nElem[iDim]];
		else if (type == 6) ptr = new char[nElem[iDim]];
		else if (type == 7) ptr = new unsigned char[nElem[iDim]];
		memset(ptr, 0, nElem[iDim] * typeSize[type]);
		return ptr;
	}
	else {
		void **ptr = new void*[nElem[iDim]];
		for (unsigned long i = 0; i < nElem[iDim]; i++) {
			ptr[i] = AllocateArray(type, iDim + 1, nDim, nElem);
			if (!ptr[i]) return NULL;
		}
		return ptr;
	}
	}
	catch(...)
	{
		return NULL;
	}
}

long long Compiler::ArraySize(long type, long iDim, long nDim, unsigned long *nElem, unsigned long *nElements, double *sizeDbl)
{
	BOOL contiguous;
	if (type >= 0) contiguous = FALSE;
	else {
		contiguous = TRUE;
		type = -type - 1;
	}
	int i;
	double sizeTmp = 0.;
	double nElementsDbl = 1.;
	for (i = iDim; i < nDim - 1; i++) {
		nElementsDbl *= nElem[i];
		if (!contiguous) sizeTmp += nElementsDbl;
	}
	nElementsDbl *= nElem[i];
	if (!contiguous) sizeTmp *= sizeof(unsigned long);
	sizeTmp += nElementsDbl * typeSize[type];
	if (nElements) *nElements = (unsigned long)(nElementsDbl + 0.5);
	if (sizeDbl) *sizeDbl = sizeTmp;
	return (long long)(sizeTmp + 0.5);
}

int Compiler::TypeSize(long type)
{
	if (type == 0) return sizeof(double);
	else if (type == 1) return sizeof(float);
	else if (type == 2) return sizeof(long);
	else if (type == 3) return sizeof(unsigned long);
	else if (type == 4) return sizeof(short);
	else if (type == 5) return sizeof(unsigned short);
	else if (type == 6) return sizeof(char);
	else if (type == 7) return sizeof(unsigned char);
	else return 0;
}

char *SearchOutsideTheQuotes(char *ptr2, char *charSet)
{
	if (*ptr2 == '\"') {
		ptr2 = strchr(ptr2 + 1, '\"');
		if (!ptr2) return NULL;
		ptr2++;
	}
	bool insideQuotes;
	do {
		char *ptr_quotes = strchr(ptr2, '\"');
		ptr2 = strpbrk(ptr2, charSet);
		if (!ptr2) break;
		insideQuotes = false;
		while(ptr_quotes && ptr_quotes < ptr2) {
			insideQuotes = !insideQuotes;
			ptr_quotes = strchr(ptr_quotes + 1, '\"');
		}
		if (insideQuotes) {
			if (ptr_quotes) ptr2 = ptr_quotes + 1;
			else ptr2 = NULL;
		}
	} while(insideQuotes && ptr2);
	return ptr2;
}

int Compiler::Compile(char *code, int iLine, EQDATA *eqData, OPDATA *opData/*, CGraphView *v*/, int iFunc, BOOL ignoreUnloadableSubroutines)
{
	int i, j, k;
	__int64 n;
	int nArrInd = 0;
	int subroutineArgCount = 0;
	opData->isSubroutine = TRUE;
	subroutineArgCount = subroutineArgInfo[iFunc].count;

	opData->Reset(subroutineArgCount, eqData->nParm);
	for (i = NSYSTEMVARS + nArrInd, j = 0; j < subroutineArgCount; j++, i++) opData->isConst[i] = (subroutineArgInfo[iFunc].type[j] != 1);

	int iUnloadableSubroutine = -1;
	int iUnloadableDllFunc = -1;
	int unloadableSubroutineLineNum/*, unloadableDllFuncLineNum*/;
	Array_String vars;
	Array_int numVar, creationDepth, assignmentDepth, integralControl;
	Array_int isVisible, createdInElse, assignedInElse, integralProcessed;
	int nIntegralControlsStored = 0, curIntegralControl = -1, nIntegrationVarsStored = 0;
	int warnVar = -1;
	int nVars0 = opData->nOperands;
	int nVars = nVars0;
	vars.SetSize(nVars);
	numVar.SetSize(nVars);
	int nSubroutineArgUsed = 0;
	int codeLength = (int)strlen(code);
	int tmpLen = codeLength + 10 + MAXIDENTLEN;
	if (iFunc >= 0) tmpLen += (int)strlen(subroutineArgInfo[iFunc].list_singleLine) + 1;
//	if (nArrInd > 0) tmpLen += strlen(arrayIndices) + 1;
	char *tmp = new char[tmpLen];
	char *tmp2;
	for (i = 0; i < nVars; i++) numVar[i] = i;
	for (i = 0; i < NSYSTEMVARS; i++) vars[i] = systemVarNames[i];
/*	if (nArrInd > 0) {
		strcpy(tmp, arrayIndices);
		char *name = strtok(tmp, ",");
		for (j = 0; j < nArrInd && name; j++, i++) {
			vars[i] = name;
			name = strtok(NULL, ",");
			if (name && name[0] == ' ') name++;
		}
		for (; j < nArrInd; j++, i++) vars[i] = "";
	}*/
	char *subroutineArgNames[MAXARGS];
	if (iFunc >= 0) {
		strcpy(tmp, subroutineArgInfo[iFunc].list_singleLine);
		tmp2 = tmp;
		for (j = 0; j < subroutineArgCount; j++, i++) {
			char *name = strtok(tmp2, ",(");
			if (name[0] == ' ') name++;
			BOOL isAddr;
			if (strcmpi(name, "loc")) isAddr = FALSE;
			else {
				isAddr = TRUE;
				name = strtok(NULL, ")[(");
			}
			vars[i] = name;
			subroutineArgNames[j] = new char[strlen(name)+1];
			strcpy(subroutineArgNames[j], name);
			if (isAddr) {
				int depth = 1;
				for (n = name + strlen(name) - tmp; depth; n++) {
					char c = subroutineArgInfo[iFunc].list_singleLine[n];
					if (c == ')') depth--;
					else if (c == '(') depth++;
				}
				tmp2 = tmp + n;
			}
			else tmp2 = name + strlen(name) + 1;
		}
	}
	for (j = 0; j < eqData->nParm; j++, i++) {
		vars[i] = eqData->parm[j].name;
	}
	for (j = 0; j < nGParm; j++, i++) {
		vars[i] = globalParms[j].name;
	}

	char *condKW[3] = {"if", "while", "for"};
	size_t nextPos = 0, len2;
	int lowestWaitingLevel = -2;

	BOOL equalityPresent, equalityFound, prevTokenIsIdent, prevTokenIsFunc, prevTokenIsArray, prevTokenIsComma;
	char *strpos, *strend;
	String prevToken;
	int currentState = -1, functionScopeDepth = -1, curFuncLevel = -1;

	int arrayAssignmentPos = 0;
	BOOL isArrayAssignment = FALSE;

	int nSpecFunc = 11;
	int specFunc[11] = {NSTDFUNC0 + 4, NSTDFUNC0 + 5, NSTDFUNC0 + 6, NSTDFUNC0 + 7, NSTDFUNC0 + 8, NSTDFUNC0 + 9, NSTDFUNC0 + 10, NSTDFUNC0 + 11, NSTDFUNC0 + 12, NSTDFUNC0 + 13, NSTDFUNC0 + 14}; // Integration, summation, iteration functions and root finder
	int nSpecArgs[11] = {3, 3, 2, 2, 3, 3, 3, 3, 3, 3, 3};
	int specArgs[11][3] = {
		3, 1, 0,
		6, 4, 3,
		3, 1, 0,
		6, 4, 3,
	//	6, 4, 0, // ???
		5, 3, 2,
		9, 7, 6,
		3, 1, 0,
		4, 2, 1,
		3, 1, 0,
		3, 1, 0,
		6, 4, 3
	};
	String string, stdFuncName;

	char *input2_0 = NULL;

	Array_String idents;
	symbolType.RemoveAll();
	subType.RemoveAll();
	constants.RemoveAll();

	Array_String stack_idents;
	stack_symbolType.RemoveAll();
	stack_subType.RemoveAll();
	stack_constants.RemoveAll();

	addType = 0;
	int nOut = 0;
	char *input0 = new char[tmpLen];
	char *input = new char[tmpLen];
	double num;
	int id, l;
	char opc;
	char ident[1001];
	ident[1000] = '\0';

	double sc_number;
	int type, subTp, sc_type, sc_subType;
	nOut = 0;
	i = 0;
 
	int sl;  // stack length
	char *c, *c_prev, *c_prev1, testString[6], *input2;
 
	char res[11];

	int xAssignment = -1;
	if (iFunc >= 0) xAssignment = 0;
	int currentIF = -1, currentIFdepth = -1;
	int nCond = 3, nLeftBraces = 0, kwLen[3] = {2, 5, 3};
	BOOL insideIFheader = FALSE, justOutOfIFheader = FALSE, justOutOfIF = FALSE, searchingForLeftBrace = FALSE, rightBraceFound = FALSE;
	int curFuncNum;

	Array_int nLinkedIF, headerControlNum, elseControlNum, cond, nInterm_bak, nControl_bak, nXbreaks_bak, nLeft, nRight;
	Array_int statementFound, funcNum, argNum, correctArgNum, functionScopeDepth0, sl0;
	Array_int insideElse, leftBracePresent, funcIsArray;
	Array_String nestedFuncName;
	Array_int listStartPos; // Offset of the first character of the argument list (immediately after opening '(') from the start of input
	Array_int listEndPos; // Offset of the closing ')' of the argument list from the start of input
	Array_int argStartPos; // Offset of the comma (or of the character that is immediately after the last ',' or after the opening '(') from the start of input
	Array_int integrationVarNum, integrationVar_stackPos;

	int forDepth = 0, braceDepth = 0, forConditionStatementNum = -1;
	Array_charPtr forEndExpr;
	Array_int forBraceDepth, forLineNum;

	String currentFormatString;

	int nl0 = iLine, nl = 0;
	Array_charPtr input1;
	Array_int lineNum;
	char *ptr, *ptr0, *ptr1, *ptr2, sep[4] = {'\0', ';', '{', '}'};
	strcpy(tmp, code);
	int len = (int)strlen(tmp);
	size_t maxLen = 0;
	input1.SetSize(0);
	lineNum.SetSize(0);
	int forHeaderLineNum = -1; // >= 0 when the line contains only the header of a "for" loop
	ptr0 = tmp;
	ptr = strchr(ptr0, 0x0d);
	bool insideQuotes;
	while(ptr || ptr0 - tmp < len) {
		if (ptr) *ptr = '\0';
		ptr2 = strchr(ptr0, '/');
		while(ptr2) {
			if (*(ptr2 + 1) == '/') {
				*ptr2 = '\0';
				break;
			}
			ptr2 = strchr(ptr2 + 1, '/');
		}
		size_t len1;
		char *endExpression, *headerEnd;
		ptr1 = ptr0 + strspn(ptr0, " \t");
		ptr2 = SearchOutsideTheQuotes(ptr0, ";{}");
//		ptr2 = ptr0 + strcspn(ptr0, ";{}");
		int separator = 0;
		if (ptr2) {
			if (*ptr2 == ';') separator = 1;
			else if (*ptr2 == '{') separator = 2;
			else separator = 3;
		}
		bool forHeader = !strnicmp(ptr1, "for", 3) && *(ptr1 + 3 + strspn(ptr1 + 3, " \t")) == '(';
		while(1) {
			size_t nHeaderEndBlanks;
			bool headerEndEmpty;
			if (forHeaderLineNum >= 0) {
				if (separator) *ptr2 = '\0';
				endExpression = forEndExpr[forDepth-1];
				headerEnd = ptr0;
				nHeaderEndBlanks = strspn(headerEnd, " \t");
				headerEndEmpty = (nHeaderEndBlanks == strlen(headerEnd));
				if (!separator && headerEndEmpty) goto CONTINUE;
				else goto FOR_LOOP_HEADER_END;
			}
			if (!forHeader) {
				if (separator) *ptr2 = '\0';
				if (forDepth) {
					if (separator == 2) braceDepth++;
					else if (separator == 3) {
						braceDepth--;
						if (braceDepth < 0) {
							sprintf(eqData->message, "Error: mismatched braces.");
							l = nl - 1;
							goto SY_ERROR;
						}
					}
				}
				len1 = strlen(ptr0);
				if (separator > 1 && (!forDepth || separator == 2 || braceDepth != forBraceDepth[forDepth-1])) {
					len1++;
					if (len1 > maxLen) maxLen = len1;
					input1.Add(new char[len1+1]);
					sprintf(input1[nl], "%s%c", ptr0, sep[separator]);
					lineNum.Add(nl0);
					nl++;
				}
				else if (strspn(ptr0, " \t") != len1) {
					if (len1 > maxLen) maxLen = len1;
					input1.Add(new char[len1+1]);
					strcpy(input1[nl], ptr0);
					lineNum.Add(nl0);
					nl++;
				}
				if (forDepth && separator == 3 && braceDepth == forBraceDepth[forDepth-1]) {
					forDepth--;
					len1 = strlen(forEndExpr[forDepth]);
					if (strspn(forEndExpr[forDepth], " \t") != len1) {
						len1++;
						if (len1 > maxLen) maxLen = len1;
						input1.Add(new char[len1+1]);
						sprintf(input1[nl], "%s}", forEndExpr[forDepth]);
					}
					else {
						input1.Add(new char[2]);
						strcpy(input1[nl], "}");
					}
					lineNum.Add(forLineNum[forDepth]);
					forEndExpr.RemoveAt(forDepth);
					forBraceDepth.RemoveAt(forDepth);
					forLineNum.RemoveAt(forDepth);
					nl++;
				}
			}
			else { // "for" loop header
				forHeader = false;
				ptr1 += 3;
				ptr1 += strspn(ptr1, " \t");
				int depth = 1, nSemicolons = 0;
				bool headerErr = false;
				insideQuotes = false;
				for (j = 1; ptr1[j] != '\0' && nSemicolons < 3; j++) {
					if (ptr1[j] == '\"') insideQuotes = !insideQuotes;
					if (insideQuotes) continue;
					if (ptr1[j] == '(') depth++;
					else if (ptr1[j] == ')') depth--;
					else if (ptr1[j] == ';') nSemicolons++;
					else if (ptr1[j] == '{' || ptr1[j] == '}') {
						headerErr = true;
						sprintf(eqData->message, "Error: braces are not allowed inside the \"for\" loop header.");
						break;
					}
					if (!depth) break;
				}
				if (!headerErr && nSemicolons != 2) {
					headerErr = true;
					sprintf(eqData->message, "Incorrect format of the \"for\" loop header: the number of semicolons is not equal to 2.");
				}
				if (!headerErr && depth) {
					headerErr = true;
					sprintf(eqData->message, "Incorrect format of the \"for\" loop header: mismatched parentheses.");
				}
				if (headerErr) {
					l = nl;
					lineNum.Add(nl0);
					goto SY_ERROR;
				}
				ptr1[j] = '\0'; // the closing parenthesis of the header
				headerEnd = ptr1 + j + 1;
				
				ptr0 = ptr1 + 1; // initialization expression
				ptr1 = SearchOutsideTheQuotes(ptr0, ";");
//				ptr1 = strchr(ptr0, ';');
				*ptr1 = '\0';
				len1 = strlen(ptr0);
				if (strspn(ptr0, " \t") != len1) {
					if (len1 > maxLen) maxLen = len1;
					input1.Add(new char[len1+1]);
					strcpy(input1[nl], ptr0);
					lineNum.Add(nl0);
					nl++;
				}

				ptr0 = ptr1 + 1; // condition of the "for" loop
				ptr1 = SearchOutsideTheQuotes(ptr0, ";");
//				ptr1 = strchr(ptr0, ';');
				*ptr1 = '\0';
				len1 = 4 + strlen(ptr0) + 3;
				if (len1 > maxLen) maxLen = len1;
				input1.Add(new char[len1+1]);
				sprintf(input1[nl], "for(%s)", ptr0);
				lineNum.Add(nl0);
				forConditionStatementNum = nl;
				nl++;
				if (strspn(ptr0, " \t") == strlen(ptr0)) {
					sprintf(eqData->message, "Incorrect format of the \"for\" loop header: the condition statement is empty.");
					l = nl - 1;
					goto SY_ERROR;
				}

				endExpression = ptr1 + 1;
				ptr2 = SearchOutsideTheQuotes(headerEnd, ";{}");
//				ptr2 = headerEnd + strcspn(headerEnd, ";{}");
				separator = 0;
				if (ptr2) {
					if (*ptr2 == ';') separator = 1;
					else if (*ptr2 == '{') separator = 2;
					else separator = 3;
				}
				if (separator) *ptr2 = '\0';
				nHeaderEndBlanks = strspn(headerEnd, " \t");
				headerEndEmpty = (nHeaderEndBlanks == strlen(headerEnd));
				if (!separator && headerEndEmpty) { // the end expression will be added later
					forHeaderLineNum = nl0;
					forEndExpr.SetAtGrow(forDepth, endExpression);
					forBraceDepth.SetAtGrow(forDepth, braceDepth);
					forLineNum.SetAtGrow(forDepth, forHeaderLineNum);
					forDepth++;
					goto CONTINUE;
				}
				forHeaderLineNum = -1;
				forDepth++;
FOR_LOOP_HEADER_END:
				int addEndExpression = 0; // 1 - only the expression, 2 - the expression with the closing brace
				if (headerEndEmpty) { // separator cannot be 0 in this case
					if (separator == 1) addEndExpression = 1;
					else if (separator == 2) { // the end expression will be added later
						braceDepth++;
						strcat(input1[forConditionStatementNum], " {");
						forHeaderLineNum = -1;
						len1 = strlen(endExpression);
						if (len1 > maxLen) maxLen = len1;
						forEndExpr.SetAtGrow(forDepth - 1, endExpression);
						forBraceDepth.SetAtGrow(forDepth - 1, (separator != 2 ? braceDepth : braceDepth - 1));
						forLineNum.SetAtGrow(forDepth - 1, forHeaderLineNum >= 0 ? forHeaderLineNum : nl0);
					}
					else { // separator = 3
						sprintf(eqData->message, "The header of a \"for\" loop cannot be followed by the right brace.");
						if (forHeaderLineNum < 0) l = nl - 1;
						else {
							l = nl;
							lineNum.Add(nl0);
						}
						goto SY_ERROR;
					}
				}
				else {
					if (separator == 2) { // The left brace can be placed only after the header of a conditional operator (not after a statement)
						sprintf(eqData->message, "The left brace cannot be placed after a statement.");
						if (forHeaderLineNum < 0) l = nl - 1;
						else {
							l = nl;
							lineNum.Add(nl0);
						}
						goto SY_ERROR;
					}
					for (j = 0; j < nCond; j++) {
						strncpy(testString, headerEnd + nHeaderEndBlanks, kwLen[j]);
						testString[kwLen[j]] = '\0';
						if (!strcmpi(testString, condKW[j])) break;
					}
					if (j < nCond) {
						sprintf(eqData->message, "If the first keyword inside the body of a \"for\" loop is \"if\", \"while\" or \"for\", then the entire body of the loop must be enclosed by braces.");
						if (forHeaderLineNum < 0) l = nl - 1;
						else {
							l = nl;
							lineNum.Add(nl0);
						}
						goto SY_ERROR;
					}
					len1 = strlen(input1[forConditionStatementNum]) + 2 + strlen(headerEnd);
					if (len1 > maxLen) maxLen = len1;
					char *newString = new char[len1+1];
					sprintf(newString, "%s {%s", input1[forConditionStatementNum], headerEnd);
					delete input1[forConditionStatementNum];
					input1[forConditionStatementNum] = newString;
					addEndExpression = 2;
				}

				if (addEndExpression) {
					len1 = strlen(endExpression) + addEndExpression - 1;
					if (len1 > maxLen) maxLen = len1;
					if (addEndExpression > 1) {
						input1.Add(new char[len1+1]);
						sprintf(input1[nl], "%s}", endExpression);
					}
					else if (strspn(endExpression, " \t") != len1) {
						input1.Add(new char[len1+1]);
						strcpy(input1[nl], endExpression);
					}
					else {
						input1.Add(new char[4]);
						strcpy(input1[nl], "{ }");
					}
					lineNum.Add(forHeaderLineNum >= 0 ? forHeaderLineNum : nl0);
					nl++;
					addEndExpression = 0;
					forDepth--;
					forHeaderLineNum = -1;
				}

				if (separator == 3) {
					*ptr2 = sep[separator];
					ptr2--;
				}
			}
CONTINUE:
			if (!ptr2) break;
			ptr0 = ptr2 + 1;
			ptr1 = ptr0 + strspn(ptr0, " \t");
			ptr2 = SearchOutsideTheQuotes(ptr0, ";{}");
//			ptr2 = ptr0 + strcspn(ptr0, ";{}");
			separator = 0;
			if (ptr2) {
				if (*ptr2 == ';') separator = 1;
				else if (*ptr2 == '{') separator = 2;
				else separator = 3;
			}
			forHeader = !strnicmp(ptr1, "for", 3) && *(ptr1 + 3 + strspn(ptr1 + 3, " \t")) == '(';
		}
		nl0++;

		if (!ptr || ptr - tmp >= len) break;
		if (ptr - tmp < len) ptr++;
		if (ptr - tmp < len && *ptr == 0x0a) ptr++;
		if (ptr - tmp >= len) break;
		ptr0 = ptr;
		ptr = strchr(ptr0, 0x0d);
	}
	if (!nl) {
		eqData->errorType = 2;
		if (iFunc < 0) eqData->message[0] = '\0';
		else LoadString(resInst, 2671, eqData->message, STRLEN_MAX);
		for (j = 0; j < subroutineArgCount; j++) delete subroutineArgNames[j];
		delete tmp;
		return -1;
	}

	input2_0 = new char[maxLen+1];

	for (l = 0; l < nl; l++) {
		int Write_nOut_init = -1, WriteArgCount = -1;
		bool expectingDoubleQuote = false;
		currentFormatString = "";
		if (arrayAssignmentPos > 0) {
			input2 += (arrayAssignmentPos + 1);
			arrayAssignmentPos = 0;
			isArrayAssignment = TRUE;
			opData->arrayModified = TRUE;
		}
		else if (nextPos == 0) {
			strcpy(input2_0, input1[l]);
			input2 = input2_0;
		}
		size_t nBlanks = strspn(input2, " \t");
		int len = (int)strlen(input2);
		if (nBlanks == len) {
			if (isArrayAssignment) {
				LoadString(resInst, 2698, eqData->message, STRLEN_MAX);
				goto SY_ERROR;
			}
			nextPos = 0;
			continue;
		}
		if (input2 == input2_0) {
			string = input2_0;
			BOOL modified = FALSE;
			j = -1;
			while((j = FindSign(string, j)) != -1) {
				int j0 = j;
				for (j--; j >= 0 && (string[j] == ' ' || string[j] == '\t'); j--);
				BOOL insertZero = (j < 0);
				if (!insertZero) {
					char *buffer = string.GetBuffer();
					insertZero = (strcspn(buffer + j, "+-=><&|,([\"") == 0);
					if (!insertZero && strcspn(buffer + j, "*/^") == 0) {
						String ident2;
						char *c = buffer + j0 + 1;
						double num;
						char *ptr = is_number(c, num, FALSE);
						__int64 closingPos = -1;
						size_t nBlanks = strspn(c, " \t");
						size_t len1;
						if (ptr) {
							closingPos = j0 + 1 + (ptr - c);
							ptr = NULL;
						}
						else if (c[nBlanks] == '(' || c[nBlanks] == '[') ptr = c + nBlanks;
						else if (is_ident(c, ident2)) {
							ident2 = c;
							if (strtok(ident2.GetBuffer() + strspn(c, " \t"), "+-/*^=><&|!,()[]")) {
								ident2.ReleaseBuffer();
								len1 = ident2.GetLength();
								ptr = c + len1;
							}
							else ident2.ReleaseBuffer();
						}
						if (ptr) {
							if (*ptr != '(' && *ptr != '[') {
								ident2.TrimRight();
								closingPos = j0 + 1 + len1;
							}
							else {
								int depth = 1;
								ptr++;
								while (depth && *ptr != '\0') {
									if (*ptr == '(' || *ptr == '[') depth++;
									else if (*ptr == ')' || *ptr == ']') depth--;
									ptr++;
								}
								if (!depth) closingPos = j0 + 1 + (ptr - c);
							}
						}
						if (closingPos >= 0) {
							string.ReleaseBuffer();
							string.Insert((int)closingPos, ")");
							string.Insert(j0, "(0");
							modified = TRUE;
							j = j0 + 3;
							continue;
						}
					}
					string.ReleaseBuffer();
				}
				if (insertZero) {
					string.Insert(j0, "0");
					modified = TRUE;
					j = j0 + 2;
				}
				else j = j0 + 1;
			}
			if (modified) {
				size_t stringLen = string.GetLength();
				if (stringLen > maxLen) {
					maxLen = stringLen;
					delete input2_0;
					input2_0 = new char[maxLen+1];
				}
				strcpy(input2_0, string.GetBuffer()); string.ReleaseBuffer();
			}

			string.MakeLower();
			modified = FALSE;
			for (n = 0; n < nSpecFunc; n++) {
				stdFuncName = stdFuncNames[specFunc[n]];
				stdFuncName.MakeLower();
				int nameLength = stdFuncName.GetLength();
				int j = -1;
				while((j = string.FindOutsideTheQuotes(stdFuncName, j)) != -1) {
					if (j) {
						if (strcspn(string.GetBuffer() + j - 1, "+-/*^=><&|,()[] \t")) {
							j += nameLength;
							string.ReleaseBuffer();
							continue;
						}
						string.ReleaseBuffer();
					}
					j += nameLength;
					int j_bak = j;
					j = string.Find('(', j);
					if (j < 0) break;
					for (k = j_bak; k < j && (string[k] == ' ' || string[k] == '\t'); k++);
					if (k < j) continue;
					j++;
					j_bak = j;
					int depth = 0, m = 0;
					for (k = 0; k < nSpecArgs[n]; k++) {
						int iSpec = (specFunc[n] < NSTDFUNC0 || specFunc[n] >= NSTDFUNC || !reverseArgList[specFunc[n]]) ? specArgs[n][k] : stdFuncArgInfo[specFunc[n]].count - specArgs[n][k] - 1;
						while(m < iSpec && string[j] != '\0') {
							if (string[j] == '(' || string[j] == '[') depth++;
							else if (string[j] == ')' || string[j] == ']') depth--;
							else if (!depth && string[j] == ',') m++;
							j++;
						}
						if (string[j] == '\0') break;

						BOOL charsFound = FALSE, insert = TRUE;
						int j0 = j;
						while((depth || string[j] != ',' && string[j] != ')') && string[j] != '\0') {
							if (strcspn(string.GetBuffer() + j, "+-/*^=><&|") == 0) {
								insert = FALSE;
								break;
							}
							else if (string[j] == '(' || string[j] == '[') {
								if (charsFound) {
									insert = FALSE;
									break;
								}
								depth++;
							}
							else if (string[j] == ')' || string[j] == ']') depth--;
							else if (string[j] != ' ' && string[j] != '\t') charsFound = TRUE;
							string.ReleaseBuffer();
							j++;
						}
						if (insert && !depth && string[j] != '\0') {
							string.Insert(j0, "0+");
							j += 2;
							modified = TRUE;
						}
					}
					if (k < nSpecArgs[n]) break;
					j = j_bak;
				}
				if (j >= 0) break;
			}
			if (modified && n >= nSpecFunc) {
				size_t stringLen = string.GetLength();
				if (stringLen > maxLen) {
					maxLen = stringLen;
					delete input2_0;
					input2_0 = new char[maxLen+1];
				}
				strcpy(input2_0, string.GetBuffer()); string.ReleaseBuffer();
			}
			input2 = input2_0;
			len = (int)strlen(input2);
		}
		BOOL attemptToModifyReadOnlyArray = FALSE;
		if (justOutOfIF) {
			if (cond[currentIF]) { // "while" or "for" loop
				opData->control.SetAtGrow(opData->nControl, opData->nInterm);
				opData->isConditional.SetAtGrow(opData->nControl, FALSE);
				opData->transferTo.SetAtGrow(opData->nControl, nInterm_bak[currentIF]);
				opData->newNextControl.SetAtGrow(opData->nControl, nControl_bak[currentIF]);
				opData->newNextXbreak.SetAtGrow(opData->nControl, nXbreaks_bak[currentIF]);
				opData->nControl++;
			}
			elseControlNum.SetAtGrow(currentIF, opData->nControl);
			opData->control.SetAtGrow(opData->nControl, opData->nInterm);
			opData->isConditional.SetAtGrow(opData->nControl, FALSE);
			opData->nControl++;
			opData->transferTo.SetAtGrow(headerControlNum[currentIF], opData->nInterm);
			opData->newNextControl.SetAtGrow(headerControlNum[currentIF], opData->nControl);
			opData->newNextXbreak.SetAtGrow(headerControlNum[currentIF], opData->nXbreaks);
			insideElse.SetAtGrow(currentIF, TRUE);
			rightBraceFound = FALSE;
			strncpy(testString, input2 + nBlanks, 4);
			testString[4] = '\0';
			char nextChar = *(input2 + nBlanks + 4);
			if (!strcmpi(testString, "else") && (nextChar == '\0' || nextChar == ' ' ||nextChar == '\t' ||nextChar == '{')) {
				justOutOfIF = FALSE;
				searchingForLeftBrace = TRUE;
				nextPos = nBlanks + 4;
				input2 += nextPos;
				nBlanks = strspn(input2, " \t");
				if (nBlanks == strlen(input2)) {
					nextPos = 0;
					continue;
				}
			}
			else {
				searchingForLeftBrace = FALSE;
				l--;
				goto ENDSCOPE;
			}
		}
		insideIFheader = FALSE;
		if (justOutOfIFheader || searchingForLeftBrace) {
			leftBracePresent.SetAtGrow(currentIF, (*(input2 + nBlanks) == '{'));
			if (leftBracePresent[currentIF]) nLeftBraces++;
			justOutOfIFheader = FALSE;
			if (leftBracePresent[currentIF]) {
				nextPos += (nBlanks + 1);
				input2 += (nBlanks + 1);
				nBlanks = strspn(input2, " \t");
				if (nBlanks == strlen(input2)) {
					nextPos = 0;
					searchingForLeftBrace = FALSE;
					continue;
				}
			}
			else {
				nextPos += nBlanks;
				input2 += nBlanks;
				nBlanks = 0;
			}
		}
		for (j = 0; j < nCond; j++) {
			strncpy(testString, input2 + nBlanks, kwLen[j]);
			testString[kwLen[j]] = '\0';
			if (!strcmpi(testString, condKW[j])) break;
		}
		if (j < nCond) {
			int cnd = j;
			int kwl = kwLen[j];
			size_t len1 = (size_t)(len - nBlanks - kwl);
			size_t nBlanks1 = strspn(input2 + nBlanks + kwl, " \t");
			if (nBlanks1 < len1 && *(input2 + nBlanks + kwl + nBlanks1) == '(') {
				int depth = 1;
				for (n = nBlanks + kwl + nBlanks1 + 1; n < len; n++) {
					if (input2[n] == '(') depth++;
					else if (input2[n] == ')') depth--;
					if (depth == 0) break;
				}
				if (n < len) {
					len2 = (size_t)(n - nBlanks - kwl - nBlanks1 - 1);
					size_t nBlanks2 = strspn(input2 + nBlanks + kwl + nBlanks1 + 1, " \t");
					if (nBlanks2 < len2) {
						insideIFheader = TRUE;
						justOutOfIFheader = FALSE;
						currentIF++;
						nInterm_bak.SetAtGrow(currentIF, opData->nInterm);
						nControl_bak.SetAtGrow(currentIF, opData->nControl);
						nXbreaks_bak.SetAtGrow(currentIF, opData->nXbreaks);
						cond.SetAtGrow(currentIF, cnd);
						nextPos += (nBlanks + kwl + nBlanks1 + 1);
						input2 += (nBlanks + kwl + nBlanks1 + 1);
						input2[len2] = '\0';
						nBlanks = strspn(input2, " \t");
						if (nBlanks == strlen(input2)) {
							LoadString(resInst, 2373, eqData->message, STRLEN_MAX);
							goto SY_ERROR;
						}
						insideElse.SetAtGrow(currentIF, FALSE);
					}
				}
			}
		}
		if (insideIFheader) {
			if (!searchingForLeftBrace) {
				currentIFdepth++;
				nLinkedIF.SetAtGrow(currentIFdepth, 0);
			}
			else {
				nLinkedIF[currentIFdepth]++;
				searchingForLeftBrace = FALSE;
			}
		}
		searchingForLeftBrace = FALSE;
		len = (int)strlen(input2);
		if (currentIF >= 0 && !insideIFheader && leftBracePresent[currentIF]) {
			if (!rightBraceFound) {
				char *rightBrace = SearchOutsideTheQuotes(input2, "}");
//				char *rightBrace = strchr(input2, '}');
				if (rightBrace) {
					if (*(rightBrace + 1 + strspn(rightBrace + 1, " \t")) == '\0') {
						*rightBrace = '\0';
						rightBraceFound = TRUE;
						nLeftBraces--;
						if (strlen(input2) == strspn(input2, " \t")) goto ENDSCOPE;
					}
					else {
						LoadString(resInst, 2374, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
				}
			}
		}
		else rightBraceFound = FALSE;

		int bracketDepth = 0;
		BOOL parenthesisFound = FALSE, bracketFound = FALSE;
		BOOL possibleArrayAssignment1 = FALSE, possibleArrayAssignment2 = FALSE, possibleArrayAssignment3 = FALSE;
		nLeft.SetAtGrow(bracketDepth, 0);
		nRight.SetAtGrow(bracketDepth, 0);
		char arrayName[1000];
		int startPos = 0;
		insideQuotes = false;
		for (j = 0; j < len; j++) {
			if (input2[j] == '\"') {
				insideQuotes = !insideQuotes;
				startPos = j + 1;
			}
			if (insideQuotes) continue;
			if (input2[j] == '[') {
				if (!bracketFound) {
					if (j > 0) strncpy(arrayName, input2 + startPos, j - startPos);
					arrayName[j-startPos] = '\0';
					if (strlen(arrayName) > strspn(arrayName, " \t")) {
						String arrName = arrayName + strspn(arrayName, " \t");
						arrName.TrimRight();
						strcpy(arrayName, arrName);
						if (arrayAssignmentPos <= 0 && !isArrayAssignment) possibleArrayAssignment1 = TRUE;
					}
					bracketFound = TRUE;
				}
				bracketDepth++;
				nLeft.SetAtGrow(bracketDepth, 0);
				nRight.SetAtGrow(bracketDepth, 0);
			}
			else if (input2[j] == ']') {
				bracketFound = TRUE;
				if (nLeft[bracketDepth] != nRight[bracketDepth]) break;
				bracketDepth--;
				if (bracketDepth < 0) break;
				if (possibleArrayAssignment1 && bracketDepth == 0) {
					possibleArrayAssignment1 = FALSE;
					possibleArrayAssignment2 = TRUE;
				}
			}
			else if (input2[j] == '(') {
				nLeft[bracketDepth]++;
				parenthesisFound = TRUE;
			}
			else if (input2[j] == ')') {
				nRight[bracketDepth]++;
				parenthesisFound = TRUE;
				if (nRight[bracketDepth] > nLeft[bracketDepth]) break;
			}
			else if (possibleArrayAssignment2) {
				if (input2[j] == '=') {
					possibleArrayAssignment2 = FALSE;
					possibleArrayAssignment3 = TRUE;
				}
				else if (input2[j] != ' ' && input2[j] != '\t') possibleArrayAssignment2 = FALSE;
			}
			else if (possibleArrayAssignment3) {
				if (input2[j] != '=') {
					for (k = 0; k < subroutineArgCount && strcmpi(arrayName, subroutineArgNames[k]); k++);
					if (k < subroutineArgCount) {
						if (subroutineArgInfo[iFunc].type[k] > 1 && subroutineArgInfo[iFunc].type[k] < MAXARGS + 2) {
							arrayAssignmentPos = j - 1;
							input2[arrayAssignmentPos] = '\0';
							len = arrayAssignmentPos;
						}
						else {
							LoadString(resInst, 2654, S, STRLEN_MAX);
							if (strlen(arrayName) > MAXIDENTLEN) arrayName[MAXIDENTLEN] = '\0';
							sprintf(eqData->message, S, arrayName);
							goto SY_ERROR;
						}
					}
					else {
						int nConf = 0;
						int *typeConf = (int *)malloc(0);
						int *numConf = (int *)malloc(0);
						char **nameConf = (char **)malloc(0);
						char *nameConfBuffer = (char *)malloc(0);
						char msg[STRLEN_MAX+1];
						int num = -1;
						BOOL rc = CheckIdentifier(arrayName, 5, num, NULL, nConf, typeConf, numConf, nameConf, nameConfBuffer, msg);
						free(typeConf);
						free(numConf);
						free(nameConf);
						free(nameConfBuffer);
						if (num != -1) {
							arrayAssignmentPos = j - 1;
							input2[arrayAssignmentPos] = '\0';
							len = arrayAssignmentPos;
							opData->arrayMod[num] = TRUE;
						}
						else if (!rc) {
							for (i = 0; i < NSPECARRAYS && strcmpi(arrayName, arrays[i+nArrays0].name); i++);
							if (i < NSPECARRAYS) {
								LoadString(resInst, 2781, S, STRLEN_MAX); sprintf(eqData->message, S, arrays[i+nArrays0].name);
							}
							else {
								LoadString(resInst, 2654, S, STRLEN_MAX);
								if (strlen(arrayName) > MAXIDENTLEN) arrayName[MAXIDENTLEN] = '\0';
								sprintf(eqData->message, S, arrayName);
							}
							goto SY_ERROR;
						}
						else {
							LoadString(resInst, 2654, S, STRLEN_MAX);
							if (strlen(arrayName) > MAXIDENTLEN) arrayName[MAXIDENTLEN] = '\0';
							sprintf(eqData->message, S, arrayName);
							goto SY_ERROR;
						}
					}
				}
				possibleArrayAssignment3 = FALSE;
			}
		}

		if (bracketDepth || nLeft[0] != nRight[0]) {
			if (!bracketFound) {
				LoadString(resInst, 2381, eqData->message, STRLEN_MAX);
			}
			else if (parenthesisFound) LoadString(resInst, 2651, eqData->message, STRLEN_MAX);
			else LoadString(resInst, 2652, eqData->message, STRLEN_MAX);
			goto SY_ERROR;
		}
		insideQuotes = false;
		for (j = 0; j < len; j++) {
			if (input2[j] == '\"') insideQuotes = !insideQuotes;
			if (insideQuotes) continue;
			if (input2[j] != '=') continue;
			if (j) {
				char prev = input2[j-1];
				if (prev == '<' || prev == '>' || prev == '!') continue;
			}
			if (j < len - 1) {
				if (input2[j+1] == '=') {
					j++;
					continue;
				}
			}
			break;
		}
		if (j >= len) equalityPresent = FALSE;
		else {
			if (insideIFheader) {
				LoadString(resInst, 2375, eqData->message, STRLEN_MAX);
				goto SY_ERROR;
			}
			else if (isArrayAssignment) {
				LoadString(resInst, 2388, eqData->message, STRLEN_MAX);
				goto SY_ERROR;
			}
			equalityPresent = TRUE;
		}
		if (equalityPresent && strcspn(input2, "+-*/^=><&|!") == nBlanks || !equalityPresent && strcspn(input2, "*/^=><&|!") == nBlanks) {
			LoadString(resInst, 2376, eqData->message, STRLEN_MAX);
			goto SY_ERROR;
		}
		if (equalityPresent && strcspn(input2, "()[]") == nBlanks || !equalityPresent && strcspn(input2, ")]") == nBlanks) {
			LoadString(resInst, 2377, eqData->message, STRLEN_MAX);
			goto SY_ERROR;
		}
		if (strcspn(input2, ",") == nBlanks) {
			LoadString(resInst, 2378, eqData->message, STRLEN_MAX);
			goto SY_ERROR;
		}

		if ((int)maxLen > codeLength) {
			delete tmp;
			delete input0;
			delete input;
			codeLength = (int)maxLen;
			int tmpLen = codeLength + 10;
			if (iFunc >= 0) tmpLen += (int)strlen(subroutineArgInfo[iFunc].list_singleLine) + 1;
//			if (nArrInd > 0) tmpLen += (int)strlen(arrayIndices) + 1;
			tmp = new char[tmpLen];
			input0 = new char[tmpLen];
			input = new char[tmpLen];
		}

		strcpy(input0, input2);
		strrev(input0);
		nBlanks = strspn(input0, " \t");
		if (strcspn(input0, "+-*/^=><&|!") == nBlanks) {
			LoadString(resInst, 2379, eqData->message, STRLEN_MAX);
			goto SY_ERROR;
		}
		if (strcspn(input0, ",") == nBlanks) {
			LoadString(resInst, 2380, eqData->message, STRLEN_MAX);
			goto SY_ERROR;
		}
		if (equalityPresent) {
			strcpy(input0, input2);
			equalityFound = FALSE;
		}
		else if (arrayAssignmentPos <= 0) {
			if (strpbrk(input2, "+-") == input2 + strspn(input2, " \t")) sprintf(input0, "0%s", input2);
			else {
				char *endPtr;
				double num = strtod(input2 + strspn(input2, " \t(["), &endPtr);
				if (!SearchOutsideTheQuotes(input2, "+-/*^=><&|") || strspn(endPtr, " \t)]") == strlen(endPtr)) {
//				if (!strpbrk(input2, "+-/*^=><&|") || strspn(endPtr, " \t)]") == strlen(endPtr)) {
					BOOL charsFound = FALSE;
					for (j = 0; j < len; j++) {
						if (input2[j] == '(' || input2[j] == '[') {
							if (charsFound) break;
						}
						else if (input2[j] != ' ' && input2[j] != '\t') charsFound = TRUE;
					}
					if (j >= len) sprintf(input0, "0+%s", input2);
					else strcpy(input0, input2);
				}
				else strcpy(input0, input2);
			}
			equalityFound = TRUE;
		}
		else {
			strcpy(input0, input2);
			equalityFound = TRUE;
		}

		strcpy(input, input0);
		strpos = input;
		c_prev = input;
		strend = input + strlen(input);
		opc = '\0';
		sl = 0;
		prevTokenIsIdent = FALSE;
		prevTokenIsFunc = FALSE;
		prevTokenIsArray = FALSE;
		prevTokenIsComma = FALSE;
		functionScopeDepth = -1;
		subTp = -1;
		int insideLoc = 0;
		BOOL start = TRUE;
		while(strpos < strend) {
			// The "core" of the body of this loop is a C sample code that was included in the Wikipedia article Shunting-yard algorithm. That sample code was later removed (http://en.wikipedia.org/wiki/Shunting-yard_algorithm)
			if (strpos != input || start) {
				start = FALSE;
				opc = *(input0 + (strpos - input));
				if (insideLoc == 3 && prevTokenIsFunc && opc != ')') {
					LoadString(resInst, 2810, S, STRLEN_MAX);
					if (prevToken.GetLength() > MAXIDENTLEN) prevToken = (char *)(prevToken.Left(MAXIDENTLEN));
					sprintf(eqData->message, S, prevToken.GetBuffer()); prevToken.ReleaseBuffer();
					goto SY_ERROR;
				}
				if (functionScopeDepth >= 0 && curFuncNum >= NSTDFUNC0 && curFuncNum < NSTDFUNC && insideLoc != 3 &&
					reverseArgList[curFuncNum] && functionScopeDepth == functionScopeDepth0[curFuncLevel]) {
					if (opc == ')' && argStartPos[curFuncLevel] > listStartPos[curFuncLevel]) opc = ',';
					else if (opc == ',' && argStartPos[curFuncLevel] == listStartPos[curFuncLevel]) {
						strpos = input + listEndPos[curFuncLevel];
						opc = *(input0 + (strpos - input));
					}
				}
				// If the token is a function argument separator (e.g., a comma):
				if (opc == ',') {
					if (!equalityFound || functionScopeDepth < 0) {
						LoadString(resInst, 2382, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					if (functionScopeDepth >= 0 && functionScopeDepth == functionScopeDepth0[curFuncLevel]) {
						if (curFuncNum >= NBINARYOP && curFuncNum < NBINARYOP + 10) {
							expectingDoubleQuote = true;
							if (argNum[curFuncLevel] > 1) { // the case "== 1" is handled elsewhere
								formatStrings.Add(currentFormatString);
//								formatStrings.Add(defaultNumberFormat);
							}
							currentFormatString = "";
							prevTokenIsComma = TRUE; // to be used only if the next token (not counting the string arguments) is the right parenthesis
						}
						argNum[curFuncLevel]++;
					}
					if (curFuncNum >= NSTDFUNC0 && curFuncNum < NSTDFUNC && reverseArgList[curFuncNum]) {
						strpos = input + argStartPos[curFuncLevel] - 1;
						if (*strpos == ',') {
							strpos--;
						}
						int depth = 1;
						while (depth > 1 || depth == 1 && *strpos != ',') {
							char c1 = *(input0 + (strpos - input));
							if (c1 == ')' || c1 == ']') depth++;
							else if (c1 == '(' || c1 == '[') depth--;
							if (!depth) break;
							strpos--;
						}
						strpos++;
						argStartPos[curFuncLevel] = (int)(strpos - input);
					}
					else strpos++;
					type = 0;
					subTp = 19;
					if (!OnComma(sl, nOut)) {
						// If no left parentheses are encountered, either the separator was misplaced
						// or parentheses were mismatched.
						LoadString(resInst, 2383, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					if (curFuncNum >= NSTDFUNC0 + 4 && curFuncNum <= NSTDFUNC0 + 14 && argNum[curFuncLevel] == correctArgNum[curFuncLevel]) {
						// Integration, summation and iteration functions, end of the argument defining the integration variable or index of iteration
						addType = 10000;
					}
					insideLoc = 0;
				}
				// If the token is a left parenthesis or a left bracket, then push it onto the stack.
				else if (opc == '(' || opc == '[') {
					strpos++;
					if (subTp == 21) {
						LoadString(resInst, 2384, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					if (!equalityFound) {
						LoadString(resInst, 2385, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					if (opc == '(' && (prevTokenIsIdent || prevTokenIsArray)) {
						LoadString(resInst, 2386, S, STRLEN_MAX);
						if (prevToken.GetLength() > MAXIDENTLEN) prevToken = (char *)(prevToken.Left(MAXIDENTLEN));
						sprintf(eqData->message, S, prevToken.GetBuffer()); prevToken.ReleaseBuffer();
						goto SY_ERROR;
					}
					if (opc == '[' && (prevTokenIsIdent || prevTokenIsFunc)) {
						LoadString(resInst, 2654, S, STRLEN_MAX);
						if (prevToken.GetLength() > MAXIDENTLEN) prevToken = (char *)(prevToken.Left(MAXIDENTLEN));
						sprintf(eqData->message, S, prevToken.GetBuffer()); prevToken.ReleaseBuffer();
						goto SY_ERROR;
					}
					if (prevTokenIsArray && insideLoc == 3) { // push the token onto the stack
						sl0.SetAtGrow(curFuncLevel, sl);
						stack_symbolType.SetAtGrow(sl, 1);
						stack_subType.SetAtGrow(sl, id);
						++sl;
						insideLoc = 0;
					}
					if (functionScopeDepth >= 0) functionScopeDepth++;
					if (prevTokenIsFunc || prevTokenIsArray) {
						if (functionScopeDepth < 0) {
							functionScopeDepth = 0;
							curFuncLevel = 0;
						}
						else curFuncLevel++;
						functionScopeDepth0.SetAtGrow(curFuncLevel, functionScopeDepth);
						argNum.SetAtGrow(curFuncLevel, 0);
						funcNum.SetAtGrow(curFuncLevel, curFuncNum);
						correctArgNum.SetAtGrow(curFuncLevel, op_arg_count(funcNum[curFuncLevel], iFunc));
						funcIsArray.SetAtGrow(curFuncLevel, FALSE);
						if (curFuncNum < NSTDFUNC) {
							nestedFuncName.SetAtGrow(curFuncLevel, stdFuncNames[curFuncNum]);
							if (curFuncNum >= NSTDFUNC0 && reverseArgList[curFuncNum]) { // Integration, summation and iteration functions
								listStartPos.SetAtGrow(curFuncLevel, (int)(strpos - input));
								int depth = 1;
								while (*strpos != '\0') {
									if (*strpos == '(' || *strpos == '[') depth++;
									else if (*strpos == ')' || *strpos == ']') depth--;
									if (!depth) break;
									strpos++;
								}
								listEndPos.SetAtGrow(curFuncLevel, (int)(strpos - input));
								if (!depth) depth = 1; // closing ')' found
								while (depth > 1 || depth == 1 && *strpos != ',') {
									strpos--;
									char c1 = *(input0 + (strpos - input));
									if (c1 == ')' || c1 == ']') depth++;
									else if (c1 == '(' || c1 == '[') depth--;
								}
								strpos++;
								argStartPos.SetAtGrow(curFuncLevel, (int)(strpos - input));
							}
							if (curFuncNum >= NBINARYOP && curFuncNum < NBINARYOP + 10) {
								if (Write_nOut_init >= 0) {
									sprintf(eqData->message, "Error: Functions %s, %s, %s, %s, %s, %s, %s, %s, %s and %s cannot be specified more than once in a single statement.",
										stdFuncNames[NBINARYOP], stdFuncNames[NBINARYOP+1], stdFuncNames[NBINARYOP+2], stdFuncNames[NBINARYOP+3], stdFuncNames[NBINARYOP+4], stdFuncNames[NBINARYOP+5], stdFuncNames[NBINARYOP+6], stdFuncNames[NBINARYOP+7], stdFuncNames[NBINARYOP+8], stdFuncNames[NBINARYOP+9]);
									goto SY_ERROR;
								}
								Write_nOut_init = nOut;
								expectingDoubleQuote = true;
							}
						}
						else if (curFuncNum < NSTDFUNC + subroutineArgCount) {
							nestedFuncName.SetAtGrow(curFuncLevel, subroutineArgNames[curFuncNum-NSTDFUNC]);
							if (subroutineArgInfo[iFunc].type[funcNum[curFuncLevel]-NSTDFUNC] < 22) funcIsArray[curFuncLevel] = TRUE;
						}
						else if (curFuncNum < NSTDFUNC + subroutineArgCount + nSubroutines) nestedFuncName.SetAtGrow(curFuncLevel, subroutine[curFuncNum-NSTDFUNC-subroutineArgCount].name);
//						else if (curFuncNum < NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc) nestedFuncName.SetAtGrow(curFuncLevel, dllFuncName2[curFuncNum-NSTDFUNC-subroutineArgCount-nSubroutines]);
						else {
							nestedFuncName.SetAtGrow(curFuncLevel, arrays[curFuncNum-NSTDFUNC-subroutineArgCount-nSubroutines-nDllFunc].name);
							funcIsArray.SetAtGrow(curFuncLevel, TRUE);
						}
					}
					subTp = 20;
					stack_symbolType.SetAtGrow(sl, 0);
					stack_subType.SetAtGrow(sl, 20);
					++sl;
				}
				// If the token is a right parenthesis or a right bracket:
				else if (opc == ')' || opc == ']') {
					if (opc == ')' && insideLoc == 3 && (prevTokenIsFunc || prevTokenIsArray)) { // If the function or array identifier is used as an argument of "loc", add it to the output queue 
						if (prevTokenIsFunc && curFuncNum >= NSTDFUNC + subroutineArgCount && curFuncNum < NSTDFUNC + subroutineArgCount + nSubroutines) {
							int iSubroutine = curFuncNum - NSTDFUNC - subroutineArgCount;
							if (iSubroutine != iFunc && iUnloadableSubroutine < 0 && subroutine[iSubroutine].errorType > 1) {
								iUnloadableSubroutine = iSubroutine;
								unloadableSubroutineLineNum = l;
							}
						}
						symbolType.SetAtGrow(nOut, 3 + addType); addType = 0;
						idents.SetAtGrow(nOut, prevToken.GetBuffer()); prevToken.ReleaseBuffer();
						nOut++;
						prevTokenIsFunc = FALSE;
						prevTokenIsArray = FALSE;
					}
					insideLoc = 0;
					strpos++;
					if (!equalityFound) {
						LoadString(resInst, 2385, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					if (functionScopeDepth >= 0) {
						if (functionScopeDepth == functionScopeDepth0[curFuncLevel]) {
							if (funcNum[curFuncLevel] >= NBINARYOP && funcNum[curFuncLevel] < NBINARYOP + 10) { // Dir, Dir2, Write, Write2, Print, Print2, System, FopenW, FopenR or SetNumberFormat
								if (prevTokenIsComma) {
									if (!currentFormatString.GetLength()) {
										LoadString(resInst, 2407, eqData->message, STRLEN_MAX);
										goto SY_ERROR;
									}
									argNum[curFuncLevel]--;
									formatStrings.Add(currentFormatString);
								}
								else if (!argNum[curFuncLevel]){
									formatStrings.Add(currentFormatString);
									currentFormatString = "";
								}
								else {
									if (argNum[curFuncLevel] > 1) { // the case "==1" is handled elsewhere
										formatStrings.Add(currentFormatString);
//										formatStrings.Add(defaultNumberFormat);
									}
									currentFormatString = "";
									formatStrings.Add(currentFormatString);
								}
								expectingDoubleQuote = false;
								currentFormatString = "";
								WriteArgCount = argNum[curFuncLevel];
								int nFormatStrings = formatStrings.GetSize();
								int firstFormatStringIndex = nFormatStrings - (WriteArgCount + 1);
//								int firstFormatStringIndex = nFormatStrings - (2 * WriteArgCount + 1);
								if (Write_nOut_init >= idents.GetSize()) idents.SetAtGrow(Write_nOut_init, "");
								else idents.InsertAt(Write_nOut_init, "");
								idents.InsertAt(Write_nOut_init, "");
								if (Write_nOut_init >= symbolType.GetSize()) symbolType.SetAtGrow(Write_nOut_init, 2);
								else symbolType.InsertAt(Write_nOut_init, 2);
								symbolType.InsertAt(Write_nOut_init, 2);
								if (Write_nOut_init >= subType.GetSize()) subType.SetAtGrow(Write_nOut_init, 0);
								else subType.InsertAt(Write_nOut_init, 0);
								subType.InsertAt(Write_nOut_init, 0);
								if (Write_nOut_init >= constants.GetSize()) constants.SetAtGrow(Write_nOut_init, WriteArgCount);
								else constants.InsertAt(Write_nOut_init, WriteArgCount);
								constants.InsertAt(Write_nOut_init, firstFormatStringIndex);
								nOut += 2;
							}
							else if (funcNum[curFuncLevel] < NSTDFUNC || funcNum[curFuncLevel] < NSTDFUNC + subroutineArgCount && subroutineArgInfo[iFunc].type[funcNum[curFuncLevel]-NSTDFUNC] > MAXARGS + 1 ||
								funcNum[curFuncLevel] >= NSTDFUNC + subroutineArgCount && funcNum[curFuncLevel] < NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc) {
								if (argNum[curFuncLevel] < correctArgNum[curFuncLevel]) {
									LoadString(resInst, 2737, S, STRLEN_MAX); sprintf(eqData->message, S, nestedFuncName[curFuncLevel].GetBuffer()); nestedFuncName[curFuncLevel].ReleaseBuffer();
									goto SY_ERROR;
								}
								if (argNum[curFuncLevel] > correctArgNum[curFuncLevel]) {
									LoadString(resInst, 2738, S, STRLEN_MAX); sprintf(eqData->message, S, nestedFuncName[curFuncLevel].GetBuffer()); nestedFuncName[curFuncLevel].ReleaseBuffer();
									goto SY_ERROR;
								}
							}
							else if (argNum[curFuncLevel] > correctArgNum[curFuncLevel]) {
								LoadString(resInst, 2740, S, STRLEN_MAX); sprintf(eqData->message, S, nestedFuncName[curFuncLevel].GetBuffer()); nestedFuncName[curFuncLevel].ReleaseBuffer();
								goto SY_ERROR;
							}
							else if ((curFuncLevel <= 0 || funcNum[curFuncLevel-1] != NSTDFUNC0) && argNum[curFuncLevel] < correctArgNum[curFuncLevel]) {
								LoadString(resInst, 2739, S, STRLEN_MAX); sprintf(eqData->message, S, nestedFuncName[curFuncLevel].GetBuffer()); nestedFuncName[curFuncLevel].ReleaseBuffer();
								goto SY_ERROR;
							}
							else {
								if (curFuncLevel > 0 && funcNum[curFuncLevel-1] == NSTDFUNC0) { // loc(arrayName[1,2,...])
									for (k = argNum[curFuncLevel]; k <= correctArgNum[curFuncLevel]; k++) {
										if (k && !OnComma(sl, nOut)) {
											// If no left parentheses are encountered, either the separator was misplaced
											// or parentheses were mismatched.
											LoadString(resInst, 2383, eqData->message, STRLEN_MAX);
											goto SY_ERROR;
										}
										symbolType.SetAtGrow(nOut, 2 + addType); addType = 0;
										subType.SetAtGrow(nOut, 0);
										constants.SetAtGrow(nOut, argNum[curFuncLevel]);
										nOut++;
									}
									stack_symbolType.SetAtGrow(sl0[curFuncLevel-1], -argNum[curFuncLevel] - 1);
								}
								else {
									if (!OnComma(sl, nOut)) {
										// If no left parentheses are encountered, either the separator was misplaced
										// or parentheses were mismatched.
										LoadString(resInst, 2383, eqData->message, STRLEN_MAX);
										goto SY_ERROR;
									}
									symbolType.SetAtGrow(nOut, 2 + addType); addType = 0;
									subType.SetAtGrow(nOut, 0);
									constants.SetAtGrow(nOut, -1);
									nOut++;
								}
							}
							curFuncLevel--;
							if (curFuncLevel >= 0) curFuncNum = funcNum[curFuncLevel];
						}
						functionScopeDepth--;
					}
					subTp = 21;
					bool pe = false;
					// Until the token at the top of the stack is a left parenthesis,
					// pop operators off the stack onto the output queue
									int size_symbolType = stack_symbolType.GetSize();
									int size_subType = stack_subType.GetSize();
									int size_constants = stack_constants.GetSize();
					while (sl > 0) {
						if (sl <= size_symbolType) sc_type = stack_symbolType[sl-1];
						if (sl <= size_subType) sc_subType = stack_subType[sl-1];
						if (sl <= size_constants) sc_number = stack_constants[sl-1];
						if(sc_type == 0 && sc_subType == 20) {
							pe = true;
							break;
						}
						else {
							symbolType.SetAtGrow(nOut, sc_type + addType); addType = 0;
							subType.SetAtGrow(nOut, sc_subType);
							constants.SetAtGrow(nOut, sc_number);
							nOut++;
							sl--;
						}
					}
					if(!pe)  {
						// If the stack runs out without finding a left parenthesis, then there are mismatched parentheses.
						LoadString(resInst, 2387, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					// Pop the left parenthesis from the stack, but not onto the output queue.
					sl--;
					// If the token at the top of the stack is a function token or an array token, pop it onto the output queue.
					if(sl > 0)   {
						if (sl <= size_symbolType) sc_type = stack_symbolType[sl-1];
						if (sl <= size_subType) sc_subType = stack_subType[sl-1];
						if(sc_type == 1)   {
							symbolType.SetAtGrow(nOut, sc_type + addType); addType = 0;
							subType.SetAtGrow(nOut, sc_subType);
							nOut++;
							sl--;
						}
					}
				}
				// If the token is an operator, op1, then:
				else if(is_operator(opc, subTp, strpos)) {
					if (subTp == 5) {
						if (equalityFound) {
							LoadString(resInst, 2388, eqData->message, STRLEN_MAX);
							goto SY_ERROR;
						}
						int symbolType_temp = symbolType[nOut-1];
						if (symbolType_temp > 5000 && symbolType_temp <= 15000) symbolType_temp -= 10000;
						if (symbolType_temp != 3) { // '='
							LoadString(resInst, 2389, eqData->message, STRLEN_MAX);
							goto SY_ERROR;
						}
						equalityFound = TRUE;
					}
					else if (!equalityFound) {
						LoadString(resInst, 2390, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					if (curFuncLevel >= 0 && funcNum[curFuncLevel] == NSTDFUNC0) { // loc
						LoadString(resInst, 2732, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					insideLoc = 0;
									int size_symbolType = stack_symbolType.GetSize();
									int size_subType = stack_subType.GetSize();
									int size_constants = stack_constants.GetSize();
					while(sl > 0)    {
						if (sl <= size_symbolType) sc_type = stack_symbolType[sl-1];
						if (sl <= size_subType) sc_subType = stack_subType[sl-1];
						// While there is an operator token, op2, at the top of the stack
						// op1 is left-associative and its precedence is less than or equal to that of op2,
						// or op1 is right-associative and its precedence is less than that of op2,
						if(sc_type == 0 && sc_subType < NBINARYOP && 
							((op_left_assoc(subTp) && (op_preced(subTp) <= op_preced(sc_subType))) ||
							   (!op_left_assoc(subTp) && (op_preced(subTp) < op_preced(sc_subType))))) {
							// Pop o2 off the stack, onto the output queue;
							symbolType.SetAtGrow(nOut, addType); addType = 0;
							subType.SetAtGrow(nOut, sc_subType);
							nOut++;
							sl--;
						}
						else   {
							break;
						}
					}
					// push op1 onto the stack.
					stack_symbolType.SetAtGrow(sl, 0);
					stack_subType.SetAtGrow(sl, subTp);
					++sl;
				}
				else if (opc == '!') {
					LoadString(resInst, 2391, eqData->message, STRLEN_MAX);
					goto SY_ERROR;
				}
			}
			if ((prevTokenIsFunc || prevTokenIsArray) && insideLoc < 2) {
				if (subTp != 20) {
					if (prevTokenIsFunc) LoadString(resInst, 2392, S, STRLEN_MAX);
					else LoadString(resInst, 2653, S, STRLEN_MAX);
					sprintf(eqData->message, S, prevToken.GetBuffer()); prevToken.ReleaseBuffer();
					goto SY_ERROR;
				}
				if (insideLoc == 1) insideLoc = 2;
			}
			prevTokenIsIdent = FALSE;
			prevTokenIsFunc = FALSE;
			prevTokenIsArray = FALSE;
			char *symbols;
			// read one token from the input stream
			strpos += strspn(strpos, " \t");
			if (expectingDoubleQuote) {
				while (*strpos == '\"') {
					strpos++;
					char *startPtr = strpos;
					strpos = strchr(strpos, '\"');
					while(strpos && strpos[1] == '\"') {
						strpos[1] = '\0';
						currentFormatString += startPtr;
						strpos[1] = '\"';
						strpos += 2;
						startPtr = strpos;
						strpos = strchr(strpos, '\"');
					}
					if (strpos) {
						*strpos = '\0';
						currentFormatString += startPtr;
						*strpos = '\"';
						strpos++;
						strpos += strspn(strpos, ", \t");
					}
					else {
						sprintf(eqData->message, "Error: Missing closing double quote");
						goto SY_ERROR;
					}
				}
				expectingDoubleQuote = false;
			}
			if (subTp < 21) symbols = "()[]";
			else symbols = "+-/*^=><&|!,()[]";
			if (strpos < strend && strpbrk(strpos, symbols) != strpos) { // the character at strpos is not any of the symbols
				c = strpos;
				// If the token is a number, then add it to the output queue.
				BOOL isNumber = (is_number(c, num) != NULL);
				if(isNumber) {
					if (!equalityFound) {
						LoadString(resInst, 2393, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					if (subTp == 21) {
						LoadString(resInst, 2394, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					symbolType.SetAtGrow(nOut, 2 + addType); addType = 0;
					constants.SetAtGrow(nOut, num);
					nOut++;
					if (c - input > 0) c_prev1 = c - 1;
				}
				else {
					c = strtok(c, "+-/*^=><&|!,()[]");
					if (!c) {
						LoadString(resInst, 2395, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
					if (equalityFound && subTp < 21 && c_prev < c - 1) {
						if (*(c - 1) == '-') {
							c_prev1 = c - 2;
						}
						else if (*(c - 1) == '+') {
							c_prev1 = c - 2;
						}
						else {
							if (c - input > 0) c_prev1 = c - 1;
						}
					}
					else {
						if (c - input > 0) c_prev1 = c - 1;
					}
				}
				if (c - input > 0 && c_prev1 - strpos >= 0) {
					char *ptr = strpbrk(strpos, "+-/*^=><&|!,()[]");
					if (ptr && c_prev1 - ptr >= 0) {
						LoadString(resInst, 2396, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
				}
				if (!isNumber) {
					// If the token is a function or array token, then push it onto the stack.
					if(is_function(c, id, iFunc, subroutineArgNames)) {
						curFuncNum = id;
						prevTokenIsFunc = (id < NSTDFUNC || id < NSTDFUNC + subroutineArgCount && subroutineArgInfo[iFunc].type[id-NSTDFUNC] > MAXARGS + 1 ||
							id >= NSTDFUNC + subroutineArgCount && id < NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc);
						if (!insideLoc) {
							if (id == NSTDFUNC0) insideLoc = 1;
						}
						else if (insideLoc == 2) insideLoc = 3;
						prevTokenIsArray = !prevTokenIsFunc;

						if (prevTokenIsArray && curFuncLevel > 0 && funcNum[curFuncLevel] == NSTDFUNC0) {
							if (funcNum[curFuncLevel-1] == NSTDFUNC0 + 1 || // "Invert(loc(...))"
								funcNum[curFuncLevel-1] == NSTDFUNC0 + 18 && argNum[curFuncLevel-1] == 3 || // "Hist(...)", arg.No.3
								funcNum[curFuncLevel-1] == NSTDFUNC0 + 19 && argNum[curFuncLevel-1] == 3) { // "Hist2(...)", arg.No.3
								int arrayNum = id - NSTDFUNC - subroutineArgCount - nSubroutines - nDllFunc;
								if (arrayNum >= nArrays0) {
									LoadString(resInst, 2781, S, STRLEN_MAX); sprintf(eqData->message, S, arrays[arrayNum].name);
									attemptToModifyReadOnlyArray = TRUE;
								}
								else if (arrayNum >= 0) opData->arrayMod[arrayNum] = TRUE;
							}
						}

						int id2;
						if (prevTokenIsArray) {
							opData->arraysUsed = TRUE;
							id2 = id - NSTDFUNC - subroutineArgCount - nSubroutines - nDllFunc;
							if (id >= NSTDFUNC + subroutineArgCount) {
								opData->arrayUsed[id2] = TRUE;
								if (id2 < nArrays0) opData->userArraysUsed = TRUE;
							}
						}
						else if (id >= RAND_TIME_START && id <= RAND_TIME_END) {
							opData->randTimeUsed = TRUE;
						}
						else if (id >= NSTDFUNC + subroutineArgCount) {
							if (id < NSTDFUNC + subroutineArgCount + nSubroutines) {
								opData->subroutinesUsed = TRUE;
								opData->subroutineUsed[id-NSTDFUNC-subroutineArgCount] = TRUE;
							}
							else {
								opData->dllsUsed = TRUE;
							}
						}

						if (!equalityFound) {
							if (prevTokenIsArray) {
								if (id2 >= nArrays0) {
									LoadString(resInst, 2782, S, STRLEN_MAX); sprintf(eqData->message, S, arrays[id2].name);
								}
								else {
									LoadString(resInst, 2783, S, STRLEN_MAX);
									if (id2 >= 0) sprintf(eqData->message, S, arrays[id2].name);
									else sprintf(eqData->message, S, subroutineArgNames[id-NSTDFUNC]);
								}
							}
							else if (id < NSTDFUNC) {
								LoadString(resInst, 2784, S, STRLEN_MAX); sprintf(eqData->message, S, stdFuncNames[id]);
							}
							else if (id < NSTDFUNC + subroutineArgCount) {
								LoadString(resInst, 2785, S, STRLEN_MAX); sprintf(eqData->message, S, subroutineArgNames[id-NSTDFUNC]);
							}
							else if (id < NSTDFUNC + subroutineArgCount + nSubroutines) {
								LoadString(resInst, 2786, S, STRLEN_MAX); sprintf(eqData->message, S, subroutine[id-NSTDFUNC-subroutineArgCount].name);
							}
//							else {
//								LoadString(resInst, 2787, S, STRLEN_MAX); sprintf(eqData->message, S, dllFuncName2[id-NSTDFUNC-subroutineArgCount-nSubroutines]);
//							}
							goto SY_ERROR;
						}
						if (subTp == 21) {
							if (prevTokenIsFunc) {
								LoadString(resInst, 2398, eqData->message, STRLEN_MAX);
							}
							else {
								LoadString(resInst, 2655, eqData->message, STRLEN_MAX);
							}
							goto SY_ERROR;
						}
						if (insideLoc != 3) { // push the token onto the stack
							stack_symbolType.SetAtGrow(sl, 1);
							stack_subType.SetAtGrow(sl, id);
							++sl;
						}
						prevToken = c;
						prevToken.TrimRight();
					}
					// If the token is an identifier, then add it to the output queue.
					else if(is_ident(c, ident)) {
						if (subTp == 21) {
							LoadString(resInst, 2399, S, STRLEN_MAX);
							strcpy(tmp, ident);
							tmp[MAXIDENTLEN] = '\0';
							sprintf(eqData->message, S, tmp);
							goto SY_ERROR;
						}
						symbolType.SetAtGrow(nOut, 3 + addType); addType = 0;
						idents.SetAtGrow(nOut, ident);
						nOut++;
						prevTokenIsIdent = TRUE;
						prevToken = c;
						prevToken.TrimRight();
					}
					else {
						size_t firstPos = strspn(c, " \t");
						char separator = '\0';
						if (strcspn(c, ".?;:{}\\'\"") == firstPos) {
							separator = c[firstPos];
						}
						else {
							char *c_rev = new char[strlen(c)+1];
							strcpy(c_rev, c);
							strrev(c_rev);
							firstPos = strspn(c_rev, " \t");
							if (strcspn(c_rev, ".?;:{}\\'\"") == firstPos) {
								separator = c_rev[firstPos];
							}
							delete c_rev;
						}
						int opId;
						char *tmpPtr = NULL;
						if (separator != '\0') {
							if (separator == '}' && nLeftBraces > 0 && currentIF >= 0 && !leftBracePresent[currentIF]) {
								LoadString(resInst, 2646, eqData->message, STRLEN_MAX); 
							}
							else if (separator == '\"' || separator == '.' || separator == ';' || separator == '{' || separator == '}') {
								LoadString(resInst, 2906, S, STRLEN_MAX); sprintf(eqData->message, S, separator);
							}
							else {
								LoadString(resInst, 2400, S, STRLEN_MAX); sprintf(eqData->message, S, separator);
							}
						}
						else if (c != input && (*c == ' ' || !is_operator(input0[c-1-input], opId, tmpPtr))) {
							LoadString(resInst, 2401, S, STRLEN_MAX);
							if (strlen(c) > MAXIDENTLEN) c[MAXIDENTLEN] = '\0';
							sprintf(eqData->message, S, input0[c-1-input], c);
						}
						else {
							LoadString(resInst, 2402, S, STRLEN_MAX);
							if (strlen(c) > MAXIDENTLEN) c[MAXIDENTLEN] = '\0';
							sprintf(eqData->message, S, c);
						}
						goto SY_ERROR;
					}
				}
				subTp = -1;
				strpos = c + strlen(c);
				if (functionScopeDepth >= 0 && !argNum[curFuncLevel]) {
					argNum[curFuncLevel] = 1;
					if (funcNum[curFuncLevel] >= NBINARYOP && funcNum[curFuncLevel] < NBINARYOP + 10) {
						formatStrings.Add(currentFormatString);
//						formatStrings.Add(defaultNumberFormat);
						currentFormatString = "";
					}
				}
				prevTokenIsComma = FALSE;
			}
			c_prev = strpos;
		}

		if ((prevTokenIsFunc || prevTokenIsArray) && subTp != 20) {
			if (prevTokenIsFunc) LoadString(resInst, 2392, S, STRLEN_MAX);
			else LoadString(resInst, 2653, S, STRLEN_MAX);
			sprintf(eqData->message, S, prevToken.GetBuffer()); prevToken.ReleaseBuffer();
			goto SY_ERROR;
		}
		// When there are no more tokens to read:
		// While there are still operator tokens in the stack:
									int size_symbolType = stack_symbolType.GetSize();
									int size_subType = stack_subType.GetSize();
									int size_constants = stack_constants.GetSize();
		while (sl > 0) {
			if (sl <= size_symbolType) sc_type = stack_symbolType[sl-1];
			if (sl <= size_subType) sc_subType = stack_subType[sl-1];
			if (sl <= size_constants) sc_number = stack_constants[sl-1];
			if(sc_type == 0 && (sc_subType == 20 || sc_subType == 21))   {
				LoadString(resInst, 2387, eqData->message, STRLEN_MAX);
				goto SY_ERROR;
			}
			symbolType.SetAtGrow(nOut, sc_type + addType); addType = 0;
			subType.SetAtGrow(nOut, sc_subType);
			constants.SetAtGrow(nOut, sc_number);
			nOut++;
			--sl;
		}
		sl = 0;
		if (currentIF >= 0 && !insideIFheader && nOut && !statementFound[currentIFdepth] && currentIFdepth <= lowestWaitingLevel) {
			statementFound.SetAtGrow(currentIFdepth, 1);
			lowestWaitingLevel--;
		}
		BOOL incompatibleArgs = FALSE;
		int loc_array = 0;
		int newVarNum;
		// While there are input tokens left
		while(i < nOut) {
			// The "core" of the body of this loop is a C sample code that was included in the Wikipedia article Shunting-yard algorithm. That sample code was later removed (http://en.wikipedia.org/wiki/Shunting-yard_algorithm)
			int j;
			if (symbolType[i] > 5000 && symbolType[i] <= 15000) {
				symbolType[i] -= 10000;
				opData->control.SetAtGrow(opData->nControl, opData->nInterm);
				opData->isConditional.SetAtGrow(opData->nControl, FALSE);
				opData->transferTo.SetAtGrow(opData->nControl, -1);
				opData->newNextControl.SetAtGrow(opData->nControl, -1);
				opData->newNextXbreak.SetAtGrow(opData->nControl, -1);
				integralControl.Add(opData->nControl);
				integralProcessed.Add(FALSE);
				curIntegralControl = nIntegralControlsStored;
				nIntegralControlsStored++;
				opData->nControl++;
				if (symbolType[i-1] == 3) {
					for (j = NBINARYOP; j < NSTDFUNC && strcmpi(stdFuncNames[j], stack_idents[sl-1]); j++);
					if (j >= NSTDFUNC) {
						for (j = 0; j < nSubroutines && strcmpi(subroutine[j].name, stack_idents[sl-1]); j++);
						if (j >= nSubroutines) {
//							for (j = 0; j < nDllFunc && strcmpi(dllFuncName2[j], stack_idents[sl-1]); j++);
//							if (j >= nDllFunc) {
								for (j = 0; j < nArrays && strcmpi(arrays[j].name, stack_idents[sl-1]); j++);
								if (j >= nArrays) {
									integrationVar_stackPos.Add(sl - 1);
									integrationVarNum.Add(opData->nOperands);
									nIntegrationVarsStored++;
									opData->SetOperandAt(opData->nOperands, 0.);
									opData->isConst.SetAtGrow(opData->nOperands, 0);
									opData->nOperands++;
									opData->checkSum += (opData->nOperands + 1) * 2;
								}
//							}
						}
					}
				}
			}
			// If the token is a value
			if(symbolType[i] == 2) {
				// Push it onto the stack.
				stack_symbolType.SetAtGrow(sl, 2);
				stack_constants.SetAtGrow(sl, constants[i]);
				++sl;
			}
			// If the token is an identifier
			else if(symbolType[i] == 3) {
				// Push it onto the stack.
				stack_symbolType.SetAtGrow(sl, 3);
				stack_idents.SetAtGrow(sl, idents[i]);
				++sl;
			}
			// If the token is an intermediate result
			else if(symbolType[i] >= 4) {
				// Push it onto the stack.
				stack_symbolType.SetAtGrow(sl, symbolType[i]);
				stack_idents.SetAtGrow(sl, idents[i]);
				++sl;
			}
			// Otherwise, the token is an operator (operator here includes both operators, and functions or arrays).
			else {
				sprintf(res, "%d", opData->nInterm);
				opData->SetOperandAt(opData->nOperands, 0.);
				opData->isConst.SetAtGrow(opData->nOperands, 0);
				opData->numInterm.SetAtGrow(opData->nInterm, opData->nOperands);
				opData->nOperands++;
				opData->checkSum += (opData->nOperands + 2) * (2 + subType[i] + 1) * (subType[i] + 1);
				// It is known a priori that the operator takes n arguments.
				int nargs0;
				if (subType[i] >= NBINARYOP && subType[i] < NBINARYOP + 10) nargs0 = WriteArgCount + 2;
				else {
					nargs0 = op_arg_count(subType[i], iFunc);
					// If there are fewer than n values on the stack
					if(sl < nargs0) {
						// (Error) The user has not input sufficient values in the expression.
						LoadString(resInst, 2405, eqData->message, STRLEN_MAX);
						goto SY_ERROR;
					}
				}
				// Else, Pop the top n values from the stack.
				// Evaluate the operator, with the values as arguments.
				opData->funcNum.SetAtGrow(opData->nInterm, subType[i]);
				int nargs;
				int nDim = 0;
				int correctArgType[3];
				if (subType[i] < NSTDFUNC) {
					if (subType[i] < NSTDFUNC0) {
						nargs = nargs0;
					}
					else if (subType[i] == NSTDFUNC0) { // loc
						nargs = 2;
					}
					else {
						nargs = nargs0;
						if (subType[i] == NSTDFUNC0 + 1 || subType[i] > NSTDFUNC0 + 17 && subType[i] < NSTDFUNC0 + 24) opData->arrayModified = TRUE; // Invert, Hist, Hist2, ReadArray, ReadArray2, Sort, Sort2
						else if (subType[i] >= NSTDFUNC0 + 4 && subType[i] <= NSTDFUNC0 + 14) { // Integration, summation and iteration functions
							if (subType[i] == NSTDFUNC0 + 14) opData->formalParmModified = TRUE; // Root2
							int controlNum = integralControl[curIntegralControl];
							opData->newNextControl[controlNum] = controlNum + 1;
							opData->transferTo[controlNum] = opData->nInterm;
							integralProcessed[curIntegralControl] = TRUE;
							do {
								curIntegralControl--;
							}
							while(curIntegralControl >= 0 && integralProcessed[curIntegralControl]);
						}
					}
				}
				else if (subType[i] < NSTDFUNC + subroutineArgCount) {
					k = subType[i] - NSTDFUNC;
					if (subroutineArgInfo[iFunc].type[k] < MAXARGS + 2) {
						nDim = subroutineArgInfo[iFunc].type[k] - 1;
					//	if (symbolType[i] < 0), this means that it is a reference to an array (with -symbolType[i] - 1 indices), used as an argument of loc()
						if (symbolType[i] < -2) nDim += (symbolType[i] + 2);
						nargs = nargs0 + 1;
					}
				//	else if (subroutineArgInfo[iFunc].type[k] < MAXARGS + 4) nargs = 2; // Because it could be a simple built-in function (all simple built-in functions have two arguments)
					else {
						nargs = nargs0;
						int funcType = subroutineArgInfo[iFunc].type[k] - MAXARGS - 2;
						int jArg;
						for (jArg = 0; funcType > 0; jArg++) {
							BYTE funcType_new = funcType / 5;
							correctArgType[jArg] = funcType - 5 * funcType_new;
							if (correctArgType[jArg] > 0) correctArgType[jArg]--;
							else correctArgType[jArg] = 4; // Should not be possible
							funcType = funcType_new;
						};
					}
					if (!opData->subroutineArgUsed[k]) {
						opData->subroutineArgUsed[k] = TRUE;
						nSubroutineArgUsed++;
					}
				}
				else if (subType[i] < NSTDFUNC + subroutineArgCount + nSubroutines) {
					nargs = nargs0;
					int iSubroutine = subType[i] - NSTDFUNC - subroutineArgCount;
					if (iSubroutine != iFunc && iUnloadableSubroutine < 0 && subroutine[iSubroutine].errorType > 1) {
						iUnloadableSubroutine = iSubroutine;
						unloadableSubroutineLineNum = l;
					}
				}
/*				else if (subType[i] < NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc) {
					nargs = nargs0;
					int iDllFunc = subType[i] - NSTDFUNC - subroutineArgCount - nSubroutines;
					if (iUnloadableDllFunc < 0 && !userDllFuncAddr[iDllFunc]) {
						iUnloadableDllFunc = iDllFunc;
						unloadableDllFuncLineNum = l;
					}
				}*/
				else {
					int iArray = subType[i] - NSTDFUNC - subroutineArgCount - nSubroutines - nDllFunc;
				//	if (symbolType[i] < 0), this means that it is a reference to an array (with -symbolType[i] - 1 indices), used as an argument of loc()
					if (symbolType[i] >= -2) nDim = arrays[iArray].nDim;
					else nDim = arrays[iArray].nDim + symbolType[i] + 2;
					nargs = nargs0 + 1;
				}
				if (nargs > 0) {
					opData->argPtr.SetAtGrow(opData->nInterm, new double*[nargs]);
					opData->argNum.SetAtGrow(opData->nInterm, new long[nargs]);
				}
				else {
					opData->argPtr.SetAtGrow(opData->nInterm, NULL);
					opData->argNum.SetAtGrow(opData->nInterm, NULL);
				}
				opData->argCount.SetAtGrow(opData->nInterm, nargs);
				opData->lineNum.SetAtGrow(opData->nInterm, lineNum[l]);
				opData->isAddress.SetAtGrow(opData->nInterm, 0);
				opData->isArrayAssignment.SetAtGrow(opData->nInterm, FALSE);
				opData->nInterm++;

				int loc_type = 0;

				while(nargs > 0) {
					int iArg = (subType[i] < NSTDFUNC0 || subType[i] >= NSTDFUNC || !reverseArgList[subType[i]]) ? nargs : nargs0 - nargs + 1;
					sc_type = stack_symbolType[sl-1];
					if (sc_type == 2) { // constant
						if (subType[i] == NSTDFUNC0) { // loc()
							LoadString(resInst, 2730, eqData->message, STRLEN_MAX);
							goto SY_ERROR;
						}
						if (subType[i] >= NSTDFUNC0 + 4 && subType[i] <= NSTDFUNC0 + 14 && nargs == nargs0 - 1) { // Integration, summation and iteration functions, argument No. 2 (nargs = nargs0 - 1)
							LoadString(resInst, 2795, S, STRLEN_MAX); sprintf(eqData->message, S, iArg, stdFuncNames[subType[i]]);
							goto SY_ERROR;
						}
						if (subType[i] >= NSTDFUNC0 && subType[i] < NSTDFUNC) {
							if (!incompatibleArgs && nargs <= stdFuncArgInfo[subType[i]].count && stdFuncArgInfo[subType[i]].type[iArg-1]) {
								LoadString(resInst, 2746, S, STRLEN_MAX); sprintf(eqData->message, S, stdFuncNames[subType[i]], iArg);
								incompatibleArgs = TRUE;
							}
						}
						else if (subType[i] >= NSTDFUNC && subType[i] < NSTDFUNC + subroutineArgCount) {
							int k = subType[i] - NSTDFUNC;
							if (!incompatibleArgs && subroutineArgInfo[iFunc].type[k] >= MAXARGS + 2 && correctArgType[iArg-1] != 0) {
								LoadString(resInst, 2905, S, STRLEN_MAX); sprintf(eqData->message, S, subroutineArgNames[k], iArg);
								incompatibleArgs = TRUE;
							}
						}
						else if (subType[i] >= NSTDFUNC + subroutineArgCount && subType[i] < NSTDFUNC + subroutineArgCount + nSubroutines) {
							int iFunc = subType[i] - NSTDFUNC - subroutineArgCount;
							if (!incompatibleArgs && nargs <= subroutineArgInfo[iFunc].count && subroutineArgInfo[iFunc].type[nargs-1]) {
								LoadString(resInst, 2736, S, STRLEN_MAX); sprintf(eqData->message, S, subroutine[iFunc].name, iArg);
								incompatibleArgs = TRUE;
							}
						}
/*						else if (subType[i] >= NSTDFUNC + subroutineArgCount + nSubroutines && subType[i] < NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc) {
							int iDll = subType[i] - NSTDFUNC - subroutineArgCount - nSubroutines;
							if (dllFuncArgCount[iDll] < 0) {
								LoadString(resInst, 2733, S, STRLEN_MAX); sprintf(eqData->message, S, dllFuncName2[iDll], iArg);
								incompatibleArgs = TRUE;
							}
						}*/
						opData->SetOperandAt(opData->nOperands, stack_constants[sl-1]);
						opData->isConst.SetAtGrow(opData->nOperands, 1);
						opData->argPtr[opData->nInterm-1][nargs-1] = opData->operands + opData->nOperands;
						opData->argNum[opData->nInterm-1][nargs-1] = opData->nOperands;
						opData->nOperands++;
					}
					else if (sc_type == 3) { // variable (identifier)
						loc_type = 0;
						if (nIntegrationVarsStored > 0) {
							if (subType[i] >= NSTDFUNC0 + 4 && subType[i] <= NSTDFUNC0 + 14 && nargs == nargs0 - 1) { // Integration, summation and iteration functions, argument No. 2 (nargs = nargs0 - 1)
								opData->argPtr[opData->nInterm-1][nargs-1] = opData->operands + integrationVarNum[nIntegrationVarsStored-1];
								opData->argNum[opData->nInterm-1][nargs-1] = integrationVarNum[nIntegrationVarsStored-1];
								nIntegrationVarsStored--;
								integrationVarNum.SetSize(nIntegrationVarsStored);
								integrationVar_stackPos.SetSize(nIntegrationVarsStored);
								if (nargs <= nargs0 || nDim) sl--;
								--nargs;
								continue;
							}
							else {
								for (j = nIntegrationVarsStored - 1; j >= 0 && strcmpi(stack_idents[integrationVar_stackPos[j]], stack_idents[sl-1]); j--);
								if (j >= 0) {
									opData->argPtr[opData->nInterm-1][nargs-1] = opData->operands + integrationVarNum[j];
									opData->argNum[opData->nInterm-1][nargs-1] = integrationVarNum[j];
									if (nargs <= nargs0 || nDim) sl--;
									--nargs;
									continue;
								}
							}
						}
						int systemVarNum = -1, globalVarNum = -1, localVarNum = -1;
						int mode = -1;
						if (subType[i] == NSTDFUNC0) { // loc
							for (j = NBINARYOP; j < NSTDFUNC && strcmpi(stdFuncNames[j], stack_idents[sl-1]); j++);
							if (j >= NSTDFUNC) {
								for (j = 0; j < nSubroutines && strcmpi(subroutine[j].name, stack_idents[sl-1]); j++);
								if (j >= nSubroutines) {
//									for (j = 0; j < nDllFunc && strcmpi(dllFuncName2[j], stack_idents[sl-1]); j++);
//									if (j >= nDllFunc) {
										for (j = 0; j < nArrays && strcmpi(arrays[j].name, stack_idents[sl-1]); j++);
										if (j < nArrays) {
											mode = 2;
											loc_type = 5 + arrays[j].nDim;
										}
/*									}
									else {
										if (iUnloadableDllFunc < 0 && !userDllFuncAddr[j]) {
											iUnloadableDllFunc = j;
											unloadableDllFuncLineNum = l;
										}
										mode = 5;
										loc_type = 6 + MAXARGS + funcType[j+NSTDFUNC+nSubroutines];
									}*/
								}
								else {
									mode = 4;
									loc_type = 6 + MAXARGS + funcType[j+NSTDFUNC];
								}
							}
							else {
								mode = 3;
								if (j != NSTDFUNC0) loc_type = 6 + MAXARGS + funcType[j];
								else loc_type = 1000;
							}
						}
						if (mode < 0) {
							for (j = 0; j < nVars && strcmpi(vars[j], stack_idents[sl-1]); j++);
							if (j < NSYSTEMVARS) systemVarNum = j;
							else if (j >= NSYSTEMVARS + nArrInd + subroutineArgCount) {
								if (j < NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm) localVarNum = j - NSYSTEMVARS -  nArrInd - subroutineArgCount;
								else if (j < NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm + nGParm) globalVarNum = j - NSYSTEMVARS -  nArrInd - subroutineArgCount - eqData->nParm;
							}
						}
						if (subType[i] == NSTDFUNC0) { // loc
							if (mode < 0 && j >= NSYSTEMVARS + nArrInd && j < NSYSTEMVARS + nArrInd + subroutineArgCount) {
								j = j - NSYSTEMVARS - nArrInd;
								int argType = subroutineArgInfo[iFunc].type[j];
								if (!argType) {
									LoadString(resInst, 2731, S, STRLEN_MAX); sprintf(eqData->message, S, subroutineArgNames[j]);
									goto SY_ERROR;
								}
								mode = 0;
								if (argType < 2) loc_type = 5;
								else loc_type = 4 + argType;
								if (!opData->subroutineArgUsed[j]) {
									opData->subroutineArgUsed[j] = TRUE;
									nSubroutineArgUsed++;
								}
							}
							if (mode >= 0) {
								for (k = 0; k < 2; k++) {
									if (k == 0) opData->SetOperandAt(opData->nOperands, mode);
									else opData->SetOperandAt(opData->nOperands, j);
									opData->isConst.SetAtGrow(opData->nOperands, 1);
									opData->argPtr[opData->nInterm-1][k] = opData->operands + opData->nOperands;
									opData->argNum[opData->nInterm-1][k] = opData->nOperands;
									opData->nOperands++;
									opData->checkSum += (opData->nOperands + mode + 3) * (mode + 3 + j + 1) * (j + 1);
								}
								sl--;
								nargs = 0;
								break;
							}
						}
						if (subType[i] != 5 || nargs > 1) {
							if (j >= nVars) {
								size_t firstPos = strspn(stack_idents[sl-1], " \t");
								char separator = '\0';
								if (strcspn(stack_idents[sl-1], ".?;:{}\\'\"") == firstPos) {
									separator = stack_idents[sl-1][firstPos];
								}
								else {
									char *rev = new char[strlen(stack_idents[sl-1])+1];
									strcpy(rev, stack_idents[sl-1]);
									strrev(rev);
									firstPos = strspn(rev, " \t");
									if (strcspn(rev, ".?;:{}\\'\"") == firstPos) {
										separator = rev[firstPos];
									}
									delete rev;
								}
								if (separator == '\"' || separator == '.' || separator == ';' || separator == '{' || separator == '}') {
									LoadString(resInst, 2906, S, STRLEN_MAX); sprintf(eqData->message, S, separator);
								}
								else if (separator != '\0') {
									LoadString(resInst, 2400, S, STRLEN_MAX); sprintf(eqData->message, S, separator);
								}
								else if (sl > 0) {
									LoadString(resInst, 2402, S, STRLEN_MAX);
									if (strlen(stack_idents[sl-1]) > MAXIDENTLEN) {
										String tmp = (char *)(stack_idents[sl-1]);
										stack_idents[sl-1] = (char *)(tmp.Left(MAXIDENTLEN));
									}
									sprintf(eqData->message, S, stack_idents[sl-1].GetBuffer()); stack_idents[sl-1].ReleaseBuffer();
								}
								else {
									LoadString(resInst, 2407, eqData->message, STRLEN_MAX);
								}
								goto SY_ERROR;
							}
							else if (subType[i] == NSTDFUNC0) { // loc
								if (!loc_type) {
									if (opData->isConst[numVar[j]]) {
										LoadString(resInst, 2731, S, STRLEN_MAX); sprintf(eqData->message, S, vars[numVar[j]].GetBuffer()); vars[numVar[j]].ReleaseBuffer();
										goto SY_ERROR;
									}
									if (j >= NSYSTEMVARS + nArrInd + subroutineArgCount) {
										if (j < NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm) {
											opData->localsUsed = TRUE;
											opData->localParmAddrUsed = TRUE;
										}
										else if (j < NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm + nGParm) {
											opData->globalsUsed = TRUE;
											opData->globalParmAddrUsed = TRUE;
										}
									}
									loc_type = 5;
									opData->SetOperandAt(opData->nOperands, 1);
									opData->isConst.SetAtGrow(opData->nOperands, 1);
									opData->argPtr[opData->nInterm-1][0] = opData->operands + opData->nOperands;
									opData->argNum[opData->nInterm-1][0] = opData->nOperands;
									opData->nOperands++;
									opData->checkSum += (opData->nOperands + 1) * (1 + j + 2) * (j + 2);
									if (systemVarNum < 0 && globalVarNum < 0 && localVarNum < 0) opData->argPtr[opData->nInterm-1][1] = opData->operands + numVar[j];
									else if (systemVarNum >= 0) opData->argPtr[opData->nInterm-1][1] = sysVarValues + systemVarNum;
									else if (globalVarNum >= 0) opData->argPtr[opData->nInterm-1][1] = gParmValues + globalVarNum;
									else opData->argPtr[opData->nInterm-1][1] = opData->parmValues + localVarNum;
									opData->argNum[opData->nInterm-1][1] = numVar[j];
									sl--;
									nargs = 0;
									break;
								}
							}
							else if (subType[i] >= NSTDFUNC0 + 4 && subType[i] <= NSTDFUNC0 + 14 && nargs == nargs0 - 1) { // Integration, summation and iteration functions, argument No. 2 (nargs = nargs0 - 1)
								if (opData->isConst[numVar[j]]) {
									LoadString(resInst, 2794, S, STRLEN_MAX); sprintf(eqData->message, S, iArg, stdFuncNames[subType[i]]);
									goto SY_ERROR;
								}
							}
							else if (subType[i] >= NSTDFUNC0 && subType[i] < NSTDFUNC) {
								if (!incompatibleArgs && nargs <= stdFuncArgInfo[subType[i]].count && stdFuncArgInfo[subType[i]].type[iArg-1]) {
									LoadString(resInst, 2746, S, STRLEN_MAX); sprintf(eqData->message, S, stdFuncNames[subType[i]], iArg);
									incompatibleArgs = TRUE;
								}
							}
							else if (subType[i] >= NSTDFUNC && subType[i] < NSTDFUNC + subroutineArgCount) {
								int k = subType[i] - NSTDFUNC;
								if (!incompatibleArgs && subroutineArgInfo[iFunc].type[k] >= MAXARGS + 2 && correctArgType[iArg-1] != 0) {
									LoadString(resInst, 2905, S, STRLEN_MAX); sprintf(eqData->message, S, subroutineArgNames[k], iArg);
									incompatibleArgs = TRUE;
								}
							}
							else if (subType[i] >= NSTDFUNC + subroutineArgCount && subType[i] < NSTDFUNC + subroutineArgCount + nSubroutines) {
								int iFunc = subType[i] - NSTDFUNC - subroutineArgCount;
								if (!incompatibleArgs && nargs <= subroutineArgInfo[iFunc].count && subroutineArgInfo[iFunc].type[nargs-1]) {
									LoadString(resInst, 2736, S, STRLEN_MAX); sprintf(eqData->message, S, subroutine[iFunc].name, iArg);
									incompatibleArgs = TRUE;
								}
							}
/*							else if (subType[i] >= NSTDFUNC + subroutineArgCount + nSubroutines && subType[i] < NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc) {
								int iDll = subType[i] - NSTDFUNC - subroutineArgCount - nSubroutines;
								if (!incompatibleArgs && dllFuncArgCount[iDll] < 0) {
									if (opData->isConst[numVar[j]]) {
										LoadString(resInst, 2735, S, STRLEN_MAX); sprintf(eqData->message, S, vars[numVar[j]], dllFuncName2[iDll]);
										incompatibleArgs = TRUE;
									}
								}
							}*/
							if (j < NSYSTEMVARS) {
							}
							else if (j < NSYSTEMVARS + nArrInd) {
								opData->arrIndUsed = TRUE;
							}
							else if (j < NSYSTEMVARS + nArrInd + subroutineArgCount) {
								int k = j - NSYSTEMVARS - nArrInd;
								if (!opData->subroutineArgUsed[k]) {
									opData->subroutineArgUsed[k] = TRUE;
									nSubroutineArgUsed++;
								}
							}
							else if (j < NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm) {
								opData->localsUsed = TRUE;
							}
							else if (j < NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm + nGParm) {
								opData->globalsUsed = TRUE;
							}
							else {
								newVarNum = j - nVars0;
								if (warnVar < 0 && !isVisible[newVarNum]) {
									warnVar = j;
									opData->warnLineNum = lineNum[l];
									eqData->errorType = 1;
								}
							}
						}
						else {
							if (j == nVars) {
								if (strpbrk(stack_idents[sl-1], "+-/*^=,().!?<>;:[]{}|\\'%\"")) {
									if (strlen(stack_idents[sl-1]) > MAXIDENTLEN) stack_idents[sl-1] = stack_idents[sl-1].Left(MAXIDENTLEN);
									LoadString(resInst, 2907, S, STRLEN_MAX); sprintf(eqData->message, S, stack_idents[sl-1].GetBuffer()); stack_idents[sl-1].ReleaseBuffer();
									goto SY_ERROR;
								}
								vars.Add(stack_idents[sl-1]);
								numVar.Add(opData->numInterm[opData->nInterm-1]);
								opData->isConst.SetAtGrow(opData->numInterm[opData->nInterm-1], 0); // ???
						//		opData->isConst.SetAtGrow(nVars, 0); // ???
								nVars++;
								if (eqData->gParms) {
									if (nGParm + nNewGParm == MAXGLOBALPARMS) {
										sprintf(eqData->message, "Error: the number of global parameters is greater than %d.\n", MAXGLOBALPARMS);
										goto SY_ERROR;
									}
									newGlobalParms = (PARMDATA *)realloc(newGlobalParms, (nNewGParm + 1) * sizeof(PARMDATA));
									strcpy(newGlobalParms[nNewGParm].name, stack_idents[sl-1]);
									newGlobalParms[nNewGParm].value = 0.;
									nNewGParm++;
								}
								creationDepth.Add(-2);
								isVisible.Add(FALSE);
							}
							if (j >= nVars0) {
								newVarNum = j - nVars0;
								if (warnVar < 0 && !isVisible[newVarNum]) {
									if (creationDepth[newVarNum] == -2) {
										creationDepth.SetAtGrow(newVarNum,currentIFdepth);
										createdInElse.SetAtGrow(newVarNum, currentIF >= 0 && (insideElse[currentIF]||currentIFdepth>=0&&nLinkedIF[currentIFdepth]>0));
										assignmentDepth.SetAtGrow(newVarNum,creationDepth[newVarNum]);
										assignedInElse.SetAtGrow(newVarNum,createdInElse[newVarNum]);
									}
									else if (creationDepth[newVarNum] == currentIFdepth) {
										if (insideElse[currentIF] && !createdInElse[newVarNum]) {
											creationDepth[newVarNum]--;
											assignmentDepth[newVarNum]--;
											if (creationDepth[newVarNum] >= 0) {
												createdInElse[newVarNum] = (insideElse[currentIF-nLinkedIF[currentIFdepth]-1] || nLinkedIF[creationDepth[newVarNum]] > 0);
												assignedInElse[newVarNum] = createdInElse[newVarNum];
											}
										}
									}
									else if (assignmentDepth[newVarNum] < currentIFdepth) {
										assignmentDepth[newVarNum] = currentIFdepth;
										assignedInElse[newVarNum] = insideElse[currentIF];
									}
									else if (insideElse[currentIF] && !assignedInElse[newVarNum]) {
										assignmentDepth[newVarNum]--;
										if (assignmentDepth[newVarNum] >= 0) {
											assignedInElse[newVarNum] = (insideElse[currentIF-nLinkedIF[currentIFdepth]-1] || nLinkedIF[assignmentDepth[newVarNum]] > 0);
										}
									}
									isVisible[newVarNum] = TRUE;
								}
							}
							else if (j >= NSYSTEMVARS + nArrInd && j < NSYSTEMVARS + nArrInd + subroutineArgCount) opData->formalParmModified = TRUE;
							else if (globalVarNum >= 0) opData->globalParmModified = TRUE;
							else if (localVarNum >= 0) opData->localParmModified = TRUE;
						}

						if (systemVarNum < 0 && globalVarNum < 0 && localVarNum < 0) opData->argPtr[opData->nInterm-1][nargs-1] = opData->operands + numVar[j];
						else if (systemVarNum >= 0) opData->argPtr[opData->nInterm-1][nargs-1] = sysVarValues + systemVarNum;
						else if (globalVarNum >= 0) opData->argPtr[opData->nInterm-1][nargs-1] = gParmValues + globalVarNum;
						else opData->argPtr[opData->nInterm-1][nargs-1] = opData->parmValues + localVarNum;
						opData->argNum[opData->nInterm-1][nargs-1] = numVar[j];
					}
					else if (sc_type >= 4) { // intermediate result
						BOOL invertArgNum = FALSE;
						if (subType[i] >= NSTDFUNC0 && subType[i] < NSTDFUNC) {
							if (subType[i] == NSTDFUNC0) { // loc()
								if (!loc_array) {
									LoadString(resInst, 2732, eqData->message, STRLEN_MAX);
									goto SY_ERROR;
								}
							}
							if (subType[i] >= NSTDFUNC0 + 4 && subType[i] <= NSTDFUNC0 + 14 && nargs == nargs0 - 1) { // Integration, summation and iteration functions, argument No. 2 (nargs = nargs0 - 1)
								LoadString(resInst, 2796, S, STRLEN_MAX); sprintf(eqData->message, S, iArg, stdFuncNames[subType[i]]);
								goto SY_ERROR;
							}
							if (!incompatibleArgs && nargs <= stdFuncArgInfo[subType[i]].count && stdFuncArgInfo[subType[i]].type[iArg-1] && stdFuncArgInfo[subType[i]].type[iArg-1] != sc_type - 4) {
								LoadString(resInst, 2746, S, STRLEN_MAX); sprintf(eqData->message, S, stdFuncNames[subType[i]], iArg);
								incompatibleArgs = TRUE;
							}
						}
						else if (subType[i] >= NSTDFUNC && subType[i] < NSTDFUNC + subroutineArgCount) {
							int k = subType[i] - NSTDFUNC;
							if (!incompatibleArgs && subroutineArgInfo[iFunc].type[k] >= MAXARGS + 2 && correctArgType[iArg-1] != 0 && correctArgType[iArg-1] != sc_type - 4) {
								LoadString(resInst, 2905, S, STRLEN_MAX); sprintf(eqData->message, S, subroutineArgNames[k], iArg);
								incompatibleArgs = TRUE;
							}
						}
						else if (subType[i] >= NSTDFUNC + subroutineArgCount && subType[i] < NSTDFUNC + subroutineArgCount + nSubroutines) {
							int iFunc = subType[i] - NSTDFUNC - subroutineArgCount;
							if (!incompatibleArgs && nargs <= subroutineArgInfo[iFunc].count && subroutineArgInfo[iFunc].type[nargs-1] && subroutineArgInfo[iFunc].type[nargs-1] != sc_type - 4) {
								if (subroutineArgInfo[iFunc].type[nargs-1] != MAXARGS + 3 || sc_type != 1000) {
									LoadString(resInst, 2736, S, STRLEN_MAX); sprintf(eqData->message, S, subroutine[iFunc].name, iArg);
								}
								else {
									LoadString(resInst, 2750, eqData->message, STRLEN_MAX);
								}
								incompatibleArgs = TRUE;
							}
						}
/*						else if (subType[i] >= NSTDFUNC + subroutineArgCount && subType[i] < NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc) {
							int iDll = subType[i] - NSTDFUNC - subroutineArgCount - nSubroutines;
							if (dllFuncArgCount[iDll] < 0) {
								if (sc_type == 5) invertArgNum = TRUE;
								else if (!incompatibleArgs) {
									LoadString(resInst, 2733, S, STRLEN_MAX); sprintf(eqData->message, S, dllFuncName2[iDll], iArg);
									incompatibleArgs = TRUE;
								}
							}
						}*/
						int num = atoi(stack_idents[sl-1]);
						if (subType[i] != NSTDFUNC0 || !loc_array) {
							opData->argPtr[opData->nInterm-1][nargs-1] = opData->operands + opData->numInterm[num];
							if (!invertArgNum) opData->argNum[opData->nInterm-1][nargs-1] = opData->numInterm[num];
							else opData->argNum[opData->nInterm-1][nargs-1] = -opData->numInterm[num];
						}
						else {
							opData->SetOperandAt(opData->nOperands, -1);
							opData->isConst.SetAtGrow(opData->nOperands, 1);
							opData->argPtr[opData->nInterm-1][0] = opData->operands + opData->nOperands;
							opData->argNum[opData->nInterm-1][0] = opData->nOperands;
							opData->nOperands++;
							opData->argPtr[opData->nInterm-1][1] = opData->operands + opData->numInterm[num];
							opData->argNum[opData->nInterm-1][1] = opData->numInterm[num];
							sl--;
							nargs = 0;
							loc_type = 5 + loc_array;
							break;
						}
					}
					if (nargs <= nargs0 || nDim) sl--;
					--nargs;
				}

			/*	}

				else {
					opData->argPtr[opData->nInterm-1][0] = opData->operands;
					opData->argNum[opData->nInterm-1][0] = 0;
					opData->argPtr[opData->nInterm-1][1] = opData->operands;
					opData->argNum[opData->nInterm-1][1] = 0;
				}*/

				// Push the returned results, if any, back onto the stack.

				if (subType[i] != NSTDFUNC0 || !loc_type) {
					stack_symbolType.SetAtGrow(sl, 4);
				}
				else { // loc
					stack_symbolType.SetAtGrow(sl, loc_type);
				}
				loc_array = nDim;
				stack_idents.SetAtGrow(sl, res);
			//	++opData->nInterm;
				++sl;
			}
			if (symbolType[i] == 0 && subType[i] == 5) {
				// If there are 2 or more values in the stack
				// (Error) The user input has too many values.
				if (sl > 1) {
					LoadString(resInst, 2415, eqData->message, STRLEN_MAX);
					goto SY_ERROR;
				}
				if (j >= NSYSTEMVARS + nArrInd && j < NSYSTEMVARS + nArrInd + subroutineArgCount) { // subroutineArgInfo
					int argNum = j - NSYSTEMVARS - nArrInd;
					if (subroutineArgInfo[iFunc].type[argNum] != 1) {
						LoadString(resInst, 2416, S, STRLEN_MAX); sprintf(eqData->message, S, vars[j].GetBuffer()); vars[j].ReleaseBuffer();
						goto SY_ERROR;
					}
					else if (!opData->subroutineArgUsed[argNum]) {
						opData->subroutineArgUsed[argNum] = TRUE;
						nSubroutineArgUsed++;
					}
				}
				else if (j < NSYSTEMVARS + nArrInd + subroutineArgCount || j >= NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm + nGParm && j < nVars0/*)*/) {
					LoadString(resInst, 2416, S, STRLEN_MAX); sprintf(eqData->message, S, vars[j].GetBuffer()); vars[j].ReleaseBuffer();
					goto SY_ERROR;
				}
				else if (j < NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm) {
					opData->localsUsed = TRUE;
				}
				else if (j < nVars0) {
					opData->globalsUsed = TRUE;
				}
				// If there is only one value in the stack
				// That value is the result of the calculation.
				sl--;
			}
			i++;
		}
		if (sl > 0) {
			int symbolType_temp = symbolType[i-1];
			if (symbolType_temp > 5000 && symbolType_temp <= 15000) symbolType_temp -= 10000;
			if (symbolType_temp != 0 || subType[i-1] != 5) {
				if (sl > 1) {
					LoadString(resInst, 2415, eqData->message, STRLEN_MAX);
					goto SY_ERROR;
				}
			}
			else {
				LoadString(resInst, 2395, eqData->message, STRLEN_MAX);
				goto SY_ERROR;
			}
		}
		if (incompatibleArgs || attemptToModifyReadOnlyArray) goto SY_ERROR;
		if (insideIFheader) {
			nextPos += (len2 + 1);
			input2 += (len2 + 1);
			if (*input2 != '\0') l--;
			else nextPos = 0;
			justOutOfIFheader = TRUE;
			if (!nLinkedIF[currentIFdepth] || statementFound[currentIFdepth] >= 0) {
				if (lowestWaitingLevel == -2) lowestWaitingLevel = 1;
				else if (lowestWaitingLevel >= currentIFdepth) lowestWaitingLevel++;
				statementFound.SetAtGrow(currentIFdepth, 0);
			}
			else statementFound.SetAtGrow(currentIFdepth, -1);
			headerControlNum.SetAtGrow(currentIF, opData->nControl);
			opData->control.SetAtGrow(opData->nControl, opData->nInterm);
			opData->isConditional.SetAtGrow(opData->nControl, TRUE);
			opData->nControl++;
			justOutOfIF = FALSE;
		}
		else if (currentIF >= 0 && (!leftBracePresent[currentIF] || rightBraceFound) && !arrayAssignmentPos) {
ENDSCOPE:
			if (warnVar < 0) {
				for (j = nVars0; j < nVars; j++) {
					int newVarNum = j - nVars0;
					if (isVisible[newVarNum]) {
						if (assignmentDepth[newVarNum] == currentIFdepth) {
							isVisible[newVarNum] = FALSE;
							if (assignedInElse[newVarNum]) assignmentDepth[newVarNum] = -2;
						}
						if (creationDepth[newVarNum] == currentIFdepth && createdInElse[newVarNum]) {
							creationDepth[newVarNum] = -2;
						}
					}
					else {
						assignmentDepth[newVarNum] = -2;
						if (creationDepth[newVarNum] == currentIFdepth) {
							creationDepth[newVarNum] = -2;
						}
					}
				}
			}
			if (!insideElse[currentIF]) {
				justOutOfIF = TRUE;
				if (!nLinkedIF[currentIFdepth] || statementFound[currentIFdepth] >= 0) {
					if (lowestWaitingLevel >= currentIFdepth) lowestWaitingLevel++;
					if (statementFound[currentIFdepth] > 0) statementFound.SetAtGrow(currentIFdepth, 0);
					else statementFound.SetAtGrow(currentIFdepth, -1);
				}
				else statementFound.SetAtGrow(currentIFdepth, -1);
			}
			else {
				if (lowestWaitingLevel > currentIFdepth) lowestWaitingLevel = currentIFdepth;
				opData->transferTo.SetAtGrow(elseControlNum[currentIF], opData->nInterm);
				opData->newNextControl.SetAtGrow(elseControlNum[currentIF], opData->nControl);
				opData->newNextXbreak.SetAtGrow(elseControlNum[currentIF], opData->nXbreaks);
				insideElse.SetAtGrow(currentIF, FALSE);
				currentIF--;
				while (nLinkedIF[currentIFdepth]) {
					opData->transferTo.SetAtGrow(elseControlNum[currentIF], opData->nInterm);
					opData->newNextControl.SetAtGrow(elseControlNum[currentIF], opData->nControl);
					opData->newNextXbreak.SetAtGrow(elseControlNum[currentIF], opData->nXbreaks);
					insideElse.SetAtGrow(currentIF, FALSE);
					currentIF--;
					nLinkedIF[currentIFdepth]--;
				}
				rightBraceFound = FALSE;
				int statementFoundInPrevious = statementFound[currentIFdepth];
				currentIFdepth--;
				if (currentIFdepth < 0) {
					if (statementFoundInPrevious > 0) lowestWaitingLevel--;
				}
				else if (statementFound[currentIFdepth] <= 0 && currentIFdepth <= lowestWaitingLevel && statementFoundInPrevious > 0) {
					statementFound.SetAtGrow(currentIFdepth, 1);
					lowestWaitingLevel--;
				}
				justOutOfIF = (currentIF >= 0 && !leftBracePresent[currentIF]);
			}
			nextPos = 0;
			currentState = opData->nInterm - 1;
			if (isArrayAssignment) {
				opData->isArrayAssignment.SetAtGrow(currentState, TRUE);
				isArrayAssignment = FALSE;
			}
		}
		else {
			justOutOfIF = FALSE;
			nextPos = 0;
			currentState = opData->nInterm - 1;
			if (currentIF < 0) lowestWaitingLevel = -1;
			if (arrayAssignmentPos > 0) {
				opData->isAddress.SetAtGrow(currentState, 1);
				l--;
			}
			else if (isArrayAssignment) {
				opData->isArrayAssignment.SetAtGrow(currentState, TRUE);
				isArrayAssignment = FALSE;
			}
		}
	}

	if (justOutOfIF) {
		if (cond[currentIF]) { // "while" or "for" loop
			opData->control.SetAtGrow(opData->nControl, opData->nInterm);
			opData->isConditional.SetAtGrow(opData->nControl, FALSE);
			opData->transferTo.SetAtGrow(opData->nControl, nInterm_bak[currentIF]);
			opData->newNextControl.SetAtGrow(opData->nControl, nControl_bak[currentIF]);
			opData->newNextXbreak.SetAtGrow(opData->nControl, nXbreaks_bak[currentIF]);
			opData->nControl++;
		}
		elseControlNum.SetAtGrow(currentIF, opData->nControl);
		opData->control.SetAtGrow(opData->nControl, opData->nInterm);
		opData->isConditional.SetAtGrow(opData->nControl, FALSE);
		opData->nControl++;
		opData->transferTo.SetAtGrow(headerControlNum[currentIF], opData->nInterm);
		opData->newNextControl.SetAtGrow(headerControlNum[currentIF], opData->nControl);
		opData->newNextXbreak.SetAtGrow(headerControlNum[currentIF], opData->nXbreaks);
		insideElse.SetAtGrow(currentIF, TRUE);
		l--;
		goto ENDSCOPE;
	}

	if (l == nl) {
		opData->control.SetAtGrow(opData->nControl, opData->nInterm);
		opData->isConditional.SetAtGrow(opData->nControl, FALSE);
		opData->nControl++;
	}

	if (currentIFdepth >= 0) {
		LoadString(resInst, 2406, eqData->message, STRLEN_MAX);
		l = -1;
		goto SY_ERROR;
	}

	if (!nOut) {
		if (iFunc < 0) eqData->message[0] = '\0';
		else LoadString(resInst, 2671, eqData->message, STRLEN_MAX);
		l = -1;
		goto SY_ERROR;
	}

	opData->nXbreaks++;

	if (!ignoreUnloadableSubroutines && iUnloadableSubroutine >= 0) {
		LoadString(resInst, 2650, S, STRLEN_MAX); sprintf(eqData->message, S, subroutine[iUnloadableSubroutine].name);
		l = unloadableSubroutineLineNum;
		goto SY_ERROR;
	}

//	if (iUnloadableDllFunc >= 0) {
//		LoadString(resInst, 2418, S, STRLEN_MAX); sprintf(eqData->message, S, dllFuncName2[iUnloadableDllFunc]);
//		l = unloadableDllFuncLineNum;
//		goto SY_ERROR;
//	}

	for (i = 0; i < nl; i++) delete input1[i];
	input1.RemoveAll();
	delete input2_0;

	if (opData->nOperands > 0) {
		opData->operands = (double *)realloc(opData->operands, 2 * opData->nOperands * sizeof(double));
		for (i = opData->nOperands; i < 2 * opData->nOperands; i++) opData->operands[i] = 0.;
	}

	for (i = 0; i < opData->nInterm; i++) {
		if (opData->argCount[i] > 0) {
			for (j = 0; j < opData->argCount[i]; j++) {
				long argNum = abs(opData->argNum[i][j]);
				if (argNum >= NSYSTEMVARS && argNum < NSYSTEMVARS + nArrInd + subroutineArgCount ||
					argNum >= NSYSTEMVARS + nArrInd + subroutineArgCount + eqData->nParm + nGParm) {
					opData->argPtr[i][j] = opData->operands + argNum;
				}
			}
		}
	}

	opData->globalObjectsUsed = opData->globalsUsed || opData->arraysUsed || opData->subroutinesUsed;
	opData->userGlobalObjectsUsed = opData->globalsUsed || opData->userArraysUsed || opData->subroutinesUsed;

	if (nSubroutineArgUsed < subroutineArgCount) {
		for (i = 0; i < subroutineArgCount && opData->subroutineArgUsed[i]; i++);
		LoadString(resInst, 2670, S, STRLEN_MAX); sprintf(eqData->message, S, (char *)(vars[NSYSTEMVARS+i]));
		opData->warnLineNum = nl0 - 1;
		eqData->errorType = 1;
	}
	else if (lowestWaitingLevel != -1) {
		LoadString(resInst, 2419, eqData->message, STRLEN_MAX);
		opData->warnLineNum = nl0 - 1;
		eqData->errorType = 1;
	}
	else if (warnVar >= 0) {
		LoadString(resInst, 2420, S, STRLEN_MAX); sprintf(eqData->message, S, (char *)(vars[warnVar]));
	}
	else {
		eqData->message[0] = '\0';
	}
	for (j = 0; j < subroutineArgCount; j++) delete subroutineArgNames[j];
	delete tmp;
	delete input0;
	delete input;
	return -1;

SY_ERROR:

	eqData->errorType = 2;

	for (i = 0; i < nl; i++) delete input1[i];
	input1.RemoveAll();
	if (input2_0) delete input2_0;

	for (j = 0; j < subroutineArgCount; j++) delete subroutineArgNames[j];

	opData->globalObjectsUsed = opData->globalsUsed || opData->arraysUsed || opData->subroutinesUsed;
	opData->userGlobalObjectsUsed = opData->globalsUsed || opData->userArraysUsed || opData->subroutinesUsed;

	opData->errLineNum = l >= 0 ? lineNum[l] : l;
	opData->warnLineNum = -1;

	delete tmp;
	delete input0;
	delete input;
	return opData->errLineNum;
}

BOOL OnComma(int &sl, int &nOut)
{
	bool pe = false;
	int sc_type = 0, sc_subType = 0;
	double sc_number = 0.;
			int size_symbolType = stack_symbolType.GetSize();
			int size_subType = stack_subType.GetSize();
			int size_constants = stack_constants.GetSize();
	while(sl > 0) {
		if (sl <= size_symbolType) sc_type = stack_symbolType[sl-1];
		if (sl <= size_subType) sc_subType = stack_subType[sl-1];
		if (sl <= size_constants) sc_number = stack_constants[sl-1];
		if(sc_type == 0 && sc_subType == 20)  {
			pe = true;
			break;
		}
		else  {
			// Until the token at the top of the stack is a left parenthesis,
			// pop operators off the stack onto the output queue.
			symbolType.SetAtGrow(nOut, sc_type + addType); addType = 0;
			subType.SetAtGrow(nOut, sc_subType);
			constants.SetAtGrow(nOut, sc_number);
			nOut++;
			sl--;
		}
	}
	return (pe != 0);
}

int FindSign(char *string, int j)
{
	String tmp = string;
	int j1 = tmp.FindOutsideTheQuotes('-', j);
	int j2 = tmp.FindOutsideTheQuotes('+', j);
	if (j1 < 0) j = j2;
	else if (j2 < 0) j = j1;
	else j = j1 < j2 ? j1 : j2;
	return j;
}

char *is_number(char *c, double &num, BOOL insertNull) {
	char *endPtr;
	size_t nBlanks = strspn(c, " \t");
	if (strlen(c) == nBlanks) return (char *)NULL;
	c += nBlanks;
	num = strtod(c, &endPtr);
	endPtr += strspn(endPtr, " \t");
	bool isNumber = (endPtr != c && (*endPtr == '\0' || strchr("+-/*^=<>&|,()[]!", *endPtr)));
	if (isNumber) {
		if (insertNull) *endPtr = '\0';
		return endPtr;
	}
	else return (char *)NULL;
}
 
int op_preced(const int c)
{
	switch(c)    {
		case 4: // '^'
			return 7;
		case 2: case 3: // '/', '*'
			return 6;
		case 0: case 1: // '+', '-'
			return 5;
		case 6: case 7: case 8: case 9: case 10: case 11: // '>', '<', '>=', '<=', '==', ('<>','!=')
			return 4;
		case 12: // '&'
			return 3;
		case 13: // '|'
			return 2;
		case 5: // '='
			return 1;
	}
	return 0;
}
 
bool op_left_assoc(const int c)
{
	switch(c)    {
		// left to right
		case 0: case 1: case 2: case 3: case 4: case 6: case 7: case 8: case 9: case 10: case 11: case 12: case 13:
			return true;
		// right to left
		case 5: // '='
			return false;
	}
	return false;
}
 
unsigned int Compiler::op_arg_count(const int c, int iFunc)
{
	int subroutineArgCount = 0;
	if (iFunc >= 0) subroutineArgCount = subroutineArgInfo[iFunc].count;
	if (c < NSTDFUNC0) {
		switch(c)  {
			case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 8: case 9: case 10: case 11: case 12: case 13: case 38: case 44: case 47: case 49: case 50: case 66: case 67: case 68:
				return 2;
			case 24: case 25: case 56: case 58: case 59:
				return 0;
			case 61: case 69:
				return 3;
			default:
				return 1;
		}
	}
	else if (c < NSTDFUNC) {
		return stdFuncArgInfo[c].count;
	}
	else if (c < NSTDFUNC + subroutineArgCount) {
		int argType = subroutineArgInfo[iFunc].type[c-NSTDFUNC];
		if (argType < MAXARGS + 2) return argType - 1;
		else {
			argType -= (MAXARGS + 2);
			if (!argType) return 0;
			else if (argType < 6) return 1;
			else if (argType < 31) return 2;
			else return 3;
		}
	}
	else if (c < NSTDFUNC + subroutineArgCount + nSubroutines) {
		return subroutineArgInfo[c-NSTDFUNC-subroutineArgCount].count;
	}
//	else if (c < NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc) {
//		return abs(dllFuncArgCount[c-NSTDFUNC-subroutineArgCount-nSubroutines]);
//	}
	else {
		return arrays[c-NSTDFUNC-subroutineArgCount-nSubroutines-nDllFunc].nDim;
	}
	return 0;
}
 
bool Compiler::is_function(char *c, int &id, int iFunc, char **argNames) {
	size_t nBlanks = strspn(c, " \t");
	if (strlen(c) == nBlanks) return false;
	char tmp[MAXFUNCNAMELEN+1], tmp2[MAXFUNCNAMELEN+1];
	strncpy(tmp, c + nBlanks, MAXFUNCNAMELEN);
	tmp[MAXFUNCNAMELEN] = '\0';
	strrev(tmp);
	strcpy(tmp2, tmp + strspn(tmp, " \t"));
	strrev(tmp2);
	id = -1;
	int i, subroutineArgCount = 0;
	for (i = NBINARYOP; i < NSTDFUNC; i++) {
		if (!strcmpi(tmp2, stdFuncNames[i])) {
			id = i;
			break;
		}
	}
	if (id < 0 && iFunc >= 0) {
		subroutineArgCount = subroutineArgInfo[iFunc].count;
		for (i = 0; i < subroutineArgCount; i++) {
			if (subroutineArgInfo[iFunc].type[i] > 1 && !strcmpi(tmp2, argNames[i])) {
				id = NSTDFUNC + i;
				break;
			}
		}
	}
	if (id < 0) {
		for (i = 0; i < nSubroutines; i++) {
			if (!strcmpi(tmp2, subroutine[i].name)) {
				id = NSTDFUNC + subroutineArgCount + i;
				break;
			}
		}
	}
/*	if (id < 0) {
		for (i = 0; i < nDllFunc; i++) {
			if (!strcmpi(tmp2, dllFuncName2[i])) {
				id = NSTDFUNC + subroutineArgCount + nSubroutines + i;
				break;
			}
		}
	}*/
	if (id < 0) {
		for (i = 0; i < nArrays; i++) {
			if (!strcmpi(tmp2, arrays[i].name)) {
				id = NSTDFUNC + subroutineArgCount + nSubroutines + nDllFunc + i;
				break;
			}
		}
	}
	return (id >= 0);
}
 
bool is_operator(char c, int &opId, char *&pos) {
	if (c == '+') opId = 0;
	else if (c == '-') opId = 1;
	else if (c == '/') opId = 2;
	else if (c == '*') opId = 3;
	else if (c == '^') opId = 4;
	else if (c == '=') {
		if (pos && *(pos + 1) == '=') opId = 10;
		else opId = 5;
	}
	else if (c == '>') {
		if (pos && *(pos + 1) == '=') opId = 8;
		else opId = 6;
	}
	else if (c == '<') {
		if (pos && *(pos + 1) == '=') opId = 9;
		else if (pos && *(pos + 1) == '>') opId = 11;
		else opId = 7;
	}
	else if (c == '!' && pos && *(pos + 1) == '=') opId = 11;
	else if (c == '&') opId = 12;
	/*else if (c == '&') {
		if (pos && *(pos + 1) == '&') opId = 12;
		else opId = -1;
	}*/
	else if (c == '|') opId = 13;
	/*else if (c == '|') {
		if (pos && *(pos + 1) == '|') opId = 13;
		else opId = -1;
		}
	}*/
	else opId = -1;
	if (opId >= 0) {
		if (pos) {
		//	if (opId >= 8 && opId <= 13) pos += 2;
			if (opId >= 8 && opId <= 11) pos += 2;
			else pos++;
		}
		return true;
	}
	else return false;
}
 
bool is_ident(char *c, char *ident) {
	size_t nBlanks = strspn(c, " \t");
	if (strlen(c) == nBlanks) return false;
	String tmp = c + nBlanks;
	tmp.TrimRight();
	strcpy(ident, tmp);
	char *endPtr;
	double num = strtod(ident, &endPtr);
	bool rc = (endPtr == ident);
	return rc;
}
 
BYTE Compiler::FuncType(int id) {
	BYTE type = 0, base;
	int i, argCount;
	if (id < NSTDFUNC) {
		argCount = op_arg_count(id);
		if (id < NSTDFUNC0) {
			for (base = 1, i = 0; i < argCount; i++, base *= 5) type += base;
		}
		else {
			if (argCount < 4) {
				for (base = 1, i = 0; i < argCount; i++, base *= 5) {
					BYTE argType = stdFuncArgInfo[id].type[i];
					if (argType < 4) type += (argType + 1) * base;
					else type += 5 * base;
				}
			}
			else type = 255;
		}
	}
	else {
		if (id < NSTDFUNC + nSubroutines) {
			int iFunc = id - NSTDFUNC;
			argCount = subroutineArgInfo[iFunc].count;
			if (argCount < 4) {
				for (base = 1, i = 0; i < argCount; i++, base *= 5) {
					BYTE argType = subroutineArgInfo[iFunc].type[i];
					if (argType < 4) type += (argType + 1) * base;
					else type += 5 * base;
				}
			}
			else type = 255;
		}
/*		else if (id < NSTDFUNC + nSubroutines + nDllFunc) {
			int iDll = id - NSTDFUNC - nSubroutines;
			argCount = dllFuncArgCount[iDll];
			if (argCount >= 0 && argCount < 4) {
				for (base = 1, i = 0; i < argCount; i++, base *= 5) type += base;
			}
			else if (argCount < 0 && argCount > -4) {
				argCount = -argCount;
				for (base = 1, i = 0; i < argCount; i++, base *= 5) type += 2 * base;
			}
			else type = 255;
		}*/
		else type = 255;
	}
	return type;
}

OPDATA::OPDATA()
{
	operands = NULL;
	curSize = 0;
	chunkSize = 100;
	waitTime = 2.;
}

void OPDATA::SetOperandAt(int i, double value)
{
	if (i >= curSize) {
		curSize = (i / chunkSize + 1 ) * chunkSize;
		operands = (double *)realloc(operands, curSize * sizeof(double));
	}
	operands[i] = value;
}

void OPDATA::Reset0()
{
	errorType = 0;
	errorFunc = -2;
	arrayError = 0;
	builtInFuncError = 0;
	runtimeErrDll = 0;
	callDepth = 0;
	curOp = -1;
	interruptOp = -1;
	runtimeErrLineNum = -1;
}

void OPDATA::Reset(int subroutineArgCount, int nLocParms)
{
	Reset0();
	int i;
	for (i = 0; i < nInterm; i++) {
		if (argCount[i] > 0) {
			delete argPtr[i];
			delete argNum[i];
		}
	}
	int nArrInd = 0;
	base = NSYSTEMVARS + nArrInd + subroutineArgCount + nLocParms + app->nGParm;
	nOperands = base;
	subroutineUsed.SetSize(app->nSubroutines);
	for (i = 0; i < app->nSubroutines; i++) subroutineUsed[i] = FALSE;
	arrayUsed.SetSize(app->nArrays);
	arrayMod.SetSize(app->nArrays);
	for (i = 0; i < app->nArrays; i++) {
		arrayUsed[i] = FALSE;
		arrayMod[i] = FALSE;
	}
	isConst.SetSize(nOperands);
	for (i = 0; i < NSYSTEMVARS + nArrInd + subroutineArgCount; i++) isConst[i] = 1;
	for (; i < NSYSTEMVARS + nArrInd + subroutineArgCount + nLocParms + app->nGParm; i++) isConst[i] = 0;
	for (; i < nOperands; i++) isConst[i] = 1;
	checkSum = 0;
	isConditional.RemoveAll();
	isAddress.RemoveAll();
	isArrayAssignment.RemoveAll();
	numInterm.RemoveAll();
	funcNum.RemoveAll();
	argCount.RemoveAll();
	lineNum.RemoveAll();
	control.RemoveAll();
	transferTo.RemoveAll();
	newNextControl.RemoveAll();
	newNextXbreak.RemoveAll();
	argNum.RemoveAll();
	argPtr.RemoveAll();
	if (operands) free(operands);
	curSize = (nOperands / chunkSize + 1) * chunkSize;
	operands = (double *)malloc(curSize * sizeof(double));
	nInterm = 0;
	arrIndUsed = FALSE;
	subroutinesUsed = FALSE;
	arraysUsed = FALSE;
	userArraysUsed = FALSE;
	arrayModified = FALSE;
	formalParmModified = FALSE;
	globalParmModified = FALSE;
	localParmModified = FALSE;
	globalParmAddrUsed = FALSE;
	localParmAddrUsed = FALSE;
	dllsUsed = FALSE;
	globalsUsed = FALSE;
	globalObjectsUsed = FALSE;
	userGlobalObjectsUsed = FALSE;
	randTimeUsed = FALSE;
	localsUsed = FALSE;
	nXbreaks = 0;
	nControl = 0;
	errLineNum = -1;
	warnLineNum = -1;
	for (i = 0; i < MAXARGS; i++) subroutineArgUsed[i] = FALSE;
}

void OPDATA::operator=(class OPDATA const &src)
{
    if (this != &src) {
		 nOperands = src.nOperands;
		 nInterm = src.nInterm;
		 arrIndUsed = src.arrIndUsed;
		 arrayModified = src.arrayModified;
		 formalParmModified = src.formalParmModified;
		 globalParmModified = src.globalParmModified;
		 localParmModified = src.localParmModified;
		 globalParmAddrUsed = src.globalParmAddrUsed;
		 localParmAddrUsed = src.localParmAddrUsed;
		 valuesUpdated = src.valuesUpdated;
		 waitTime = src.waitTime;
		 dllsUsed = src.dllsUsed;
		 globalsUsed = src.globalsUsed;
		 localsUsed = src.localsUsed;
		 subroutinesUsed = src.subroutinesUsed;
		 arraysUsed = src.arraysUsed;
		 userArraysUsed = src.userArraysUsed;
		 globalObjectsUsed = src.globalObjectsUsed;
		 userGlobalObjectsUsed = src.userGlobalObjectsUsed;
		 randTimeUsed = src.randTimeUsed;
		 int i;
		 for (i = 0; i < MAXARGS; i++) subroutineArgUsed[i] = src.subroutineArgUsed[i];
		 nXbreaks = src.nXbreaks;
		 i_tNear = src.i_tNear;
		 errorType = src.errorType;
		 errLineNum = src.errLineNum;
		 runtimeErrDll = src.runtimeErrDll;
		 runtimeErrCode = src.runtimeErrCode;
		 arrayError = src.arrayError;
		 errorFunc = src.errorFunc;
		 builtInFuncError = src.builtInFuncError;
		 runtimeErrLineNum = src.runtimeErrLineNum;
		 warnLineNum = src.warnLineNum;
		 curOp = src.curOp;
		 interruptOp = src.interruptOp;
		 nControl = src.nControl;
		 parmValues = src.parmValues;
		 chunkSize = src.chunkSize;
		 curSize = src.curSize;
		 base = src.base;
		 checkSum = src.checkSum;
		 operands = src.operands;
		 subroutineUsed.Copy(src.subroutineUsed);
		 arrayUsed.Copy(src.arrayUsed);
		 arrayMod.Copy(src.arrayMod);
		 isConst.Copy(src.isConst);
		 isConditional.Copy(src.isConditional);
		 isAddress.Copy(src.isAddress);
		 isArrayAssignment.Copy(src.isArrayAssignment);
		 numInterm.Copy(src.numInterm);
		 funcNum.Copy(src.funcNum);
		 argCount.Copy(src.argCount);
		 lineNum.Copy(src.lineNum);
		 control.Copy(src.control);
		 transferTo.Copy(src.transferTo);
		 newNextControl.Copy(src.newNextControl);
		 newNextXbreak.Copy(src.newNextXbreak);
		 argNum.Copy(src.argNum);
		 argPtr.Copy(src.argPtr);
	 }
}

Compiler::~Compiler()
{
	int i, j;
	for (i = 0; i < nSubroutines; i++) {
		OPDATA *op = subroutine2 + i;
		op->subroutineUsed.RemoveAll();
		op->arrayUsed.RemoveAll();
		op->arrayMod.RemoveAll();
		op->isConst.RemoveAll();
		op->isConditional.RemoveAll();
		op->isAddress.RemoveAll();
		op->isArrayAssignment.RemoveAll();
		op->numInterm.RemoveAll();
		op->funcNum.RemoveAll();
		op->lineNum.RemoveAll();
		op->control.RemoveAll();
		op->transferTo.RemoveAll();
		op->newNextControl.RemoveAll();
		op->newNextXbreak.RemoveAll();
		if (op->operands) {
			free(op->operands);
			op->operands = NULL;
		}
		for (j = 0; j < op->nInterm; j++) {
			if (op->argCount[j] > 0) {
				delete op->argPtr[j];
				delete op->argNum[j];
			}
		}
		op->argCount.RemoveAll();
		op->argNum.RemoveAll();
		op->argPtr.RemoveAll();
		outputLines[i].RemoveAll();
		outputLineNumbers[i].RemoveAll();
		free(subroutineArgInfo[i].list.data); subroutineArgInfo[i].list.data = NULL;
		free(subroutineArgInfo[i].list_singleLine.data); subroutineArgInfo[i].list_singleLine.data = NULL;
		free(fpos[i]);
	}
	for (i = 0; i < nArrays0; i++) {
		if (arrays[i].elements) delete arrays[i].elements;
	}
	funcType.RemoveAll();
	subroutineCode.RemoveAll();
	formatStrings.RemoveAll();
	delete aPtr1;
	free(subroutine);
	free(subroutine2);
	free(subroutineArgInfo);
	free(outputLines);
	free(outputLineNumbers);
	free(arrays);
	free(fpos);
	free(startPos);
	free(endPos);
	symbolType.RemoveAll();
	subType.RemoveAll();
	constants.RemoveAll();
	stack_symbolType.RemoveAll();
	stack_subType.RemoveAll();
	stack_constants.RemoveAll();
}

Array::Array()
{
	nElem = 0;
	size = 0;
	data0 = NULL;
}
Array::~Array()
{
	RemoveAll0();
}
void Array::Add0(void *source)
{
	unsigned long size0 = size;
	nElem++;
	size += elSize;
	data0 = data0 ? realloc(data0, size) : malloc(size);
	memcpy((char *)data0 + size0, source, elSize);
}
void Array::InsertAt0(int index, void *source)
{
	int remSize;
	char *tmp;
	if (nElem > index) {
		int nRemaining = nElem - index;
		remSize = nRemaining * elSize;
		tmp = new char[remSize];
		memcpy(tmp, (char *)data0 + index * elSize, remSize);
	}
	else remSize = 0;
	nElem++;
	size = nElem * elSize;
	data0 = data0 ? realloc(data0, size) : malloc(size);
	memcpy((char *)data0 + index * elSize, source, elSize);
	if (remSize) {
		memcpy((char *)data0 + (index + 1) * elSize, tmp, remSize);
		delete tmp;
	}
}
void Array::SetAtGrow0(int index, void *source)
{
	if (index < nElem) memcpy((char *)data0 + index * elSize, source, elSize);
	else {
		int oldSize = size;
		nElem = index + 1;
		size = nElem * elSize;
		data0 = data0 ? realloc(data0, size) : malloc(size);;
		if (size > oldSize + elSize) memset((char *)data0 + oldSize, 0, size - oldSize - elSize);
		memcpy((char *)data0 + size - elSize, source, elSize);
	}
}
void Array::RemoveAt0(int index)
{
	nElem--;
	size = nElem * elSize;
	if (index < nElem) {
		int nRemaining = nElem - index;
		int remSize = nRemaining * elSize;
		char *tmp = new char[remSize];
		memcpy(tmp, (char *)data0 + (index + 1) * elSize, remSize);
		data0 = realloc(data0, size);
		memcpy((char *)data0 + index * elSize, tmp, remSize);
		delete tmp;
	}
	else data0 = realloc(data0, size);
}
int Array::GetSize()
{
	return nElem;
}
void Array::SetSize0(int nElem_new)
{
	int size_old = size;
	nElem = nElem_new;
	size = nElem * elSize;
	data0 = data0 ? realloc(data0, size) : malloc(size);
	if (size > size_old) memset((char *)data0 + size_old, 0, size - size_old);
}
void Array::RemoveAll0()
{
	if (data0) free(data0);
	nElem = 0;
	size = 0;
	data0 = NULL;
}
void Array::Copy0(const Array& src)
{
	if (data0) {
		free(data0);
		data0 = NULL;
	}
	nElem = src.nElem;
	elSize = src.elSize;
	size = src.size;
	if (!size) return;
	data0 = malloc(size);
	memcpy(data0, src.data0, size);
}


Array_int::Array_int()
{
	elSize = sizeof(int);
	data = (int *)data0;
}
void Array_int::Add(int value)
{
	Add0(&value);
	data = (int *)data0;
}
void Array_int::SetAt(int index, int value)
{
	data[index] = value;
}
void Array_int::SetAtGrow(int index, int value)
{
	SetAtGrow0(index, &value);
	data = (int *)data0;
}
void Array_int::InsertAt(int index, int value)
{
	InsertAt0(index, &value);
	data = (int *)data0;
}
void Array_int::RemoveAt(int index)
{
	RemoveAt0(index);
	data = (int *)data0;
}
void Array_int::RemoveAll()
{
	RemoveAll0();
	data = (int *)data0;
}
void Array_int::SetSize(int nElem_new)
{
	SetSize0(nElem_new);
	data = (int *)data0;
}
void Array_int::Copy(const Array_int& src)
{
	Copy0(src);
	data = (int *)data0;
}
int &Array_int::operator[](int index)
{
	return data[index];
}


Array_long::Array_long()
{
	elSize = sizeof(long);
	data = (long *)data0;
}
void Array_long::Add(long value)
{
	Add0(&value);
	data = (long *)data0;
}
void Array_long::SetAt(int index, long value)
{
	data[index] = value;
}
void Array_long::SetAtGrow(int index, long value)
{
	SetAtGrow0(index, &value);
	data = (long *)data0;
}
void Array_long::InsertAt(int index, long value)
{
	InsertAt0(index, &value);
	data = (long *)data0;
}
void Array_long::RemoveAt(int index)
{
	RemoveAt0(index);
	data = (long *)data0;
}
void Array_long::RemoveAll()
{
	RemoveAll0();
	data = (long *)data0;
}
void Array_long::SetSize(int nElem_new)
{
	SetSize0(nElem_new);
	data = (long *)data0;
}
void Array_long::Copy(const Array_long& src)
{
	Copy0(src);
	data = (long *)data0;
}
long &Array_long::operator[](int index)
{
	return data[index];
}


Array_BYTE::Array_BYTE()
{
	elSize = sizeof(BYTE);
	data = (BYTE *)data0;
}
void Array_BYTE::Add(BYTE value)
{
	Add0(&value);
	data = (BYTE *)data0;
}
void Array_BYTE::SetAt(int index, BYTE value)
{
	data[index] = value;
}
void Array_BYTE::SetAtGrow(int index, BYTE value)
{
	SetAtGrow0(index, &value);
	data = (BYTE *)data0;
}
void Array_BYTE::InsertAt(int index, BYTE value)
{
	InsertAt0(index, &value);
	data = (BYTE *)data0;
}
void Array_BYTE::RemoveAt(int index)
{
	RemoveAt0(index);
	data = (BYTE *)data0;
}
void Array_BYTE::RemoveAll()
{
	RemoveAll0();
	data = (BYTE *)data0;
}
void Array_BYTE::SetSize(int nElem_new)
{
	SetSize0(nElem_new);
	data = (BYTE *)data0;
}
void Array_BYTE::Copy(const Array_BYTE& src)
{
	Copy0(src);
	data = (BYTE *)data0;
}
BYTE &Array_BYTE::operator[](int index)
{
	return data[index];
}


Array_double::Array_double()
{
	elSize = sizeof(double);
	data = (double *)data0;
}
void Array_double::Add(double value)
{
	Add0(&value);
	data = (double *)data0;
}
void Array_double::SetAt(int index, double value)
{
	data[index] = value;
}
void Array_double::SetAtGrow(int index, double value)
{
	SetAtGrow0(index, &value);
	data = (double *)data0;
}
void Array_double::InsertAt(int index, double value)
{
	InsertAt0(index, &value);
	data = (double *)data0;
}
void Array_double::RemoveAt(int index)
{
	RemoveAt0(index);
	data = (double *)data0;
}
void Array_double::RemoveAll()
{
	RemoveAll0();
	data = (double *)data0;
}
void Array_double::SetSize(int nElem_new)
{
	SetSize0(nElem_new);
	data = (double *)data0;
}
void Array_double::Copy(const Array_double& src)
{
	Copy0(src);
	data = (double *)data0;
}
double &Array_double::operator[](int index)
{
	return data[index];
}


Array_longPtr::Array_longPtr()
{
	elSize = sizeof(long *);
	data = (long **)data0;
}
void Array_longPtr::Add(long *value)
{
	Add0(&value);
	data = (long **)data0;
}
void Array_longPtr::SetAt(int index, long *value)
{
	data[index] = value;
}
void Array_longPtr::SetAtGrow(int index, long *value)
{
	SetAtGrow0(index, &value);
	data = (long **)data0;
}
void Array_longPtr::InsertAt(int index, long *value)
{
	InsertAt0(index, &value);
	data = (long **)data0;
}
void Array_longPtr::RemoveAt(int index)
{
	RemoveAt0(index);
	data = (long **)data0;
}
void Array_longPtr::RemoveAll()
{
	RemoveAll0();
	data = (long **)data0;
}
void Array_longPtr::SetSize(int nElem_new)
{
	SetSize0(nElem_new);
	data = (long **)data0;
}
void Array_longPtr::Copy(const Array_longPtr& src)
{
	Copy0(src);
	data = (long **)data0;
}
long* &Array_longPtr::operator[](int index)
{
	return data[index];
}


Array_charPtr::Array_charPtr()
{
	elSize = sizeof(char *);
	data = (char **)data0;
}
void Array_charPtr::Add(char *value)
{
	Add0(&value);
	data = (char **)data0;
}
void Array_charPtr::SetAt(int index, char *value)
{
	data[index] = value;
}
void Array_charPtr::SetAtGrow(int index, char *value)
{
	SetAtGrow0(index, &value);
	data = (char **)data0;
}
void Array_charPtr::InsertAt(int index, char *value)
{
	InsertAt0(index, &value);
	data = (char **)data0;
}
void Array_charPtr::RemoveAt(int index)
{
	RemoveAt0(index);
	data = (char **)data0;
}
void Array_charPtr::RemoveAll()
{
	RemoveAll0();
	data = (char **)data0;
}
void Array_charPtr::SetSize(int nElem_new)
{
	SetSize0(nElem_new);
	data = (char **)data0;
}
void Array_charPtr::Copy(const Array_charPtr& src)
{
	Copy0(src);
	data = (char **)data0;
}
char* &Array_charPtr::operator[](int index)
{
	return data[index];
}


Array_dblPtr2::Array_dblPtr2()
{
	elSize = sizeof(double **);
	data = (double ***)data0;
}
void Array_dblPtr2::Add(double **value)
{
	Add0(&value);
	data = (double ***)data0;
}
void Array_dblPtr2::SetAt(int index, double **value)
{
	data[index] = value;
}
void Array_dblPtr2::SetAtGrow(int index, double **value)
{
	SetAtGrow0(index, &value);
	data = (double ***)data0;
}
void Array_dblPtr2::InsertAt(int index, double **value)
{
	InsertAt0(index, &value);
	data = (double ***)data0;
}
void Array_dblPtr2::RemoveAt(int index)
{
	RemoveAt0(index);
	data = (double ***)data0;
}
void Array_dblPtr2::RemoveAll()
{
	RemoveAll0();
	data = (double ***)data0;
}
void Array_dblPtr2::SetSize(int nElem_new)
{
	SetSize0(nElem_new);
	data = (double ***)data0;
}
void Array_dblPtr2::Copy(const Array_dblPtr2& src)
{
	Copy0(src);
	data = (double ***)data0;
}
double** &Array_dblPtr2::operator[](int index)
{
	return data[index];
}


Array_String::Array_String()
{
	elSize = sizeof(String);
	data = (String *)data0;
}
Array_String::~Array_String()
{
	RemoveAll();
}
void Array_String::Add(char *value0)
{
	String value = value0;
	Add0(&value);
	data = (String *)data0;
	data[nElem-1].data = (char *)malloc(strlen(value0) + 1);
	strcpy(data[nElem-1].data, value0); // "data[nElem-1] = value0" would call free(data[nElem-1].data), which is the same as free(value.data), and the destructor of value would fail.
}
void Array_String::SetAt(int index, char *value0)
{
	data[index] = value0;
}
void Array_String::SetAtGrow(int index, char *value0)
{
	if (index < nElem) free(data[index].data);
	int nElem_old = nElem;
	String value = value0;
	SetAtGrow0(index, &value);
	data = (String *)data0;
	data[index].data = (char *)malloc(strlen(value0) + 1);
	strcpy(data[index].data, value0); // "data[index] = value0" would call free(data[index].data), which is the same as free(value.data), and the destructor of value would fail.
	for (int i = nElem_old; i < index; i++) data[i] = "";
}
void Array_String::InsertAt(int index, char *value0)
{
	String value = value0;
	InsertAt0(index, &value);
	data = (String *)data0;
	data[index].data = (char *)malloc(strlen(value0) + 1);
	strcpy(data[index].data, value0); // "data[index] = value0" would call free(data[index].data), which is the same as free(value.data), and the destructor of value would fail.
}
void Array_String::RemoveAt(int index)
{
	free(data[index].data);
	RemoveAt0(index);
	data = (String *)data0;
}
void Array_String::RemoveAll()
{
	for (int i = 0; i < nElem; i++) free(data[i].data);
	RemoveAll0();
	data = (String *)data0;
}
void Array_String::SetSize(int nElem_new)
{
	int i, nElem_old = nElem;
	for (i = nElem_new; i < nElem_old; i++) free(data[i].data);
	SetSize0(nElem_new);
	data = (String *)data0;
	for (i = nElem_old; i < nElem_new; i++) data[i] = "";
}
void Array_String::Copy(const Array_String& src)
{
	Copy0(src); // does not allocate memory for char* data (copies char* pointers instead)
	data = (String *)data0;
	int i;
	for (i = 0; i < nElem; i++) {
		char *str = data[i].data;
		data[i].data = (char *)malloc(data[i].size + 1);
		strcpy(data[i].data, str); // "data[i] = str" would call free(data[i].data), which is the same as free(src[i].data), and the source array would be deallocated.
	}
}
String &Array_String::operator[](int index)
{
	return data[index];
}


String::String()
{
	size = 0;
	data = (char *)malloc(1);
	data[0] = '\0';
}
String::String(char *str)
{
	size = (int)strlen(str);
	data = (char *)malloc(size + 1);
	strcpy(data, str);
}
String::~String()
{
	if (data) free(data);
}
int String::GetLength()
{
	return size;
}
String String::Left(int n)
{
	String out;
	out.size = n < size ? n : size;
	out.data = (char *)realloc(out.data, out.size + 1);
	memcpy(out.data, data, out.size);
	out.data[out.size] = '\0';
	return out;
}
void String::MakeLower()
{
	if (data) strlwr(data);
}
void String::TrimRight()
{
	size_t nBlanks = data ? strspn(data, " \t") : 0;
	if (nBlanks == size) {
		size = 0;
		data = data ? (char *)realloc(data, 1) : (char *)malloc(1);
		data[0] = '\0';
	}
	else {
		char *ptr_end, *next_word = data + nBlanks;
		do {
			ptr_end = next_word + strcspn(next_word, " \t");
			next_word = ptr_end + strspn(ptr_end, " \t");
		} while(*next_word != '\0');
		size_t size_new = ptr_end - data;
		if (size_new < (size_t)size) {
			size = (int)size_new;
			data = (char *)realloc(data, size + 1);
			data[size] = '\0';
		}
	}
}
void String::Insert(int index, char *str)
{
	char *tmp = new char[size-index+1];
	strcpy(tmp, data + index);
	int len = (int)strlen(str);
	size += len;
	data = data ? (char *)realloc(data, size + 1) : (char *)malloc(size + 1);
	strcpy(data + index, str);
	strcpy(data + index + len, tmp);
	delete tmp;
}
int String::Find(char c)
{
	if (c == '\0') return (int)size;
	char *ptr = strchr(data, c);
	return (ptr ? (int)(ptr - data) : -1);
}
int String::Find(char c, int j)
{
	if (c == '\0') return (int)size;
	char *ptr = strchr(data + j + 1, c);
	return (ptr ? (int)(ptr - data) : -1);
}
int String::FindOutsideTheQuotes(char c, int j)
{
	if (c == '\0') return (int)size;
	char *ptr2 = data + j + 1;
	ptr2 = strchr(ptr2, c);
	if (!ptr2) return -1;
	if (*ptr2 == '\"') {
		ptr2 = strchr(ptr2 + 1, '\"');
		if (!ptr2) return -1;
		ptr2++;
	}
	bool insideQuotes;
	char *start = data;
	do {
		ptr2 = strchr(ptr2, c);
		if (!ptr2) return -1;
		char *ptr_quotes = strchr(start, '\"');
		insideQuotes = false;
		while(ptr_quotes && ptr_quotes < ptr2) {
			insideQuotes = !insideQuotes;
			ptr_quotes = strchr(ptr_quotes + 1, '\"');
		}
		if (insideQuotes) {
			if (ptr_quotes) {
				ptr2 = ptr_quotes + 1;
				start = ptr2;
			}
			else ptr2 = NULL;
		}
	} while(insideQuotes && ptr2);
	if (!ptr2) return -1;
	return (int)(ptr2 - data);
}
int String::Find(char *str)
{
	if (str[0] == '\0') return (int)size;
	char *ptr = strstr(data, str);
	return (ptr ? (int)(ptr - data) : -1);
}
int String::Find(char *str, int j)
{
	if (str[0] == '\0') return (int)size;
	char *ptr = strstr(data + j + 1, str);
	return (ptr ? (int)(ptr - data) : -1);
}
int String::FindOutsideTheQuotes(char *str, int j)
{
	if (str[0] == '\0') return (int)size;
	char *ptr2 = data + j + 1; // it is assumed that data[j] is not '\"' and it is not inside quotes
	ptr2 = strstr(ptr2, str);
	if (!ptr2) return -1;
	if (*ptr2 == '\"') {
		ptr2 = strchr(ptr2 + 1, '\"');
		if (!ptr2) return -1;
		ptr2++;
	}
	bool insideQuotes;
	char *start = data;
	do {
		ptr2 = strstr(ptr2, str);
		if (!ptr2) return -1;
		char *ptr_quotes = strchr(start, '\"');
		insideQuotes = false;
		while(ptr_quotes && ptr_quotes < ptr2) {
			insideQuotes = !insideQuotes;
			ptr_quotes = strchr(ptr_quotes + 1, '\"');
		}
		if (insideQuotes) {
			if (ptr_quotes) {
				ptr2 = ptr_quotes + 1;
				start = ptr2;
			}
			else ptr2 = NULL;
		}
	} while(insideQuotes && ptr2);
	if (!ptr2) return -1;
	return (int)(ptr2 - data);
}
char *String::GetBuffer()
{
	return data;
}
void String::ReleaseBuffer()
{
}
void String::operator =(char *str)
{
	size = (int)strlen(str);
	if (data) free(data);
	data = (char *)malloc(size + 1);
	strcpy(data, str);
}
void String::operator +=(char *str)
{
	int len = (int)strlen(str);
	int oldSize = size;
	size += len;
	data = data ? (char *)realloc(data, size + 1) : (char *)malloc(size + 1);
	strcpy(data + oldSize, str);
}
String::operator char*() const
{
	return data;
}

void Compiler::LoadString(char **strings, int ID, char *buffer, int maxLength)
{
	int i;
	char *endPtr;
	for (i = 0; i < NMSGSTRINGS && strtol(strings[i], &endPtr, 10) != ID; i++);
	if (i < NMSGSTRINGS) strncpy(buffer, endPtr + strspn(endPtr, " \t"), maxLength);
	else buffer[0] = '\0';
}

void Compiler::ArrayRuntimeErrorMsg(long errorFunc, int iFunc, long arrayError, long runtimeErrCode, char *msg)
{
	if (errorFunc == iFunc || errorFunc < 0) {
		LoadString(resInst, 2656, S1, STRLEN_MAX);
		sprintf(msg, S1, arrays[arrayError-1].name);
	}
	else {
		LoadString(resInst, 2657, S1, STRLEN_MAX);
		sprintf(msg, S1, subroutine[errorFunc].name, arrays[arrayError-1].name);
	}
	S2[0] = '\0';
	if (runtimeErrCode == 1) LoadString(resInst, 2717, S2, STRLEN_MAX);
	else if (runtimeErrCode == 2) LoadString(resInst, 2718, S2, STRLEN_MAX);
	else if (runtimeErrCode == 3) LoadString(resInst, 2741, S2, STRLEN_MAX);
	strcat(msg, " ");
	strcat(msg, S2);
}

void Compiler::BuiltInFuncRuntimeErrorMsg(long errorFunc, int iFunc, long builtInFuncError, long runtimeErrCode, char *msg)
{
	if (errorFunc == iFunc || errorFunc < 0) {
		LoadString(resInst, 2742, S1, STRLEN_MAX);
		sprintf(msg, S1, stdFuncNames[builtInFuncError-1]);
	}
	else {
		LoadString(resInst, 2743, S1, STRLEN_MAX);
		sprintf(msg, S1, subroutine[errorFunc].name, stdFuncNames[builtInFuncError-1]);
	}
	S2[0] = '\0';
	if (builtInFuncError == NSTDFUNC0 + 2) { // Invert (function No. NSTDFUNC0 + 1)
		if (runtimeErrCode == 1) LoadString(resInst, 2744, S2, STRLEN_MAX);
		else if (runtimeErrCode == 2) LoadString(resInst, 2745, S2, STRLEN_MAX);
		else if (runtimeErrCode == 3) LoadString(resInst, 2814, S2, STRLEN_MAX);
	}
	else if (builtInFuncError == NSTDFUNC0 + 4) { // Size2 (function No. NSTDFUNC0 + 3)
		if (runtimeErrCode == 1) LoadString(resInst, 2747, S2, STRLEN_MAX);
	}
	else if (builtInFuncError >= NSTDFUNC0 + 5 && builtInFuncError <= NSTDFUNC0 + 10) { // Integration (functions No. NSTDFUNC0 + 4 to NSTDFUNC0 + 9)
		if (runtimeErrCode == 1) LoadString(resInst, 2802, S2, STRLEN_MAX);
		else if (runtimeErrCode == 2) LoadString(resInst, 2803, S2, STRLEN_MAX);
		else if (runtimeErrCode == 3) LoadString(resInst, 2804, S2, STRLEN_MAX);
		else if (runtimeErrCode == 4) LoadString(resInst, 2805, S2, STRLEN_MAX);
		else if (runtimeErrCode == 5) LoadString(resInst, 2806, S2, STRLEN_MAX);
		else if (runtimeErrCode == 6) LoadString(resInst, 2807, S2, STRLEN_MAX);
		else if (runtimeErrCode == 7) LoadString(resInst, 2808, S2, STRLEN_MAX);
		else if (runtimeErrCode == 8 && builtInFuncError >= 9) LoadString(resInst, 2809, S2, STRLEN_MAX);
		else if (runtimeErrCode == 9) LoadString(resInst, 2814, S2, STRLEN_MAX);
		else if (runtimeErrCode == 10) LoadString(resInst, 2818, S2, STRLEN_MAX);
	}
	else if (builtInFuncError == NSTDFUNC0 + 12) { // Sum2 (function No. NSTDFUNC0 + 11)
		if (runtimeErrCode == 1) LoadString(resInst, 2798, S2, STRLEN_MAX);
	}
	else if (builtInFuncError == NSTDFUNC0 + 14 || builtInFuncError == NSTDFUNC0 + 15) { // Root, Root2 (functions No. NSTDFUNC0 + 13 and No. NSTDFUNC0 + 14)
		if (runtimeErrCode == 1) LoadString(resInst, 2881, S2, STRLEN_MAX);
		else if (runtimeErrCode == 2) LoadString(resInst, 2882, S2, STRLEN_MAX);
		else if (runtimeErrCode == 3) LoadString(resInst, 2883, S2, STRLEN_MAX);
		else if (runtimeErrCode == 4) LoadString(resInst, 2884, S2, STRLEN_MAX);
	}
	else if (builtInFuncError == NSTDFUNC0 + 17 || builtInFuncError == NSTDFUNC0 + 18) { // Find, Find2 (functions No. NSTDFUNC0 + 16 and No. NSTDFUNC0 + 17)
		if (runtimeErrCode == 1) LoadString(app->resInst, 2908, app->S2, STRLEN_MAX);
	}
	else if (builtInFuncError == NSTDFUNC0 + 19 || builtInFuncError == NSTDFUNC0 + 20) { // Hist, Hist2 (functions No. NSTDFUNC0 + 18 and No. NSTDFUNC0 + 19)
		if (runtimeErrCode < 3) {
			char tmp[STRLEN_MAX+1]; LoadString(resInst, 2892, tmp, STRLEN_MAX); sprintf(S2, tmp, runtimeErrCode);
		}
		else if (runtimeErrCode == 3) strcpy(S2, "(elements of array No. 3 are not 64-bit floating point numbers or 32-bit integer numbers).");
		else if (runtimeErrCode == 4) LoadString(resInst, 2893, S2, STRLEN_MAX);
	}
	else if (builtInFuncError == NSTDFUNC0 + 21 || builtInFuncError == NSTDFUNC0 + 22) { // ReadArray, ReadArray2 (functions No. NSTDFUNC0 + 20 and No. NSTDFUNC0 + 21)
		if (runtimeErrCode == 1) strcpy(S2, "(there is no file open for reading).");
	}
	else if (builtInFuncError == NSTDFUNC0 + 24) { // Sort2 (function No. NSTDFUNC0 + 23)
		if (runtimeErrCode == 1) strcpy(S2, "(the data type of array No. 2 must be 4-byte signed integer).");
		else if (runtimeErrCode == 2) strcpy(S2, "(the number of elements in array No. 2 is less than the number of elements in array No. 1).");
	}
	strcat(msg, " ");
	strcat(msg, S2);
}
/*
void Compiler::DLLruntimeErrorMsg(long errorFunc, int iFunc, long runtimeErrDll, long runtimeErrCode, char *msg)
{
	if (runtimeErrCode != dllFuncArgCount[runtimeErrDll-1]) {
		if (errorFunc == iFunc || errorFunc < 0) {
			LoadString(resInst, 2421, S, STRLEN_MAX); sprintf(msg, S, dllFuncName2[runtimeErrDll-1], runtimeErrCode);
		}
		else {
			LoadString(resInst, 2658, S, STRLEN_MAX); sprintf(msg, S, subroutine[errorFunc].name, dllFuncName2[runtimeErrDll-1], runtimeErrCode);
		}
	}
	else if (errorFunc == iFunc || errorFunc < 0) {
		LoadString(resInst, 2903, S, STRLEN_MAX); sprintf(msg, S, dllFuncName2[runtimeErrDll-1]);
	}
	else {
		LoadString(resInst, 2904, S, STRLEN_MAX); sprintf(msg, S, subroutine[errorFunc].name, dllFuncName2[runtimeErrDll-1]);
	}
}
*/

void Compiler::UpdateErrorMsg(int iFunc, EQDATA *eqData, OPDATA *opData)
{
	if (eqData->errorType == 2) { // fatal syntax error
		eqData->errLineNum = opData->errLineNum; // syntax error line number
	}
	else if (opData->errorType || opData->interruptOp >= 0 || opData->curOp >= 0) { // fatal runtime error (including user interrupt), or syntax warning (i.e., non-fatal syntax error), or no error of any kind
		eqData->errorType = 3;
		if (opData->errorType) eqData->errLineNum = opData->runtimeErrLineNum;
		else if (opData->interruptOp >= 0) eqData->errLineNum  = opData->lineNum[opData->interruptOp];
		else eqData->errLineNum  = opData->lineNum[opData->curOp];
		if (opData->arrayError) ArrayRuntimeErrorMsg(opData->errorFunc, iFunc, opData->arrayError, opData->runtimeErrCode, eqData->message);
		else if (opData->builtInFuncError) BuiltInFuncRuntimeErrorMsg(opData->errorFunc, iFunc, opData->builtInFuncError, opData->runtimeErrCode, eqData->message);
//		else if (opData->runtimeErrDll) DLLruntimeErrorMsg(opData->errorFunc, iFunc, opData->runtimeErrDll, opData->runtimeErrCode, eqData->message);
		else if (opData->errorType == 4) {
			LoadString(resInst, 2404, S, 499); sprintf(eqData->message, S, MAXNESTINGLEVEL);
		}
		else if (opData->errorType == 5) LoadString(resInst, 2766, eqData->message, 499);
		else if (opData->errorType == 6) {
			LoadString(resInst, 2902, S, 499); sprintf(eqData->message, S, MAXINTDEPTH);
		}
		else if (opData->interruptOp >= 0 || opData->curOp >= 0) LoadString(resInst, 2672, eqData->message, STRLEN_MAX);
	}
	else if (opData->warnLineNum >= 0) { // syntax warning (i.e., non-fatal syntax error)
		eqData->errorType = 1;
		eqData->errLineNum = opData->warnLineNum;
	}
	else {
		eqData->errorType = 0;
		eqData->errLineNum = -1;
	}
}

void Compiler::PostProcessing(FILE *f_in, int iSubroutine)
{
	EQDATA *eq;
	OPDATA *op;
	if (iSubroutine < 0) {
		eq = aPtr1->eq; 
		op = aPtr1->op; 
	}
	else {
		eq = subroutine + iSubroutine; 
		op = subroutine2 + iSubroutine; 
	}
	UpdateErrorMsg(currentProgram, eq, op);
	if (iSubroutine < 0 && !waiting || eq->errorType == 3) { // If there are any syntax warnings (errorType = 1), they are displayed immediately after compilation of the program
		if (eq->errorType || eq->message[0] != '\0') printf("\n******** %s ********\n", eq->message);
		else if (eq->errLineNum >= 0) printf("\n");
		if (eq->errLineNum >= 0) {
			char *errHeaderFmt[4] = {"******** Line No. %d of %s \"%s\", where the error was detected:\n", "Line No. %d of %s \"%s\" with the syntax warning:\n", "Line No. %d of %s \"%s\" with the syntax error:\n", "Line No. %d of %s \"%s\", where the runtime error occurred:\n"};
			if (eq->name[0] != '\0') printf(errHeaderFmt[eq->errorType], eq->errLineNum + 1, "user program", eq->name);
			else printf(errHeaderFmt[eq->errorType], eq->errLineNum + 1, "user code block", "PARAMETERS");
			char line[1001], *ptr;
			fpos_t curPos;
			fgetpos(f_in, &curPos);
			fsetpos(f_in, fpos[currentProgram] + eq->errLineNum);
			fgets(line, 1000, f_in);
			fsetpos(f_in, &curPos);
			line[1000] = '\0';
			ptr = strtok(line, "\n");
			printf("%s\n", ptr);
		}
		if (eq->errorType || eq->message[0] != '\0' || eq->errLineNum >= 0) printf("\n");
	}
	fatalError = (eq->errorType > 1);
	if (iSubroutine < 0 && waiting) {
		currentProgram = -1;
		inputLineCreated = false;
		newLine = true;
	}
}
